@@ -2,24 +2,22 @@
  *
  * c h a r . c				-- Characters management
  *
- * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??????
- * Last file update:  4-May-1996 11:18
+ * Last file update:  3-Sep-1999 20:18 (eg)
  */
 
 #include <ctype.h>
@@ -27,62 +25,62 @@
 
 struct charelem {
   char *name;
-  char value;
+  unsigned char value;
 };
 
 static struct charelem chartable [] = { 
-  "null",       '\000',
-  "bell",       '\007',
-  "backspace",  '\010',
-  "newline",    '\012',
-  "page",       '\014',
-  "return",     '\015',
-  "escape",     '\033',
-  "space",      '\040',
-  "delete",     '\177',
+  {"null",       '\000'},
+  {"bell",       '\007'},
+  {"backspace",  '\010'},
+  {"newline",    '\012'},
+  {"page",       '\014'},
+  {"return",     '\015'},
+  {"escape",     '\033'},
+  {"space",      '\040'},
+  {"delete",     '\177'},
 
   /* poeticless names */
-  "nul",        '\000',
-  "soh",        '\001',
-  "stx",        '\002',
-  "etx",        '\003',
-  "eot",        '\004',
-  "enq",        '\005',
-  "ack",        '\006',
-  "bel",        '\007',
+  {"nul",        '\000'},
+  {"soh",        '\001'},
+  {"stx",        '\002'},
+  {"etx",        '\003'},
+  {"eot",        '\004'},
+  {"enq",        '\005'},
+  {"ack",        '\006'},
+  {"bel",        '\007'},
 
-  "bs",         '\010',
-  "ht",         '\011',
-  "tab",        '\011',
-  "nl",         '\012',
-  "vt",         '\013',
-  "np",         '\014',
-  "cr",         '\015',
-  "so",         '\016',
-  "si",         '\017',
+  {"bs",         '\010'},
+  {"ht",         '\011'},  /*and also */ {"tab", '\011'},
+  {"nl",         '\012'},
+  {"vt",         '\013'},
+  {"np",         '\014'},
+  {"cr",         '\015'},
+  {"so",         '\016'},
+  {"si",         '\017'},
 
-  "dle",        '\020',
-  "dc1",        '\021',
-  "dc2",        '\022',
-  "dc3",        '\023',
-  "dc4",        '\024',
-  "nak",        '\025',
-  "syn",        '\026',
-  "etb",        '\027',
+  {"dle",        '\020'},
+  {"dc1",        '\021'},
+  {"dc2",        '\022'},
+  {"dc3",        '\023'},
+  {"dc4",        '\024'},
+  {"nak",        '\025'},
+  {"syn",        '\026'},
+  {"etb",        '\027'},
 
-  "can",        '\030',
-  "em",         '\031',
-  "sub",        '\032',
-  "esc",        '\033',
-  "fs",         '\034',
-  "gs",         '\035',
-  "rs",         '\036',
-  "us",         '\037',
+  {"can",        '\030'},
+  {"em",         '\031'},
+  {"sub",        '\032'},
+  {"esc",        '\033'},
+  {"fs",         '\034'},
+  {"gs",         '\035'},
+  {"rs",         '\036'},
+  {"us",         '\037'},
 
-  "sp",		'\040',					  
-  "del",	'\177',
+  {"sp",	'\040'},
+  {"del",	'\177'},
   
-  "",           '\000'};
+  {"",           '\000'}
+};
 
 
 static int my_strcmpi(register char *p1, register char *p2)
@@ -95,11 +93,10 @@ static int my_strcmpi(register char *p1, register char *p2)
 
 
 
-char STk_string2char(char *s)
+unsigned char STk_string2char(char *s)
 /* converts a char name to a char */
 {
   register struct charelem *p;
-  int diff;
   
   if (s[1] == '\000') return s[0];
   for (p=chartable; *(p->name); p++) {
@@ -109,27 +106,27 @@ char STk_string2char(char *s)
   return '\0'; /* never reached */
 }
 
-char *STk_char2string(char c)  /* convert a char to it's external representation */
-{
-  static char result[2] = " ";  /* sets the \0 */
+unsigned char *STk_char2string(unsigned char c)  /* convert a char to it's */
+{						 /* external representation */
+  static unsigned char result[2] = " ";  /* sets the \0 */
   register struct charelem *p;
 
   for (p=chartable; *(p->name); p++)
-    if (p->value == c) return p->name;
+    if (p->value == c) return (unsigned char *) p->name;
   
   /* If we are here it's a "normal" char */
   *result = c;
   return result;
 }
 
-SCM STk_makechar(char c)
+SCM STk_makechar(unsigned char c)
 {
   SCM z;
 
 #ifndef COMPACT_SMALL_CST
   NEWCELL(z,tc_char);
 #endif
-  SET_CHARACTER(z, c);
+  SET_CHARACTER(z, (int) c);
   return z;
 }
 
@@ -210,18 +207,17 @@ PRIMITIVE STk_integer2char(SCM i)
   int c = STk_integer_value(i);
 
   if (c < 0 || c > MAX_CHAR_CODE) Err("integer->char: bad integer", i);
-  return STk_makechar(c);
+  return STk_makechar((unsigned char) c);
 }
 
 PRIMITIVE STk_char_upper(SCM c)
 {
   if (NCHARP(c)) Err("char-upcase: bad character", c);
-  return STk_makechar(toupper(CHAR(c)));
+  return STk_makechar((unsigned char) toupper(CHAR(c)));
 }
 
 PRIMITIVE STk_char_lower(SCM c)
 {
   if (NCHARP(c)) Err("char-downcase: bad character", c);
-  return STk_makechar(tolower(CHAR(c)));
+  return STk_makechar((unsigned char) tolower(CHAR(c)));
 }
-
