@@ -3,34 +3,35 @@
  * e x t e n d . c			-- All the stuff dealing with 
  *					   extended types
  *
- * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
- *
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 15-Mar-1995 11:31
- * Last file update: 10-Jun-1996 00:02
+ * Last file update:  3-Sep-1999 20:19 (eg)
  */
 
 #include "stk.h"
+#include "stklos.h"
 #include "extend.h"
 
 #define EXT_TYPE_DESCR(x)	(xtypes[TYPE(x)- tc_start_extd])
 
-static STk_extended_scheme_type *xtypes[tc_stop_extd-tc_start_extd+1];
 static int extended_type_stamp = tc_start_extd;
+static STk_extended_scheme_type *xtypes[tc_stop_extd-tc_start_extd+1];
 
+#ifdef USE_STKLOS
+static SCM xclasses[tc_stop_extd-tc_start_extd+1];
+#endif
 
 /******************************************************************************
  * 
@@ -48,7 +49,7 @@ static void internal_display(SCM obj, SCM port, int mode)
 {
   sprintf(STk_tkbuffer, "#<%s %lx>", (EXT_TYPE_DESCR(obj))->type_name, 
 	  			     (unsigned long) obj);
-  Puts(STk_tkbuffer, PORT_FILE(port));
+  Puts(STk_tkbuffer, port);
 }
 
 static SCM internal_apply(SCM obj, SCM args, SCM env)
@@ -106,6 +107,29 @@ SCM STk_extended_compare(SCM x, SCM y, int equalp)
 		      : (*(EXT_TYPE_DESCR(y)->compare_fct))(x, y, equalp);
 }
 
+char *STk_get_extended_name(int i)
+{ 
+  return ((i>=tc_start_extd) && (i<extended_type_stamp)) ? 
+    		(xtypes[i - tc_start_extd])->type_name : 
+  		NULL;
+}
+
+#ifdef USE_STKLOS
+void STk_register_extended_class(SCM c, int type_index)
+{
+  if (NNULLP(c)) {
+    int i = type_index - tc_start_extd;
+    xclasses[i] = c;
+    STk_gc_protect(xclasses+i);
+  }
+}
+
+SCM STk_extended_class_of(SCM obj)   /* Warning: obj MUST be an extd type */
+{ 
+  return  xclasses[TYPE(obj)-tc_start_extd];
+}
+#endif
+
 /******************************************************************************
  * 
  * C-pointer
@@ -114,21 +138,27 @@ SCM STk_extended_compare(SCM x, SCM y, int equalp)
 
 typedef void (*STk_disp_function)(SCM x, SCM port, int mode);
 
-static int Cpointer_id 		        = 0;
+static int Cpointer_id 		        = ANONYMOUS_DYN_PTR_ID + 1;
 static int size       		        = 0;
 static STk_disp_function *display_array = NULL;
 
 
 static void Cpointer_default_display(SCM obj, SCM port, int mode)
 {
-  sprintf(STk_tkbuffer, "#<C-pointer %d %lx>", EXTID(obj), EXTDATA(obj));
-  Puts(STk_tkbuffer, PORT_FILE(port));
+  sprintf(STk_tkbuffer, "#<C-pointer %d %lx>", EXTID(obj), 
+	  				       (unsigned long) EXTDATA(obj));
+  Puts(STk_tkbuffer, port);
 }
 
 
 void STk_Cpointer_display(SCM obj, SCM port, int mode)
 {
-  (*(display_array[EXTID(obj)]))(obj, port, mode);
+  int id = EXTID(obj);
+
+  if (id <= ANONYMOUS_DYN_PTR_ID) 
+    Cpointer_default_display(obj, port, mode);
+  else
+    (*(display_array[id]))(obj, port, mode);
 }
 
 
@@ -150,14 +180,14 @@ struct get_n_set_box {
 SCM STk_apply_getter_C_variable(char *var)
 {
   Tcl_HashEntry *entry;
-  
-  if (entry = Tcl_FindHashEntry(&Cvars, var)) {
+
+  if ((entry = Tcl_FindHashEntry(&Cvars, var))) {
     struct get_n_set_box *p = (struct get_n_set_box *) Tcl_GetHashValue(entry);
     
     return (*(p->getter))(var);
   }
   else {
-    fprintf(STk_stderr, "internal error: %s variable has no getter!!\n", var);
+    Fprintf(STk_curr_eport, "internal error: %s variable has no getter!!\n", var);
     return UNDEFINED;    
   }
 }
@@ -165,14 +195,14 @@ SCM STk_apply_getter_C_variable(char *var)
 void STk_apply_setter_C_variable(char *var, SCM value)
 {  
   Tcl_HashEntry *entry;
-  
-  if (entry = Tcl_FindHashEntry(&Cvars, var)) {
+
+  if ((entry = Tcl_FindHashEntry(&Cvars, var))) {
     struct get_n_set_box *p = (struct get_n_set_box *) Tcl_GetHashValue(entry);
     
     (*(p->setter))(var, value);
   }
   else
-    fprintf(STk_stderr, "internal error: %s variable has no setter!!\n", var);
+    Fprintf(STk_curr_eport, "internal error: %s variable has no setter!!\n", var);
 }
 
   
@@ -198,6 +228,11 @@ int STk_add_new_type(STk_extended_scheme_type *p)
   /* Store the new type descriptor in the xtypes array */
   xtypes[extended_type_stamp - tc_start_extd] = p;
 
+#ifdef USE_STKLOS
+  /* Create a new class for this type and register it */
+  STk_register_extended_class(STk_make_extended_class(p->type_name),
+			      extended_type_stamp);
+#endif
   return extended_type_stamp++;
 }
 
@@ -208,7 +243,7 @@ void STk_add_new_primitive(char *fct_name, int fct_type, struct obj * (*fct_ptr)
   NEWCELL(z, fct_type);
   z->storage_as.subr0.name = fct_name;
   z->storage_as.subr0.f    = fct_ptr;
-  VCELL(Intern(fct_name))  = z;
+  STk_define_public_var(STk_selected_module, Intern(fct_name), z);
 }
 
 SCM STk_eval_C_string(char *s, SCM env)
@@ -267,7 +302,7 @@ void STk_define_C_variable(char *var, SCM (*getter)(), void (*setter)())
   p->setter = setter;
   entry     = Tcl_CreateHashEntry(&Cvars, var, &new);
   if (!new) {
-    fprintf(STk_stderr, "Attempt to multi-define C variable `%s' !!\n", var);
+    Fprintf(STk_curr_eport, "Attempt to multi-define C variable `%s' !!\n", var);
     return;
   }
   Tcl_SetHashValue(entry, p);
