@@ -17,35 +17,52 @@
  *
  *****
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 12-May-1993 10:34
- * Last file update: 17-Jul-1995 21:41
+ * Last file update:  3-Sep-1999 20:21 (eg)
  */
 
 #include "stk.h"
 
+
 #define SMALLNUMP(x)	(fabs(x) <= SMALLINT_MAX)
 #define ISINT(x)	(floor(x) == (x))
 
+/*  
+ * This interface has been added for the Frank Schmuelling complex-number
+ * package (Frank email is * <frank@zeus.ph1.Uni-Koeln.DE>). Don't rely on
+ * it, since rationnal and complex numbers will be present in a future
+ * release. As soon as the full tower will be implemented in STk, 
+ * this door will be closed.
+ */
+   
+#define do_addition 	_STk_do_addition
+#define do_substract 	_STk_do_substract
+#define do_multiply 	_STk_do_multiply
+#define do_divide 	_STk_do_divide
+#define bignum2double 	_STk_bignum2double
 
+
+void _STk_do_multiply(SCM *x, SCM y);
+void _STk_do_addition(SCM *x, SCM y);
 static double do_compare(SCM x, SCM y);
 
 
+
+
 static SCM makesmallint(long x)
 {
   SCM z;
@@ -73,7 +90,7 @@ static SCM makebignum(char *s)
  *
  ******************************************************************************/
 
-static int digitp(char c, int base)
+static int digitp(char c, long base)
 {
   c = ('0' <= c && c <= '9') ? c - '0':
       ('a' <= c && c <= 'f') ? c - 'a' + 10:
@@ -92,6 +109,15 @@ static SCM int2bignum(long n)
   return z;
 }
 
+static SCM uint2bignum(unsigned long n)
+{
+  SCM z;
+  NEWCELL(z, tc_bignum);
+  BIGNUM(z) = must_malloc(sizeof(MP_INT));
+  mpz_init_set_ui(BIGNUM(z), n);
+  return z;
+}
+
 static double bignum2double(MP_INT *bn)
 {
   char   *s= mpz_get_str(NULL, 10, bn);
@@ -101,14 +127,20 @@ static double bignum2double(MP_INT *bn)
   return d;
 }
 
-static SCM double2bignum(double n)	/* Very inefficient */
+static SCM double2integer(double n)	/* small or big depending of n's size */
 {
-  int i, j, size = 20;
+  int i, j;
+  size_t size = 20;
   char *tmp = NULL;
   SCM z;
 
   if (!ISINT(n)) Err("cannot convert this number to an integer", STk_makenumber(n));
   
+  /* Try first to convert n to a long */
+  if (((double) SMALLINT_MIN <= n) && (n <= (double) SMALLINT_MAX))
+    return STk_makeinteger((long) n);
+
+  /* n doesn't fit in a long => build a bignum. THIS IS VERY INEFFICIENT */  
   tmp = must_malloc(size);
   i = 0;
   if (n < 0.0) { tmp[i++] = '-'; n = -n; }
@@ -152,31 +184,25 @@ char *STk_number2Cstr(SCM n, long base, char buffer[])
 	   long tmp, val = INTEGER(n);
 	   int u;
 
-	   if (base==2 || base==8 || base==10 || base==16) {
-	     if (val < 0) {
-	       val  = -val;
-	       *s++ = '-';
-	     }
-	     for (s++, tmp=val; tmp >= base; tmp /= base) s++;
-	     *s = '\000'; tmp = val;
-	     do {
-	       u = tmp % base;
-	       *(--s) = u + ((u < 10) ? '0' : 'a'-10);
-	       tmp   /= base;
-	     }
-	     while (tmp);
-	     return buffer;
+	   if (val < 0) {
+	     val  = -val;
+	     *s++ = '-';
 	   }
+	   for (s++, tmp=val; tmp >= base; tmp /= base) s++;
+	   *s = '\000'; tmp = val;
+	   do {
+	     u = tmp % base;
+	     *(--s) = u + ((u < 10) ? '0' : 'a'-10);
+	     tmp   /= base;
+	   }
+	   while (tmp);
+	   return buffer;
 	 }
     case tc_bignum:
-	   if (base==2 || base==8 || base==10 || base==16) {
-	     s = must_malloc(mpz_sizeinbase(BIGNUM(n), base) + 2);
-	     s = mpz_get_str(s, base, BIGNUM(n));
-	     return s;
-	   }
+           s = must_malloc(mpz_sizeinbase(BIGNUM(n), base) + 2);
+	   s = mpz_get_str(s, base, BIGNUM(n));
+	   return s;
   }
-
-  Err("base must be 2, 8, 10 or 16", (base == LONG_MIN) ? NIL: makesmallint(base));
   return NULL; /* never reached */
 }
 
@@ -185,7 +211,6 @@ char *STk_number2Cstr(SCM n, long base, char buffer[])
 SCM STk_Cstr2number(char *str, long base)
 {
   int i, adigit=0, isint=1, exact=' ', radix = 0;
-  char *start      = str;
   register char *p = str;
 
   for (i = 0; i < 2; i++) {
@@ -198,16 +223,13 @@ SCM STk_Cstr2number(char *str, long base)
 	case 'o': if (!radix) {base = 8;  radix = 1; break;} else return Ntruth;
 	case 'd': if (!radix) {base = 10; radix = 1; break;} else return Ntruth;
 	case 'x': if (!radix) {base = 16; radix = 1; break;} else return Ntruth;
-	default:  Err("bad # syntax", STk_makestring(str));
+	default:  return Ntruth;
       }
       str += 2;
     }
     if (*p != '#') break;
   }
 
-  if (base != 2 && base != 8 && base != 10 && base != 16)
-    Err("base must be 2, 8, 10 or 16", (base==LONG_MIN) ? NIL: makesmallint(base));
-  
   if (*p == '-' || *p == '+') p+=1;
   if (*p == '#') goto End;
   while(digitp(*p, base)) { p+=1; adigit=1; if (*p == '#') isint = 0; }
@@ -234,10 +256,14 @@ SCM STk_Cstr2number(char *str, long base)
     MP_INT n;
 
     if (*str == '+') str+=1; /* mpz_init_set_str doesn't recognize +xyz !!! */
-    if (mpz_init_set_str(&n, str, base) < 0) return Ntruth;
+    if (mpz_init_set_str(&n, str, base) < 0) {
+      mpz_clear(&n);
+      return Ntruth;
+    }
     if (mpz_cmp_si(&n, SMALLINT_MIN) >=0 && mpz_cmp_si(&n, SMALLINT_MAX) <= 0) {
       long num = mpz_get_si(&n);
-      return (exact == 'i') ? STk_makenumber(num) : makesmallint(num);
+      mpz_clear(&n);
+      return (exact == 'i') ? STk_makenumber((double) num) : makesmallint(num);
     }
     /* It's a bignum */
     if (exact == 'i') return STk_makenumber(bignum2double(&n));
@@ -253,8 +279,7 @@ SCM STk_Cstr2number(char *str, long base)
   }
   
   /* It's a float */
-  if (exact == 'e')
-    Err("#e cannot be specified on this number", STk_makestring(str));
+  if (exact == 'e') return Ntruth; /* e cannot be specified on e float */
   if (base == 10) {
     /* Replace sharp signs by 0 */
     for(p=str; *p; p++) 
@@ -266,7 +291,6 @@ SCM STk_Cstr2number(char *str, long base)
     return STk_makenumber((double) atof(str));
   }
 End:
-  if (*start ==  '#') Err("Bad # syntax", STk_makestring(start));
   return Ntruth;
 }
 
@@ -293,6 +317,49 @@ int STk_equal_numbers(SCM number1, SCM number2) /* number1 = number2 */
   return do_compare(number1, number2) == 0;
 }
 
+long STk_integer2long(SCM x)
+{
+  if (BIGNUMP(x))  return mpz_get_si(BIGNUM(x));
+  return INTEGER(x);
+}
+
+unsigned long STk_integer2ulong(SCM x)
+{
+  if (BIGNUMP(x)) return mpz_get_ui(BIGNUM(x));
+  return (unsigned long) INTEGER(x);
+}
+
+/******************************************************************************/
+
+SCM STk_makenumber(double x)
+{
+  /* Floats are not stored in a struct obj since this leads to memory consumption
+   * This  memory consumption due to alignment problems. 
+   * For instance on a Sun 4, where double are 8 bytes, a struct obj with a double
+   * in line will occupy 16 bytes whereas it occupies only 12 bytes if the double
+   * is mallocated.
+   * This change (94/08/29) will give worst performances when crunching numbers,
+   * but use Fortran if this is your job :->
+   */
+
+  SCM z;
+  NEWCELL(z,tc_flonum); 
+  FLONM(z) = x; 
+  return z;
+}
+
+
+SCM STk_makeinteger(long x)
+{
+  return (SMALLINT_MIN <= x && x <= SMALLINT_MAX) ? makesmallint(x): int2bignum(x);
+}
+
+SCM STk_makeunsigned(unsigned long x)
+{
+  return (x <= SMALLINT_MAX) ? makesmallint(x): uint2bignum(x);
+}
+
+
 /******************************************************************************
  *
  * Simple operations 
@@ -319,7 +386,7 @@ static SCM clone(SCM number)
   }
 }
 
-static void do_multiply(SCM *x, SCM y)
+void _STk_do_multiply(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
@@ -330,8 +397,12 @@ static void do_multiply(SCM *x, SCM y)
 		  if (SMALLNUMP(prod=(double) INTEGER(*x) * INTEGER(y)))
 		    SET_INTEGER(*x, (long) prod);
 		  else {
+		    MP_INT tmp;
+		    
 		    *x = int2bignum(INTEGER(*x));
-		    mpz_mul_ui(BIGNUM(*x), BIGNUM(*x), INTEGER(y));
+		    mpz_init_set_si(&tmp, INTEGER(y));
+		    mpz_mul(BIGNUM(*x), BIGNUM(*x), &tmp);
+		    mpz_clear(&tmp);
 		  }
 		}
 		break;
@@ -376,19 +447,20 @@ static void do_multiply(SCM *x, SCM y)
   }
 }
 
-static void do_addition(SCM *x, SCM y)
+void _STk_do_addition(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
          switch (TYPE(y)) {
 	   case tc_integer:
 	        {
-		  double add;
-		  if (SMALLNUMP(add=(double) INTEGER(*x) + INTEGER(y)))
+		  double add =(double) INTEGER(*x) + INTEGER(y);
+
+		  if (SMALLNUMP(add))
 		    SET_INTEGER(*x, (long) add);
 		  else {
 		    *x = int2bignum(INTEGER(*x));
-		    mpz_add_ui(BIGNUM(*x), BIGNUM(*x), INTEGER(y));
+		    mpz_add(BIGNUM(*x), BIGNUM(*x), BIGNUM(int2bignum(INTEGER(y))));
 		  }
 		}
 		break;
@@ -433,19 +505,19 @@ static void do_addition(SCM *x, SCM y)
   }
 }
 
-static void do_substract(SCM *x, SCM y)
+static void _STk_do_substract(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
          switch (TYPE(y)) {
 	   case tc_integer:
 	        {
-		  double add;
-		  if (SMALLNUMP(add=(double) INTEGER(*x) - INTEGER(y)))
+		  double add = (double) INTEGER(*x) - INTEGER(y);
+		  if (SMALLNUMP(add))
 		    SET_INTEGER(*x,(long) add);
 		  else {
 		    *x = int2bignum(INTEGER(*x));
-		    mpz_sub_ui(BIGNUM(*x), BIGNUM(*x), INTEGER(y));
+		    mpz_sub(BIGNUM(*x), BIGNUM(*x), BIGNUM(int2bignum(INTEGER(y))));
 		  }
 		}
 		break;
@@ -490,7 +562,7 @@ static void do_substract(SCM *x, SCM y)
   }
 }
 
-static void do_divide(SCM *x, SCM y)
+static void _STk_do_divide(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
@@ -568,39 +640,42 @@ static void do_divide(SCM *x, SCM y)
 
 static SCM do_integer_division(SCM x, SCM y, int quotientp)
 {
+  MP_INT q, r;
+  SCM res;
+  int exact = 1;
+
   /* Divide x by y and return its quotient or remainder. y is not 0 */
-  if (INTEGERP(x) && INTEGERP(y)) {
-    return makesmallint(quotientp? INTEGER(x)/INTEGER(y):
-		       		  INTEGER(x)%INTEGER(y));
-  }
+  if (FLONUMP(x))
+    if (ISINT(FLONM(x))) { x = double2integer(FLONM(x)); exact = 0; }
+    else Err("bad number in an integer division", x);
+  
+  if (FLONUMP(y))
+    if (ISINT(FLONM(y))) { y = double2integer(FLONM(y)); exact = 0; }
+    else Err("bad number in an integer division", y);
+  
+  /* Here, x and y can only be integer or bignum (not float) */
+  if (INTEGERP(x))
+    if (INTEGERP(y)) {
+      res = makesmallint(quotientp? INTEGER(x)/INTEGER(y):
+			  	     INTEGER(x)%INTEGER(y));
+      return exact? res : STk_exact2inexact(res);
+    }
+    else
+      x = int2bignum(INTEGER(x));
   else {
-    /* result is a bignum. Eventually, transform argument to bignum  */
-    MP_INT q,  r;
-    SCM res   = makebignum("0");
-    int exact = 1;
-
-    switch (TYPE(x)) {
-      case tc_integer: x = int2bignum(INTEGER(x)); break;
-      case tc_bignum:  break;
-      case tc_flonum:  if (ISINT(FLONM(x))) {x=double2bignum(FLONM(x)); exact=0;}
-      		       else goto Error;
-    }
-    switch (TYPE(y)) {
-      case tc_integer: y = int2bignum(INTEGER(y)); break;
-      case tc_bignum:  break;
-      case tc_flonum:  if (ISINT(FLONM(y))) {y=double2bignum(FLONM(y)); exact=0;}
-      		       else { x = y; goto Error;}
-    }
-
-    mpz_init(&q); mpz_init(&r);
-    mpz_divmod(&q, &r, BIGNUM(x), BIGNUM(y)); /* use divmod instead of mdivmod */
-    mpz_set(BIGNUM(res), quotientp? &q : &r); /* Thanks to kerch@parc.xerox.com*/
-    mpz_clear(&q); mpz_clear(&r);
-    return exact? res : STk_exact2inexact(res);
+    /* x is a bignum */
+    if (INTEGERP(y))
+      y = int2bignum(INTEGER(y));
   }
-Error:
-  Err("bad number in an integer division", x);
-  return UNDEFINED; /* never reached */
+  
+  /* x and y are both bignums */
+  res = makebignum("0");
+
+  mpz_init(&q); mpz_init(&r);
+  mpz_divmod(&q, &r, BIGNUM(x), BIGNUM(y)); /* use divmod instead of mdivmod */
+  mpz_set(BIGNUM(res), quotientp? &q : &r); /* Thanks to kerch@parc.xerox.com*/
+  mpz_clear(&q); mpz_clear(&r);
+  return exact? res : STk_exact2inexact(res);
 }
 
 static double do_compare(SCM x, SCM y)
@@ -635,32 +710,6 @@ static double do_compare(SCM x, SCM y)
  *
  ******************************************************************************/
 
-SCM STk_makenumber(double x)
-{
-  /* Floats are not stored in a struct obj since this leads to memory consumption
-   * This  memory consumption due to alignment problems. 
-   * For instance on a Sun 4, where double are 8 bytes, a struct obj with a double
-   * in line will occupy 16 bytes whereas it occupies only 12 bytes if the double
-   * is mallocated.
-   * This change (94/08/29) will give worst performances when crunching numbers,
-   * but use Fortran if this is your job :->
-   */
-
-  SCM z;
-  NEWCELL(z,tc_flonum); 
-  (*z).storage_as.flonum.data = must_malloc(sizeof(double));
-  FLONM(z) = x; 
-  return z;
-}
-
-
-SCM STk_makeinteger(long x)
-{
-  return (SMALLINT_MIN <= x && x <= SMALLINT_MAX) ? makesmallint(x): int2bignum(x);
-}
-    
-
-/******************************************************************************/
 
 /**** Section 6.5 ****/
 
@@ -942,7 +991,7 @@ PRIMITIVE STk_modulo(SCM n1, SCM n2)
   if (NNUMBERP(n1) || NNUMBERP(n2))
     Err("modulo: bad arguments", Cons(n1, n2));
   if (STk_zerop(n2) == Truth) Err("modulo: division by 0", NIL);
-  
+
   z = do_integer_division(n1, n2, FALSE);
   if (STk_negativep(n1) != STk_negativep(n2) && STk_zerop(z) != Truth) 
     /*kerch@parc.xerox.com*/
@@ -1034,7 +1083,7 @@ PRIMITIVE STk_round(SCM x)
      * R4RS states that round must returns the closest integer to x, rounding to
      * even when x is halfway between two integers (round to even is for consistency
      * with IEEE standard).
-     * This explains the (too much) complicate computation below
+     * This explains the (much too) complicate computation below
      */
     double v 	      = FLONM(x); 
     double v_plus_0_5 = v + 0.5;
@@ -1046,27 +1095,20 @@ PRIMITIVE STk_round(SCM x)
   return x;
 }
 
-#ifdef __STDC__
-#define transcendental(fct)						\
-    PRIMITIVE STk_##fct(SCM z)						\
-    {									\
-      if (NUMBERP(z)) 							\
-        return STk_makenumber(fct(FLONM(STk_exact2inexact(z))));	\
-      Err(#fct ": bad number", z);					\
-      return UNDEFINED; /* never reached */				\
-    }
+#if !defined(WIN32) && !defined(__STDC__)
+#  define concat(a,b) a/**/b
 #else
-#define concat(a,b) a/**/b
-#define transcendental(fct)						\
-    PRIMITIVE concat(STk_,fct)(SCM z)					\
-    {									\
-      if (NUMBERP(z))							\
-	return STk_makenumber(fct(FLONM(STk_exact2inexact(z))));	\
-      Err("transcendental function: bad number", z);			\
-      return UNDEFINED /* never reached */;				\
-    }
+#  define concat(a,b) a##b
 #endif
 
+#define transcendental(fct)                                             \
+    PRIMITIVE concat(STk_,fct)(SCM z)                                   \
+    {                                                                   \
+      if (NUMBERP(z))                                                   \
+        return STk_makenumber(fct(FLONM(STk_exact2inexact(z))));        \
+      Err("transcendental function: bad number", z);                    \
+      return UNDEFINED /* never reached */;                             \
+    }
 
 transcendental(exp)
 transcendental(log)
@@ -1195,7 +1237,7 @@ PRIMITIVE STk_inexact2exact(SCM z)
   switch (TYPE(z)) {
     case tc_integer: 
     case tc_bignum:  return z;
-    case tc_flonum:  return double2bignum(floor(FLONM(z)+0.5));
+    case tc_flonum:  return double2integer(floor(FLONM(z)+0.5));
     default:         Err("inexact->exact: bad number", z);
       		     return UNDEFINED; /* never reached */
   }
@@ -1204,18 +1246,33 @@ PRIMITIVE STk_inexact2exact(SCM z)
 
 PRIMITIVE STk_string2number(SCM str, SCM base)
 {
+  long b = (base==UNBOUND)? 10L : STk_integer_value(base);
+
   if (NSTRINGP(str)) Err("string->number: not a string", str);
-  return STk_Cstr2number(CHARS(str), (base==UNBOUND)? 10 : STk_integer_value(base));
+  if (b != 2 && b != 8 && b != 10 && b != 16)
+    Err("string->number: base must be 2, 8, 10 or 16", NIL);
+
+  return STk_Cstr2number(CHARS(str), b);
 }
 
 PRIMITIVE STk_number2string(SCM n, SCM base)
 {
+  long b = (base==UNBOUND)? 10L : STk_integer_value(base);
   char *s, buffer[100];
   SCM z;
 
   if (NNUMBERP(n))   Err("number->string: bad number", n);
-  s = STk_number2Cstr(n, (base == UNBOUND)? 10 : STk_integer_value(base), buffer);
+  if (b != 2 && b != 8 && b != 10 && b != 16)
+    Err("number->string: base must be 2, 8, 10 or 16", NIL);
+
+  s = STk_number2Cstr(n, b, buffer);
   z = STk_makestring(s);
   if (s != buffer) free(s);
   return z;
 }
+
+PRIMITIVE  STk_bignump(SCM n)
+{
+  return BIGNUMP(n) ? Truth: Ntruth;
+}
+
