@@ -1,7 +1,5 @@
 /*
  * This file is based on a contribution of David Tolpin (dvd@pizza.msk.su)
- * It is an implementation of BSD-INET sockets and is known to run on 
- * Solaris 1 and Linux.
  *
  * Bugs correction (conversion between host and network byte order) by
  * Marc Furrer (Marc.Furrer@di.epfl.ch)
@@ -11,31 +9,47 @@
  * 
  * Win32 support by Caleb Deupree <cdeupree@erinet.com>
  *
- * Last file update: 25-Sep-1996 21:33
+ *      Modified on: 27-Apr-1999 12:54 (Steve Pruitt <steve@pruitt.net>)
+ * Last file update:  3-Sep-1999 19:46 (eg)
+ *
+ * For Win32 support the following changes were made:
+ *
+ *    1) re-arraged includes to remove define errors
+ *
+ *    2) added define for CLOSESOCKET      
+ *
+ *    3) Removed calls tcl_DeleteFileHandler and tclDeleteFileHandler
+ *
+ *    4) added new portable primitives: initialize-client-socket,
+ *            accept-connection, close-connection, socket-handle, 
+ *            socket-recv, socket-send and socket-peek
  */
 
 
 #ifdef WIN32
-#  include <fcntl.h>
-#  include <tclWinPort.h>
-#  define BAD_SOCKET(s) ((s) == INVALID_SOCKET)
-#  ifndef _O_WRONLY
-#    define _O_WRONLY O_WRONLY
-#  endif
-#  ifndef _O_RDONLY
-#    define _O_RDONLY O_RDONLY
-#  endif 
+#    define CLOSESOCKET  closesocket
+#    include <fcntl.h>
+#    include "stk.h"
+#    include "stk.h"
+#    define BAD_SOCKET(s) ((s) == INVALID_SOCKET)
+#    ifndef _O_WRONLY
+#      define _O_WRONLY O_WRONLY
+#    endif
+#    ifndef _O_RDONLY
+#      define _O_RDONLY O_RDONLY
+#    endif 
 #else 
+#  define CLOSESOCKET  close
 #  include <sys/types.h>
 #  include <sys/socket.h>
 #  include <netinet/in.h>
 #  include <arpa/inet.h>
 #  include <netdb.h>
 #  include <memory.h>
+#  include "stk.h"
 #  define BAD_SOCKET(s) ((s) < 0)
 #endif
 
-#include "stk.h"
 #include <errno.h>
 
 struct socket_type {
@@ -77,13 +91,12 @@ static void socket_error(char *who, char *message, SCM object)
 
 static void set_socket_io_ports(int s, SCM sock, char *who)
 {
-  int t, len, port;
+  int t, port;
+  size_t len;
   char *hostname, *fname;
-  FILE *fs, *ft;
+  FILE *fs= NULL, *ft = NULL;  /* initialization for GCC! */
   char buffer[200];
 
-  STk_disallow_sigint();
-
 #ifdef WIN32
   {
     int r;
@@ -117,7 +130,7 @@ static void set_socket_io_ports(int s, SCM sock, char *who)
   port     = SOCKET(sock)->portnum;
   hostname = CHARS(SOCKET(sock)->hostname);
   len      = strlen(hostname) + 20;
-  fname    =  (char*) must_malloc(len);
+  fname    = (char*) must_malloc(len);
   sprintf(fname, "%s:%d", hostname, port);
 
   /* Create input port */
@@ -125,8 +138,6 @@ static void set_socket_io_ports(int s, SCM sock, char *who)
 
   /* Create output port */
   SOCKET(sock)->output = STk_Cfile2port(strdup(fname), ft, tc_oport, 0);
-
-  STk_allow_sigint();
 }
 
 /******************************************************************************
@@ -134,15 +145,29 @@ static void set_socket_io_ports(int s, SCM sock, char *who)
  *   m a k e - c l i e n t - s o c k e t
  *
  ******************************************************************************/
-
+#ifdef WIN32
+/* initialize_client_socket is identical to make_client_socket */
+/* except that it does not call "set_socket_io_ports" */
+static PRIMITIVE initialize_client_socket(SCM hostname, SCM port);
+    
+static PRIMITIVE make_client_socket(SCM hostname, SCM port) 
+{ 
+ SCM sock;
+ sock = initialize_client_socket(hostname, port);
+ set_socket_io_ports(SOCKET(sock)->fd, sock, "make-client-socket");
+ return sock;
+}
+  
+static PRIMITIVE initialize_client_socket(SCM hostname, SCM port)
+#else
 static PRIMITIVE make_client_socket(SCM hostname, SCM port)
+#endif
 {
   char str[] = "make-client-socket";
   struct hostent *hp;
   struct sockaddr_in server;
-  struct in_addr local_ip;
-  SCM z, local_host;
   int s;
+  SCM z;
 
   /* Verify arguments */
   if(NSTRINGP(hostname)) 
@@ -166,7 +191,7 @@ static PRIMITIVE make_client_socket(SCM hostname, SCM port)
 
   /* Try to connect */
   if (connect(s, (struct sockaddr *) &server, sizeof(server)) < 0) {
-    close(s);
+    CLOSESOCKET(s);
     system_error(str);
   }
 
@@ -182,8 +207,9 @@ static PRIMITIVE make_client_socket(SCM hostname, SCM port)
   SOCKET(z)->input 	 = Ntruth;
   SOCKET(z)->output 	 = Ntruth;
   SOCKET(z)->ready_event = Ntruth;
-
+#ifndef WIN32
   set_socket_io_ports(s, z, str);
+#endif
   return z;
 }
 
@@ -198,8 +224,6 @@ static PRIMITIVE make_server_socket(SCM port)
   char msg[] = "make-server-socket";
   struct sockaddr_in sin;
   int s, portnum, len;
-  SCM local_host;
-  struct in_addr local_ip;
   SCM z;
 
   /* Determine port to use */
@@ -216,20 +240,20 @@ static PRIMITIVE make_server_socket(SCM port)
   sin.sin_addr.s_addr = INADDR_ANY;
 
   if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-    close(s);
+    CLOSESOCKET(s);
     system_error(msg);
   }
 
   /* Query the socket name (permits to get the true socket number if 0 was given */
   len = sizeof(sin);
   if (getsockname(s, (struct sockaddr *) &sin, (int *) &len) < 0) {
-    close(s);
+    CLOSESOCKET(s);
     system_error(msg);
   }
 
   /* Indicate that we are ready to listen */
   if (listen(s, 5) < 0) {
-    close(s);
+    CLOSESOCKET(s);
     system_error(msg);
   }
 
@@ -253,10 +277,26 @@ static PRIMITIVE make_server_socket(SCM port)
  *   s o c k e t - a c c e p t - c o n n e c t i o n
  *
  ******************************************************************************/
-
-static PRIMITIVE socket_accept_connection(SCM sock)
+#ifdef WIN32
+/* accept_connection is identical to socket_accept_connection */
+/* except that it does not call "set_socket_io_ports" and */
+/* it also returns the accepted socket as a SCM integer */
+static PRIMITIVE accept_connection(SCM sock);
+  
+static PRIMITIVE socket_accept_connection(SCM sock) 
+{ 
+ int new_s;
+ new_s = INTEGER(accept_connection(sock));
+ set_socket_io_ports(new_s , sock, "socket-accept-connection");
+ return UNDEFINED;
+}
+  
+static PRIMITIVE accept_connection(SCM sock)
+#else
+static PRIMITIVE socket_accept_connection(SCM sock) 
+#endif
 {
-  char buff[50], *s;
+  char *s;
   char str[]= "socket-accept-connection";
   struct sockaddr_in sin;
   struct hostent *host;
@@ -276,8 +316,12 @@ static PRIMITIVE socket_accept_connection(SCM sock)
   SOCKET(sock)->hostip   = STk_makestring(s);
   SOCKET(sock)->hostname = STk_makestring(host? (char*) (host->h_name): s);
 
+#ifdef WIN32
+  return STk_makeinteger(new_s);
+#else
   set_socket_io_ports(new_s, sock, str);
   return UNDEFINED;
+#endif
 }
 
 /******************************************************************************
@@ -285,15 +329,24 @@ static PRIMITIVE socket_accept_connection(SCM sock)
  *   w h e n - s o c k e t - r e a d y 
  *
  ******************************************************************************/
+#ifdef WIN32
+static PRIMITIVE when_socket_ready(SCM s, SCM closure)
+{
+  /* Removal of tcl_DeleteFileHandler and tclDeleteFileHandler */ 
+  /* in Tcl/Tk 8.0 make when_socket_ready impossible on Win32. */
+  /* Will be available again in Tcl/Tk version 8.1    FIXME    */
+  Err("when-socket-ready: cannot be used with Win32", NIL);
+  return UNDEFINED;
+}
+#else
 static void apply_socket_closure(SCM closure)
 {
-  Apply(closure, NIL);
+  Apply0(closure);
 }
  
 static PRIMITIVE when_socket_ready(SCM s, SCM closure)
 {
-  char str[50];
-  Tcl_File f;
+  int fd;
 
   if (NSOCKETP(s))
     Err("when-socket-ready: bad socket", s);
@@ -303,26 +356,28 @@ static PRIMITIVE when_socket_ready(SCM s, SCM closure)
     return SOCKET(s)->ready_event;
   }
   
-  f = Tcl_GetFile((ClientData) SOCKET(s)->fd,  TCL_UNIX_FD);
-  
+  fd = SOCKET(s)->fd;
+ 
   if (closure == Ntruth) {
-    Tcl_DeleteFileHandler(f);    
+    Tcl_DeleteFileHandler(fd);    
     SOCKET(s)->ready_event = Ntruth;
   }
   else {
     if (STk_procedurep(closure) == Ntruth) 
       Err("when-socket-ready: bad closure", closure);
 
-    Tcl_CreateFileHandler(f, TCL_READABLE, (Tcl_FileProc *) apply_socket_closure, 
+    Tcl_CreateFileHandler(fd, TCL_READABLE, (Tcl_FileProc *) apply_socket_closure, 
 			  (ClientData) closure);
     SOCKET(s)->ready_event = closure;
   }
   return UNDEFINED;
 }
+#endif
 
 static PRIMITIVE buggy_handler(SCM s, SCM closure)
 {
   Err("when-socket-ready: cannot be used with snow", NIL);
+  return UNDEFINED;
 }
 
 /******************************************************************************
@@ -331,15 +386,6 @@ static PRIMITIVE buggy_handler(SCM s, SCM closure)
  *
  ******************************************************************************/
 
-static void shutdown_port(SCM port)
-{
-  int fd;
-  FILE *f;
-
-  fd  = fileno(PORT_FILE(port));
-  if (!(PORT_FLAGS(port) & PORT_CLOSED)) /* not already closed */ shutdown(fd, 2);
-}
-
 static PRIMITIVE socket_shutdown(SCM sock, SCM close_socket)
 {
   if (close_socket == UNBOUND) close_socket = Truth;
@@ -348,24 +394,42 @@ static PRIMITIVE socket_shutdown(SCM sock, SCM close_socket)
   if (NBOOLEANP(close_socket)) Err("socket-shutdown: bad boolean", close_socket);
 
   if (close_socket == Truth && SOCKET(sock)->fd > 0) {
+    int fd = SOCKET(sock)->fd;
+
+#ifndef WIN32
     if (!STk_snow_is_running)
       /* We cannot use #ifdef USE_TK here to have the same socket.so
        * for both snow and stk. So we have to test if we are running 
        * snow dynamically
        */
-      Tcl_DeleteFileHandler(Tcl_GetFile((ClientData) SOCKET(sock)->fd, 
-					TCL_UNIX_FD));
-    close(SOCKET(sock)->fd);
+      Tcl_DeleteFileHandler(fd);
+#endif
+
+    /* close(fd); */
+    shutdown(fd, 2);
     SOCKET(sock)->fd = -1;
   }
  
-  shutdown_port(SOCKET(sock)->input);  
-  shutdown_port(SOCKET(sock)->output); 
-  
-  /* Unset input and ouput pointers. By doing that, GC will close the
-   * input and ouput files later.
+  /* 
+   * Warning: input and output can have already be garbaged :if the socket is 
+   * no more used, the input and output are not marked as used and can
+   * (eventually) be released before the call to shutdown (through free_socket)
+   * be done. One way could be to just set SOCKET(sock)->{in|out}put to #t
+   * and wait that next GC frees the ports if not already down. However,
+   * this will really disconnect the peer when the GC occurs rather than when
+   * the call to shutdown is done. This is not important if this function 
+   * is called by the GC, but could be annoying when it is called by the user
    */
-  SOCKET(sock)->input = SOCKET(sock)->output = Ntruth;
+
+  if (INP(SOCKET(sock)->input)) {
+    STk_close_port(SOCKET(sock)->input);
+    SOCKET(sock)->input = Ntruth;
+  }
+  if (OUTP(SOCKET(sock)->output)) {
+    STk_close_port(SOCKET(sock)->output);
+    SOCKET(sock)->output = Ntruth;
+  }
+
   return UNDEFINED;
 }
 
@@ -489,7 +553,7 @@ static void displ_socket(SCM sock, SCM port, int mode)
   sprintf(STk_tkbuffer, "#[socket %s %d]", 
 	  		(s->hostname == Ntruth)?"*none*": CHARS(s->hostname),
 	  		s->portnum);
-  Puts(STk_tkbuffer, PORT_FILE(port));
+  Puts(STk_tkbuffer, port);
 }
 
 static STk_extended_scheme_type socket_type = {
@@ -498,9 +562,73 @@ static STk_extended_scheme_type socket_type = {
   mark_socket, 		/* gc_mark_fct */
   free_socket,		/* gc_free_fct */
   NULL,			/* apply_fct */
-  displ_socket		/* display_fct */
+  displ_socket,		/* display_fct */
+  NULL			/* compare function */
 };
 
+#ifdef WIN32
+/*******************************************************************************
+ *
+ * These primitives allow WIN32 apps to work when the 
+ * socket io-ports are not available. By adding Primitives
+ * to winsock functions "send" and "recv", sockets can be
+ * operated on directly. 
+ *
+ ******************************************************************************/
+
+static PRIMITIVE close_connection(SCM connection)
+{
+  if (NINTEGERP(connection)) 
+	Err("close_connection: bad connection", connection);
+  CLOSESOCKET(INTEGER(connection));
+  return UNDEFINED;
+}
+
+/******************************************************************************/
+
+static PRIMITIVE socket_handle(SCM sock)
+{
+  if (NSOCKETP(sock)) Err("socket-handle: bad socket", sock);
+  return STk_makeinteger(SOCKET(sock)->fd);
+}
+
+/******************************************************************************/
+
+static PRIMITIVE socket_peek(SCM fd, SCM num_chars) {
+ int i, len;
+ SCM chars;
+ char *s;
+ len = INTEGER(num_chars);
+ chars = STk_makestrg(len+1, NULL);
+ s = chars->storage_as.string.data;
+ i = recv(INTEGER(fd), s, len, MSG_PEEK);
+ return chars;
+}
+
+/******************************************************************************/
+
+static PRIMITIVE socket_recv(SCM fd, SCM buffer) {
+ int len, result;
+ char *s;
+ s = buffer->storage_as.string.data;
+ len = buffer->storage_as.string.dim - 1;
+ result = recv(INTEGER(fd), s, len, 0);
+ if (result <= 0) return STk_eof_object;
+ return STk_makeinteger(result);
+}
+
+/******************************************************************************/
+
+static PRIMITIVE socket_send(SCM fd, SCM buffer, SCM num_chars) {
+ int len, result;
+ char *s;
+ len = INTEGER(num_chars);
+ s = buffer->storage_as.string.data;
+ result = send(INTEGER(fd), s, len, 0);
+ return STk_makeinteger(result);
+}
+#endif
+
 /******************************************************************************/
 
 PRIMITIVE STk_init_socket(void)
@@ -521,6 +649,16 @@ PRIMITIVE STk_init_socket(void)
     Err("Synchronous socket option setting failed", 
 	STk_makeinteger(WSAGetLastError()));
   }
+
+  /* new primitives for send and recv without using stdio */
+  STk_add_new_primitive("accept-connection", tc_subr_1, accept_connection);
+  STk_add_new_primitive("close-connection", tc_subr_1, close_connection);
+  STk_add_new_primitive("socket-handle", tc_subr_1, socket_handle);
+  STk_add_new_primitive("socket-peek", tc_subr_2, socket_peek);
+  STk_add_new_primitive("socket-recv", tc_subr_2, socket_recv);
+  STk_add_new_primitive("socket-send", tc_subr_3, socket_send);
+  STk_add_new_primitive("initialize-client-socket", tc_subr_2, 
+					initialize_client_socket);
 #endif
 
   tc_socket = STk_add_new_type(&socket_type);
