@@ -2,29 +2,27 @@
  *
  * u n i x . c					-- Some Unix primitives
  *
- * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
- *
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 29-Mar-1994 10:57
- * Last file update: 13-Sep-1996 10:17
+ * Last file update: 15-Sep-1999 18:05 (eg)
  */
 #ifndef WIN32
 #  include <unistd.h>
 #  include <pwd.h>
 #else
+#  include <windows.h>
 #  include <io.h>
 #  define F_OK   00
 #  define X_OK   01
@@ -41,14 +39,17 @@
     */
 #  undef __STDARG_H 
 
-#  ifdef MSC_VER
-#    include <direct.h>
-#    include <windows.h>
-#    define S_ISDIR(mode) ((mode&0xF0000) == 0x40000)
-#    define S_ISREG(mode) ((mode&0xF00000) == 0x100000)
+#  ifdef _MSC_VER
+#  include <direct.h>
+#  include <process.h>
+#  include <sys/stat.h>
+#  define S_ISDIR(mode) ((mode & _S_IFMT) == _S_IFDIR)
+#  define S_ISREG(mode) ((mode & _S_IFMT) == _S_IFREG)
 #  else
-      /* Borland defines the opendir/readdir/closedir functions. Use them. */
-#     include <dirent.h>
+#     ifdef BCC
+        /* Borland defines the opendir/readdir/closedir functions. Use them. */
+#       include <dirent.h>
+#     endif
 #  endif
 #else
 #  include <dirent.h>
@@ -84,32 +85,35 @@ static char *my_getcwd(char *path, int size)
 
 static char *tilde_expand(char *name, char *result)
 {
-  char *dir, *p;
-
-#ifdef WIN32
-  strcpy(result, name);
-  return name;	
-#else
   if (name[0] != '~') {
     strcpy(result, name);
     return name;
   }
-
+  
+#ifdef WIN32
+  if ((name[1] == '/') || (name[1] == '\\') || (name[1] == '\0')) {
+#else
   if ((name[1] == '/') || (name[1] == '\0')) {
-    dir = getenv("HOME");
-    if (dir == NULL)
-      Err("couldn't find HOME environment for expanding ", STk_makestring(name));
+#endif
+    char *dir = getenv("HOME");
     
+    if (dir == NULL)
+      Err("couldn't find HOME in environment when expanding", STk_makestring(name));
+
     sprintf(result, "%s%s", dir, name+1);
   } 
   else {
+#ifdef WIN32
+    Err("Form '~user' not allowed on Win32", STk_makestring(name));
+#else
+    char *p;
     struct passwd *pwPtr;
     register int len;
     for (p=&name[1]; (*p != 0) && (*p != '/'); p++) {
       /* Null body;  just find end of name. */
     }
     len = p-(name+1);
-    strncpy(result, name+1, len);
+    strncpy(result, name+1, (size_t) len);
     result[len] = '\0';
 
     pwPtr = getpwnam(result);
@@ -119,9 +123,9 @@ static char *tilde_expand(char *name, char *result)
     }
     sprintf(result, "%s%s", pwPtr->pw_dir, p);
     endpwent();
+#endif
   }
   return result;
-#endif
 }
 
 /*
@@ -136,9 +140,16 @@ static void absolute(char *s, char *pathname)
   if (!ISABSOLUTE(s)) {
     getcwd(pathname, MAX_PATH_LENGTH);
     p = &pathname[strlen(pathname)];     /* place p at end of pathname */ 
+#ifdef WIN32 
+    *p = DIRSEP;
+#endif
   }
+#ifdef WIN32
+  else *p = *s++;
+#else
   *p = DIRSEP;
-
+#endif
+  
   for ( ; *s; s++) {
     t = s;
     switch (*s) {
@@ -182,6 +193,12 @@ static void absolute(char *s, char *pathname)
   /* Place a \0 at end. If path ends with a "/", delete it */
   if (p == pathname || !ISDIRSEP(*p)) p++;
   *p = '\0';
+
+#ifdef WIN32
+  /* Replace all "/" by "\" */
+  for (p = pathname; *p; p++) 
+    if (*p == '/') *p = '\\';
+#endif
 }
 
 
@@ -379,7 +396,7 @@ static SCM fileglob(char *dir, char *rem, SCM result)
     handle = FindFirstFile(Tcl_DStringValue(&msvcname), &wfd);
     if (handle == INVALID_HANDLE_VALUE) {
       Tcl_DStringFree(&msvcname);
-      Err("Cannot find files, error = ", STk_makestring(GetLastError()));
+      Err("Cannot find files, error = ", STk_makestring((char *) GetLastError()));
     }
    
     savedChar = *p;
@@ -510,6 +527,10 @@ void STk_whence(char *exec, char *path)
 #endif
 
   p = getenv("PATH");
+  if (p == NULL) {
+    p = "/bin:/usr/bin";
+  }
+
   while (*p) {
     /* Copy the stuck of path in dir */
     for (q = dir; *p && *p != PATHSEP; p++, q++) *q = *p;
@@ -561,7 +582,11 @@ PRIMITIVE STk_expand_file_name(SCM s)
 PRIMITIVE STk_canonical_path(SCM str)
 {
   if (NSTRINGP(str)) Err("canonical-path: not a string", str);
+#ifdef WIN32
+  return str;
+#else
   return STk_resolve_link(CHARS(str), 0);
+#endif
 }
 
 PRIMITIVE STk_getcwd(void)
@@ -612,7 +637,8 @@ PRIMITIVE STk_setenv(SCM var, SCM value)
   if (strchr(CHARS(var), '=')) Err("setenv!: variable contains a '='", var);
   if (NSTRINGP(value)) Err("setenv!: value is not a string", value);
 
-  s = malloc(strlen(CHARS(var))+ strlen(CHARS(value)) + 2); /* 2 cause  '=' & \0 */
+  s = STk_must_malloc(strlen(CHARS(var))+ 
+		      strlen(CHARS(value)) + 2); /* 2 cause  '=' & \0 */
   sprintf(s, "%s=%s", CHARS(var), CHARS(value));
   putenv(s);
   return UNDEFINED;
@@ -653,6 +679,7 @@ static SCM my_stat(SCM path, int mode, char *who)
     case 1: return (S_ISDIR(info.st_mode)) ? Truth : Ntruth;
     case 2: return (S_ISREG(info.st_mode)) ? Truth : Ntruth;
   }
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_file_is_directoryp(SCM f)
@@ -696,11 +723,39 @@ PRIMITIVE STk_file_glob(SCM l, int len) /* len is unused here */
     
     tilde_expand(CHARS(CAR(l)), s);
 
-    res = STk_append(LIST2(res, 
-			   (ISDIRSEP(*s)) ? fileglob(SDIRSEP, s+1, NIL) :
-			   fileglob("", s, NIL)),
-		     2);
+    res = STk_append2(res, (ISDIRSEP(*s)) ? fileglob(SDIRSEP, s+1, NIL) :
+		                            fileglob("", s, NIL));
   }
   return res;
 }
 
+PRIMITIVE STk_remove_file(SCM filename)
+{
+  ENTER_PRIMITIVE("remove-file");
+  if (NSTRINGP(filename)) Serror("bad string", filename);
+  if (remove(CHARS(filename)) != 0)
+    Serror("cannot remove file", filename);
+  return UNDEFINED;
+}
+
+PRIMITIVE STk_rename_file(SCM filename1, SCM filename2)
+{
+  ENTER_PRIMITIVE("rename-file");
+  if (NSTRINGP(filename1)) Serror("bad string", filename1);
+  if (NSTRINGP(filename2)) Serror("bad string", filename2);
+  if (rename(CHARS(filename1), CHARS(filename2)) != 0)
+    Serror("cannot rename file", filename1);
+  return UNDEFINED;
+}
+
+PRIMITIVE STk_temporary_file_name(void)
+{
+  char *s;
+
+  s = tmpnam(NULL);
+  return s ? STk_makestring(s) : Ntruth;
+}
+
+
+  
+    
