@@ -1,23 +1,21 @@
 /*
  * d u m p . c				-- Image creation
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: ??-Jul-1993 ??:??
- * Last file update:  1-Feb-1995 14:35
+ * Last file update:  3-Sep-1999 20:19 (eg)
  */
 
 #include "stk.h"
@@ -125,12 +123,12 @@ void STk_restore_image(char *s)
   Restore_data_file(s);
   STk_dumped_core =  restoring_image = 1;
 
-  gcont = VCELL(Intern("*global-continuation*"));
+  gcont = STk_lookup_variable("*global-continuation*", NIL);
   /* After reading the file we must have a continuation in *global-continuation* */
   if (NCONTINUATIONP(gcont)) {
     Err("restore: file loaded is corrupted. DANGER.", NIL);
   }
-  Apply(gcont, LIST1(Ntruth));
+  Apply1(gcont, Ntruth);
 }
 
 /******************************************************************************
@@ -171,5 +169,6 @@ void STk_restore_image(char *s)
 PRIMITIVE STk_dump(SCM s)
 {
   dump_error();
+  return UNDEFINED; /* never reached */
 }
 #endif
