@@ -2,24 +2,21 @@
  *
  * e v a l . c				-- The evaluator
  *
- * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
- *
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 23-Oct-1993 21:37
- * Last file update:  2-Sep-1996 19:18
+ * Last file update:  3-Sep-1999 20:19 (eg)
  */
 
 #include "stk.h"
@@ -28,7 +25,7 @@
 #  include "stklos.h"
 #endif
 
-#define RETURN(x)		{tmp = (x); goto Out; }
+#define RETURN(x)			 {tmp = (x); goto Out; }
 
 /*
  * STk_eval_flag indicates if eval has something to test (a ^C has
@@ -43,71 +40,38 @@
 
 int STk_eval_flag = 0;
 
+
 /*
- * Eval stack
- * 
- * The eval stack is a stack of the arguments passed to eval. This stack permits
- * to facilitate debugging  of Scheme programs. Its contents is displayed 
- * when an error occurs.
- * Note that "STk_eval_stack" does'nt need to be protected since it contains 
- * pointers which are themselves copies of the eval C routine. Eval parameters
- * will be marked as are all the objects which are in the C stack
- * */
+ *
+ * Utilities 
+ *
+ */
 
-static struct Stack_info {
-  SCM expr, env;
-  struct Stack_info *previous;
-} *stack = NULL;
-
-
-void STk_show_eval_stack(int depth)
+static SCM extend_env(SCM proc, SCM actuals, SCM call, int len)
 {
-  int j;
-  struct Stack_info *p;
+  register int arity = CLOSARITY(proc);
 
-  fprintf(STk_stderr, "\nCurrent eval stack:\n__________________\n");
-  for (p=stack, j=0; p && j<=depth ; p=p->previous, j++) {
-    fprintf(STk_stderr, "%3d    ", j);
-    STk_print(STk_uncode(p->expr), STk_curr_eport, WRT_MODE);
-    Putc('\n', STk_stderr);
-    if (j == depth && p->previous) fprintf(STk_stderr, "...\n");
+  /* Code is weird, but we do only 2 tests for corrects call */
+  if (arity >= 0) {
+    if (arity == len) return add_frame(CLOSPARAMS(proc), actuals, CLOSENV(proc));
+    if (len < arity) goto TooFew;
+    goto TooMuch;
   }
-}
-
-void STk_reset_eval_stack(void)
-{
-  stack = NULL;
-}
-
-PRIMITIVE STk_get_eval_stack(void)
-{
-  struct Stack_info *p;
-  SCM 		     z = NIL;
-  
-  for (p = stack; p ; p = p->previous) z = Cons(p->expr, z);
-  
-  return STk_reverse(z);
-}
-
-PRIMITIVE STk_get_env_stack(void)
-{
-  struct Stack_info *p;
-  SCM 		     z = NIL;
-  
-  for (p = stack; p ; p = p->previous)  {
-    /* Avoid to create an environment for each item */
-    SCM tmp = (z!=NIL && STk_equal(CAR(z)->storage_as.env.data,p->env)==Truth) ?
-      		CAR(z): 
-      		STk_makeenv(p->env, 0);
-    z = Cons(tmp, z);
-  }
-  return  STk_reverse(z);
-}
-
-
-SCM STk_top_env_stack(void)
-{
-  return stack ? STk_makeenv(stack->env, 0): STk_globenv;
+  /* arity is < 0  <=>  proc has an &rest argument */
+  if (len >= (-arity)-1) 
+    /* When the procedure has a &rest parameter, we add a new environment
+     * in front of the extended environment. This environment is only useful
+     * for procedures which have internal defines (which will be placed in
+     * this empty environment). This empty environment is not created for 
+     * non &rest procedure since it costs (and there is perhaps no define 
+     * in the proc!).
+     */
+    return add_frame(NIL, NIL, add_frame(CLOSPARAMS(proc), actuals, CLOSENV(proc)));
+TooFew:
+  Err("too few arguments to", call);
+TooMuch:
+  Err("too many arguments to", call);
+  return UNDEFINED; /* never reached */
 }
 
 static SCM eval_args(SCM l, SCM env)
@@ -139,7 +103,7 @@ static SCM eval_cond(SCM *pform, SCM env)
       if (EQ(CAR(tmp), Sym_imply)) {
 	/* Clause is ((condition) => function) */
 	if (STk_llength(tmp) != 2) Err("cond: malformed `=>'", tmp);
-	SYNTAX_RETURN(Apply(EVALCAR(CDR(tmp)), LIST1(res)), Ntruth);
+	SYNTAX_RETURN(Apply1(EVALCAR(CDR(tmp)), res), Ntruth);
       }
       else {
 	for( ; NNULLP(CDR(tmp)); tmp=CDR(tmp))
@@ -154,6 +118,82 @@ Error:
   return UNDEFINED; /* never reached */
 }
 
+
+/*
+ * Eval stack
+ * 
+ * The eval stack is a stack of the arguments passed to eval. This stack permits
+ * to facilitate debugging  of Scheme programs. Its contents is displayed 
+ * when an error occurs.
+ * Note that "STk_eval_stack" does'nt need to be protected since it contains 
+ * pointers which are themselves copies of the eval C routine. Eval parameters
+ * will be marked as are all the objects which are in the C stack
+ * */
+
+static struct Stack_info {
+  SCM expr, env;
+  struct Stack_info *previous;
+} *stack = NULL;
+
+void STk_show_eval_stack(int depth, int uncode)
+{
+  int j;
+  struct Stack_info *p;
+
+  Fprintf(STk_curr_eport, "\nCurrent eval stack:\n__________________\n");
+  for (p=stack, j=0; p && j<=depth ; p=p->previous, j++) {
+    Fprintf(STk_curr_eport, "%3d    ", j);
+    /* if !uncode we are in panic mode (i.e. don't allocate during printing) */
+    if (uncode)
+      STk_print(STk_uncode(p->expr), STk_curr_eport, WRT_MODE);
+    else
+      STk_print(p->expr, STk_curr_eport, PANIC_MODE);
+    Putc('\n', STk_curr_eport);
+    if (j == depth && p->previous) Puts("...\n", STk_curr_eport);
+  }
+}
+
+
+void STk_reset_eval_stack(void)
+{
+  stack = NULL;
+}
+
+
+PRIMITIVE STk_user_get_eval_stack(void)
+{
+  struct Stack_info *p;
+  SCM 		     z;
+
+  if (stack) {
+    z = NIL;
+    for (p = stack; p ; p = p->previous)
+      z = Cons(p->expr, z);
+    return STk_reverse(z);
+  }
+  return NIL;
+}
+
+PRIMITIVE STk_get_env_stack(void)
+{
+  struct Stack_info *p;
+  SCM 		     z;
+
+  if (stack) {
+    z = NIL;
+    for (p = stack; p ; p = p->previous)  {
+      /* Avoid to create an environment for each item */
+      SCM tmp = (z!=NIL && STk_equal(CAR(z)->storage_as.env.data,p->env)==Truth) ?
+				CAR(z): 
+				STk_makeenv(p->env, 0);
+      z = Cons(tmp, z);
+    }
+    return  STk_reverse(z);
+  }
+  return NIL;
+}
+
+
 /*
  * *eval-hook* management.
  *
@@ -192,7 +232,7 @@ static void set_eval_hook(char *unused, SCM value)
 
     eval_hook_stack->hook  	  = value;
     eval_hook_stack->bypass_check = FALSE;
-    SET_EVAL_FLAG(1);
+    STk_eval_flag = TRUE;
   }
 }
 
@@ -208,11 +248,11 @@ static SCM handle_eval_hook(SCM x, SCM env)
   eval_hook_stack   = &info;
 
   /* Call user code */
-  res = STk_apply(info.previous->hook, LIST2(x, STk_makeenv(env, 0)));
+  res = Apply2(info.previous->hook, x, STk_makeenv(env, 0));
   
   /* If we are here, everything was correct */
   eval_hook_stack = info.previous;
-  SET_EVAL_FLAG(1);
+  STk_eval_flag = 1;
   return res;
 }
 
@@ -243,11 +283,11 @@ PRIMITIVE STk_eval_hook(SCM x, SCM env, SCM hook)
   info.bypass_check = TRUE;
   info.previous     = eval_hook_stack;
   eval_hook_stack   = &info;
-  SET_EVAL_FLAG(1);
+  STk_eval_flag     = 1;
 
   res = STk_eval(x, env->storage_as.env.data);
   eval_hook_stack = info.previous;
-  SET_EVAL_FLAG(1);
+  STk_eval_flag     = 1;
 
   return res;
 }
@@ -272,9 +312,9 @@ Top:
      *		- a ^C ?
      *		- *eval-hook*?
      */
-    
-    if (STk_control_C && !STk_sigint_counter) STk_handle_signal(SIGINT);
-    
+    if (STk_control_C) {
+      STk_handle_sigint_signal();
+    }
     if (eval_hook_stack->hook != Ntruth) {
       if (eval_hook_stack->bypass_check) {
 	eval_hook_stack->bypass_check = FALSE;
@@ -286,17 +326,18 @@ Top:
       else
 	RETURN(handle_eval_hook(x, env));
     }
-    SET_EVAL_FLAG(STk_control_C || 
-		  (eval_hook_stack->hook != Ntruth));
+    STk_eval_flag = STk_control_C || (eval_hook_stack->hook != Ntruth);
   }
 
   switch TYPE(x) {
     case tc_symbol:
          RETURN(*STk_varlookup(x, env, 1));
     case tc_globalvar:
-	 RETURN(VCELL(VCELL(x)));
+         RETURN(VCELL(VCELL(x)));
     case tc_localvar:
 	 RETURN(STk_localvalue(x, env));
+    case tc_modulevar:
+      	 RETURN(STk_modulevalue(x));
     case tc_cons: {
  	 /* Evaluate the first argument of this list (without calling eval) */
 	 tmp = CAR(x);
@@ -307,9 +348,11 @@ Top:
 	   case tc_cons:
 		fct = EVAL(tmp); break;
 	   case tc_globalvar:
-		fct = VCELL(VCELL(tmp)); break;
+	        fct = VCELL(VCELL(tmp)); break;
 	   case tc_localvar:
 		fct = STk_localvalue(tmp, env); break;
+	   case tc_modulevar:
+	     	fct = STk_modulevalue(tmp); break;
            default:
 		fct = tmp;
   	 }
@@ -372,26 +415,27 @@ Top:
 #ifdef USE_STKLOS
 	   case tc_instance:
 	        tmp = eval_args(tmp, env);
-
 	        if (PUREGENERICP(fct)) {
 		  /* Do it in C */
+		  SCM methods;
+		  
 		  if (NULLP(THE_SLOT_OF(fct, S_methods)))
-		    Apply(VCELL(Intern("no-method")), LIST2(fct, tmp));
+		    Apply2(STk_STklos_value(Intern("no-method")), fct, tmp);
 
-		  fct = STk_compute_applicable_methods(fct, tmp, len, FALSE);
-		  /* fct is the list of applicable methods. Apply the
+		  methods = STk_compute_applicable_methods(fct, tmp, len, FALSE);
+		  /* methods is the list of applicable methods. Apply the
 		   * first one with the tail of the list as first
 		   * parameter (next-method). If fct is NIL, that's because
 		   * the no-applicable-method triggered didn't call error.
 		   */
-		  if (NULLP(fct)) RETURN(UNDEFINED);
-		  tmp = Cons(STk_make_next_method(CDR(fct), tmp), tmp);
-		  fct = THE_SLOT_OF(CAR(fct), S_procedure);
-		  env = STk_extend_env(CLOSURE_PARAMETERS(fct),
-				       tmp,
-				       fct->storage_as.closure.env,
-				       x);
-		  tmp = CDR(fct->storage_as.closure.code);
+		  if (NULLP(methods)) RETURN(UNDEFINED);
+		  tmp = Cons(FASTMETHODP(CAR(methods))? 
+			     		UNBOUND: 
+			     		STk_make_next_method(CDR(methods),tmp,fct),
+			     tmp);
+		  fct = THE_SLOT_OF(CAR(methods), S_procedure);
+		  env = extend_env(fct, tmp, x, len+1);
+		  tmp = CLOSBODY(fct);
 		  goto Begin;
 		}
 		else
@@ -408,11 +452,8 @@ Top:
 	    	RETURN(STk_execute_Tcl_lib_cmd(fct, tmp, env, 1));
 #endif
 	   case tc_closure:
-		env = STk_extend_env(CLOSURE_PARAMETERS(fct),
-				     eval_args(tmp, env),
-				     fct->storage_as.closure.env,
-				     x);
-		tmp = CDR(fct->storage_as.closure.code);
+	        env = extend_env(fct, eval_args(tmp, env), x, len);
+		tmp = CLOSBODY(fct);
 		/* NOBREAK */
 Begin:	   case tc_begin:
 		for( ; NNULLP(CDR(tmp)); tmp=CDR(tmp))
@@ -420,22 +461,18 @@ Begin:	   case tc_begin:
 		x = CAR(tmp);
 	        goto Top;
 	   case tc_cont:
-		if (len == 1) STk_throw(fct, EVALCAR(tmp));
-		goto Error;
+               STk_throw(fct, eval_args(tmp, env));
 	   case tc_let:
-		env = STk_fast_extend_env(CAR(tmp), 
-					  eval_args(CAR(CDR(tmp)),env), 
-					  env);
+		env = add_frame(CAR(tmp), eval_args(CAR(CDR(tmp)),env), env);
 		tmp = CDR(CDR(tmp));
 		goto Begin;
 	   case tc_letstar:
 		{
 		  SCM l1=CAR(tmp), l2=CAR(CDR(tmp));
 		  /* Create a rib to avoid that internal def be seen as global  */
-		  env = STk_fast_extend_env(NIL, NIL, env); 
+		  env = add_frame(NIL, NIL, env); 
 		  for ( ; NNULLP(l1); l1=CDR(l1), l2=CDR(l2))
-		    env = STk_fast_extend_env(Cons(CAR(l1), NIL), 
-					      Cons(EVALCAR(l2), NIL), env);
+		    env = add_frame(LIST1(CAR(l1)), LIST1(EVALCAR(l2)), env);
 		  tmp =  CDR(CDR(tmp));
 		  goto Begin;
 		}
@@ -446,7 +483,7 @@ Begin:	   case tc_begin:
 		  /* Make a binding list an extend current with it */
 		  for (len=STk_llength(l1); len; len--) 
 		    bindings=Cons(UNBOUND,bindings);
-		  env = STk_fast_extend_env(l1, bindings, env);
+		  env = add_frame(l1, bindings, env);
 
 		  /* Eval init forms in the new environment */
 		  for (l1 = CAR(tmp); NNULLP(l1); l1=CDR(l1), l2=CDR(l2))
@@ -458,14 +495,17 @@ Begin:	   case tc_begin:
 		}
            case tc_macro:
 	        x = Apply(fct->storage_as.macro.code, x);
+/*FIXME:        x = Apply(fct->storage_as.macro.code, Cons(fct, tmp)); */
+/*		if (fct->storage_as.macro.env != Ntruth) {
+		  printf("FIXME: ==========>R5 macro\n");
+		  env = fct->storage_as.macro.env;
+		}		  
+*/
 	        goto Top;
 	   case tc_quote:
 		RETURN(CAR(tmp));
 	   case tc_lambda:
-		NEWCELL(x, tc_closure);
-		x->storage_as.closure.env  = env;
-		x->storage_as.closure.code = tmp;
-		RETURN(x);
+		RETURN(STk_makeclosure(tmp, env));
 	   case tc_if:
 		x = NEQ(EVALCAR(tmp), Ntruth) ? CAR(CDR(tmp))
 					      : CAR(CDR(CDR(tmp)));
@@ -501,23 +541,21 @@ Begin:	   case tc_begin:
 		RETURN(x);
 	   case tc_extend_env:
 	     	fct = EVALCAR(tmp);
-	        if (NENVP(fct)) Err("extend-env: bad environment", fct);
+	        if (NENVP(fct)) Err("extend-environment: bad environment", fct);
 		tmp = CDR(tmp);
-		env = STk_append(LIST2(fct->storage_as.env.data, env), 2);
+		env = STk_append2(fct->storage_as.env.data, env);
 		goto Begin;
 	   case tc_apply:
+	        if (!len) goto Error_Apply;
 		tmp = eval_args(tmp, env);
 		fct = CAR(tmp);
 		tmp = STk_liststar(CDR(tmp),len-1);
-		if (STk_llength(tmp) == -1) Err("apply: bad parameter list", tmp);
+		len = STk_llength(tmp);
+		if (len == -1) goto Error_Apply;
 
 		switch (TYPE(fct)) {
-		  case tc_closure: env=STk_extend_env(
-		    				CAR(fct->storage_as.closure.code),
-						tmp,
-						fct->storage_as.closure.env,
-						x);
-		    		   tmp = CDR(fct->storage_as.closure.code);
+		  case tc_closure: env = extend_env(fct, tmp, x, len);
+		    		   tmp = CLOSBODY(fct);
 		    		   goto Begin;
 		    case tc_apply: /* Here we are not tail recursive. (i.e. when
 				    * we have something like (apply apply f ...)
@@ -538,6 +576,8 @@ Begin:	   case tc_begin:
 #endif
 		    default: 	     RETURN(Apply(fct, tmp));
 		}
+Error_Apply:
+		Err("apply: bad parameter list", tmp);
            default:
 		if (EXTENDEDP(fct)) {
 		  if (STk_extended_eval_parameters(fct)) 
@@ -562,7 +602,7 @@ Error:
 
 SCM STk_apply(SCM fct, SCM param)
 {
-Top:
+ Top:
   switch TYPE(fct) {
     case tc_subr_0:
          if (NULLP(param)) return SUBR0(fct)();
@@ -600,16 +640,12 @@ Top:
     case tc_lsubr:
 	 return SUBRF(fct)(param, STk_llength(param));
     case tc_cont:
-	 if (STk_llength(param) == 1)
-	   STk_throw(fct, CAR(param));	 
+        STk_throw(fct, param);
     case tc_closure: { 
-         register SCM env = STk_extend_env(CAR(fct->storage_as.closure.code),
-				       param,
-				       fct->storage_as.closure.env,
-				       fct);
 	 register SCM code;
+         register SCM env = extend_env(fct, param, fct, STk_llength(param));
 	 
-	 for(code=CDR(fct->storage_as.closure.code); NNULLP(code); code=CDR(code))
+	 for(code=CLOSBODY(fct); NNULLP(code); code=CDR(code))
 	   param = EVALCAR(code);
 	 return param;
        }
@@ -638,6 +674,147 @@ Top:
   return UNDEFINED; /* never reached */
 }
 
+/* 
+ * same as STk_apply, except that there is no parameter
+ * than a list 
+*/
+SCM STk_apply0(SCM fct)
+{
+  switch TYPE(fct) {
+    case tc_subr_0: 	   return SUBRF(fct)(UNBOUND);
+    case tc_subr_1: 	   break;
+    case tc_subr_2: 	   break;
+    case tc_subr_3: 	   break;
+    case tc_subr_0_or_1:   return SUBRF(fct)(UNBOUND);
+    case tc_subr_1_or_2:   break;
+    case tc_subr_2_or_3:   break;
+    case tc_ssubr:	   return SUBRF(fct)(NIL, NIL, 0);
+    case tc_lsubr:	   return SUBRF(fct)(NIL, 0);
+    case tc_cont:	   STk_throw(fct, NIL);
+    case tc_closure:       {
+      			     register SCM code;
+			     register SCM env = extend_env(fct, NIL, fct, 0);
+			     SCM tmp = NIL;
+
+			     for(code=CLOSBODY(fct); NNULLP(code); code=CDR(code))
+			       tmp = EVALCAR(code);
+			     return tmp;
+    			   }
+#ifdef USE_STKLOS
+    case tc_instance:      return STk_apply_generic(fct, NIL);
+    case tc_next_method:   return STk_apply_next_method(fct, NIL);
+#endif
+#ifdef USE_TK
+    case tc_tkcommand:     return STk_execute_Tcl_lib_cmd(fct, NIL, NIL,0);
+#endif
+    case tc_apply:         return STk_apply(fct, NIL);
+    default:               if (EXTENDEDP(fct))
+      			     if (STk_extended_procedurep(fct)) 
+			       return STk_extended_apply(fct, NIL, UNBOUND);
+			   Err("apply: bad procedure", fct);
+    }
+    
+    Err("apply: bad number of arguments to apply", LIST1(fct));
+    return UNDEFINED; /* never reached */
+}
+
+
+/* 
+ * same as STk_apply, except that param is a single SCM value rather 
+ * than a list 
+*/
+SCM STk_apply1(SCM fct, SCM param)
+{
+    switch TYPE(fct) {
+    case tc_subr_0: 	   break;
+    case tc_subr_1: 	   return SUBRF(fct)(param);
+    case tc_subr_2: 	   break;
+    case tc_subr_3: 	   break;
+    case tc_subr_0_or_1:   return SUBRF(fct)(param);
+    case tc_subr_1_or_2:   return SUBRF(fct)(param, UNBOUND);
+    case tc_subr_2_or_3:   break;
+    case tc_ssubr:	   return SUBRF(fct)(LIST1(param), NIL, 1);
+    case tc_lsubr:	   return SUBRF(fct)(LIST1(param), 1);
+    case tc_cont:	   STk_throw(fct, LIST1(param));
+    case tc_closure:       { 
+      			     register SCM code;
+			     register SCM env = extend_env(fct, LIST1(param), 
+							   fct, 1);
+			     for(code=CLOSBODY(fct); NNULLP(code); code=CDR(code))
+			       param = EVALCAR(code);
+			     return param;
+    			   }
+#ifdef USE_STKLOS
+    case tc_instance:      return STk_apply_generic(fct, LIST1(param));
+    case tc_next_method:   return STk_apply_next_method(fct, LIST1(param));
+#endif
+#ifdef USE_TK
+    case tc_tkcommand:     return STk_execute_Tcl_lib_cmd(fct, LIST1(param),NIL,0);
+#endif
+    case tc_apply:         return STk_apply(fct, LIST1(param));
+    default:               if (EXTENDEDP(fct)) 
+      			     if (STk_extended_procedurep(fct)) 
+			       return STk_extended_apply(fct, LIST1(param), 
+							 UNBOUND);
+			   Err("apply: bad procedure", fct);
+    }
+    
+    Err("apply: bad number of arguments to apply", LIST2(fct, param));
+    return UNDEFINED; /* never reached */
+}
+
+
+/* 
+ * same as STk_apply, except that there are two arguments given explictly,
+ * rather than a list 
+*/
+SCM STk_apply2(SCM fct, SCM param1, SCM param2)
+{
+    switch TYPE(fct) {
+      case tc_subr_0:	   
+      case tc_subr_1:      break;
+      case tc_subr_2:      return SUBRF(fct)(param1, param2);
+      case tc_subr_3:      
+      case tc_subr_0_or_1: break;
+      case tc_subr_1_or_2: return SUBRF(fct)(param1, param2);
+      case tc_subr_2_or_3: return SUBRF(fct)(param1, param2, UNBOUND);
+      case tc_ssubr:	   return SUBRF(fct)(LIST2(param1, param2), NIL, 2);
+      case tc_lsubr:	   return SUBRF(fct)(LIST2(param1, param2), 2);
+      case tc_cont:        STk_throw(fct, LIST2(param1, param2));
+      case tc_closure:     {
+			     register SCM code;	
+			     register SCM env = extend_env(fct, 
+							   LIST2(param1, param2), 
+							   fct, 2);
+			     for(code=CLOSBODY(fct); NNULLP(code); code=CDR(code))
+			       param1 = EVALCAR(code);
+			     return param1;
+       			   }
+#ifdef USE_STKLOS
+      case tc_instance:    return STk_apply_generic(fct, LIST2(param1, param2));
+      case tc_next_method: return STk_apply_next_method(fct, LIST2(param1, param2));
+#endif
+#ifdef USE_TK
+      case tc_tkcommand:   return STk_execute_Tcl_lib_cmd(fct, 
+							  LIST2(param1, param2), 
+							  NIL, 0);
+#endif
+      case tc_apply:        return STk_apply(fct, LIST2(param1, param2));
+      default:  	    if (EXTENDEDP(fct)) 
+			      if (STk_extended_procedurep(fct)) 
+				return STk_extended_apply(fct, 
+							  LIST2(param1, param2), 
+							  UNBOUND);
+			    Err("apply: bad procedure", fct);
+    }
+    
+    Err("apply: bad number of arguments to apply", LIST3(fct, param1, param2));
+    return UNDEFINED; /* never reached */
+}
+
+
+/*=============================================================================*/
+
 
 PRIMITIVE STk_user_eval(SCM expr, SCM env)
 {
