@@ -3,29 +3,34 @@
  *  g c . c			-- Mark and Sweep Garbage Collector 
  *
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
- *
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *            Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 17-Feb-1993 12:27
- * Last file update:  2-Jun-1995 22:09
+ * Last file update:  3-Sep-1999 20:20 (eg)
  *
  */
 
 #include "stk.h"
 #include "gc.h"
+#include "extend.h"
+#ifdef USE_STKLOS
+#  include "stklos.h"
+#endif
+#ifdef USE_TK
+#  include "tk-glue.h"
+#endif
+
 
 #define MIN_HEAP	100	/* A too small value cause an infinite loop */
 #define VALID_ADDRESS(heap_org, p)					    \
@@ -47,34 +52,49 @@ SCM 	STk_freelist;
 SCM 	*STk_stack_start_ptr;
 double  STk_total_gc_time = 0.0;
 long    STk_alloc_cells;
+int	STk_gc_requested = 0;
 
 /* internal vars */
 static jmp_buf	save_regs_gc_mark;
 static long	gc_cells_collected;
-static long	heap_size    = 20000;
-static int	gc_verbose   = 0;
-static int	gc_calls     = 0;
-static SCM 	*heaps 	     = NULL;
-static int 	heaps_length = 0;
-static int	heaps_used   = 0;
+static long	heap_size      = INITIAL_HEAP_SIZE;
+static int	gc_verbose     = 0;
+static int	gc_calls       = 0;
+static SCM 	*heaps 	       = NULL;
+static int 	heaps_length   = 0;
+static int	heaps_used     = 0;
 static double   time_gc_start;
 
 static struct gc_protected  *protected_registers = NULL;
 
+static void no_memory(void)
+{
+  panic("No more memory. Cannot allocate a new heap.");
+}
+
 static void allocate_new_heap(void)
 {
   SCM ptr, next;
   SCM heap_org, heap_end;
 
+  /* Don't use must_malloc here since it can conduct to call GC when 
+   * allocating large heaps (typically with large -cells xxx)
+   */
+
   if (heaps_used == heaps_length) {
     /* Realloc heaps */
     heaps_length += HEAPS_INCREMENT;
-    heaps         = (heaps_used) ?  must_realloc(heaps, heaps_length*sizeof(SCM)):
+    heaps         = (heaps_used) ?  realloc(heaps, heaps_length*sizeof(SCM)):
 				    malloc(heaps_length*sizeof(SCM));
+    if (!heaps) no_memory();
   }
-
-  heap_org = heaps[heaps_used++] = (SCM) must_malloc(sizeof(struct obj)*heap_size);
-  heap_end = heap_org + heap_size;
+  ptr = (SCM) malloc(sizeof(struct obj)*heap_size);
+  if (ptr) {
+    heap_org =  heaps[heaps_used++] = ptr;
+    heap_end = heap_org + heap_size;
+  }
+  else 
+    no_memory();
 
   /* Prepare heap space */
   for(ptr = heap_org, next=ptr+1; ptr < heap_end; ptr=next, next=ptr+1) {
@@ -86,42 +106,11 @@ static void allocate_new_heap(void)
   STk_freelist = heap_org;
 
   if (gc_verbose)
-    fprintf(stderr, ";; [new heap allocated (%d/%d)]\n", heaps_used, heaps_length);
+    Fprintf(STk_curr_eport,
+	    ";; [new heap allocated (%d/%d)]\n",
+	    heaps_used,heaps_length);
 }
 
-static void gc_start(void)
-{
-  time_gc_start       = STk_my_time();
-  gc_calls           += 1;
-  gc_cells_collected  = 0;
-  gc_verbose	      = (VCELL(Intern(GC_VERBOSE)) != Ntruth);
-
-  if (gc_verbose) fprintf(stderr, ";; [starting GC]\n");
-}
-
-static void gc_end(void)
-{
-  long total_cells, used_cells;
-  double time_for_this_gc;
-
-  total_cells = heaps_used * heap_size;
-  used_cells  = total_cells - gc_cells_collected;
-
-  time_for_this_gc   = STk_my_time() - time_gc_start;
-  STk_total_gc_time += time_for_this_gc;
-
-  /* 
-   * If heap is more than 75% filled after gc, allocate a new heap to 
-   * avoid continuous GCs
-   */
-  if (((float) used_cells / total_cells) > 0.75) allocate_new_heap();
-
-  if (gc_verbose) 
-    fprintf(stderr, ";; [end of GC (cells used: %ld/%ld; time: %.2fms)]\n", 
-	    	    used_cells, total_cells, time_for_this_gc);
-}
-
-
 void STk_gc_count_cells(long *allocated, long *used, long* calls)
 {
   register SCM ptr, heap_org, heap_end;
@@ -142,8 +131,6 @@ void STk_gc_count_cells(long *allocated, long *used, long* calls)
 }
 
 
-
-
 int STk_valid_address(SCM p)	/* True if p is a valid address. Used for #Pxyz */
 {
   int i;
@@ -185,8 +172,8 @@ Top:
      case tc_ssubr:	  return;
      case tc_fsubr:	  return;
      case tc_syntax:	  return;
-     case tc_closure:	  gc_mark(ptr->storage_as.closure.code);
-			  ptr = ptr->storage_as.closure.env;
+     case tc_closure:	  gc_mark(CLOSCODE(ptr));
+			  ptr = CLOSENV(ptr);
 			  goto Top;
      case tc_free_cell:	  /* -----> Error */
      case tc_char:	  return;
@@ -199,26 +186,48 @@ Top:
 			  }
      case tc_eof:      	  return;
      case tc_undefined:	  return;
-     case tc_iport:	  return;
-     case tc_oport:	  return;
+     case tc_iport:	  
+     case tc_oport:	  gc_mark(PORT_REVENT(ptr)); 
+			  ptr = PORT_WEVENT(ptr);       
+			  goto Top;
      case tc_isport:	  return;
      case tc_osport:	  return;
+     case tc_ivport:	  
+     case tc_ovport:	  STk_mark_virtual_port(ptr);
+			  return;  
      case tc_boolean:	  return;
      case tc_macro:	  ptr = ptr->storage_as.macro.code; goto Top;
      case tc_localvar:	  ptr = ptr->storage_as.localvar.symbol; goto Top;
      case tc_globalvar:	  ptr = VCELL(ptr); goto Top;
-     case tc_cont:	  ptr = STk_mark_continuation(ptr);
-       			  goto Top;
+     case tc_modulevar:	  ptr = CAR(ptr); goto Top;
+     case tc_cont:	  STk_mark_continuation(ptr);
+       			  return;
      case tc_env:
      case tc_address:	  ptr = ptr->storage_as.env.data;
 			  goto Top;
-     case tc_autoload:	  ptr = CAR(ptr);
-       			  goto Top;
+     case tc_autoload:	  gc_mark(CAR(ptr)); ptr = CDR(ptr); goto Top;
      case tc_Cpointer:    return;
+     case tc_module:      STk_mark_module(ptr); return;
+     case tc_frame:	  gc_mark(CAR(ptr));ptr = CDR(ptr); goto Top;
+     case tc_values:	  ptr = CAR(ptr); goto Top;
+#ifdef USE_STKLOS
+     case tc_instance: 	  {
+       			    /* ACCESSORS_OF(ptr) doesn't need to be marked since it 
+			     * is always accessible from SLOTS_OF(CLASS_OF(ptr))
+			     */
+       			    long j;
+
+			    gc_mark(CLASS_OF(ptr));
+			    for (j = 0; j < NUMBER_OF_SLOTS(ptr); j++)
+			      gc_mark(THE_SLOT_OF(ptr, j));
+			    return;
+			  }
+     case tc_next_method: gc_mark(CAR(ptr)); ptr = CDR(ptr); goto Top;
+#endif
 #ifdef USE_TK
-     case tc_tkcommand:	  gc_mark(ptr->storage_as.tk.l_data); 
-       			  ptr = ptr->storage_as.tk.data->environment;
-       			  goto Top;
+     case tc_tkcommand:	  ptr = ptr->storage_as.tk.l_data; goto Top;
+     case tc_tclobject:	  ptr = CAR(ptr); if (ptr) goto Top;
+       			  return;
 #endif
      case tc_quote:	  return;
      case tc_lambda:	  return;
@@ -240,8 +249,8 @@ Top:
      default:		  if (EXTENDEDP(ptr)) {STk_extended_mark(ptr); return;}
    }
    /* if we are here, it's an implementation error. Signal it */
-   fprintf(stderr, "INTERNAL ERROR: trying to mark %lx (type=%d)\n", 
-	           (unsigned long) ptr, TYPE(ptr));
+   Fprintf(STk_curr_eport, "INTERNAL ERROR: trying to mark %lx (type=%d)\n", 
+	   		   (unsigned long) ptr, TYPE(ptr));
 }
 
 static void gc_sweep(void)
@@ -263,7 +272,7 @@ static void gc_sweep(void)
 	switch (TYPE(ptr)) {
           case tc_nil: 	       break;
 	  case tc_cons:	       break;
-	  case tc_flonum:      free(ptr->storage_as.flonum.data);
+	  case tc_flonum:      break;
 	  case tc_integer:     break;
 	  case tc_bignum:      mpz_clear(BIGNUM(ptr)); free(BIGNUM(ptr)); break;
 	  case tc_symbol:      STk_free_symbol(ptr); break;
@@ -286,27 +295,36 @@ static void gc_sweep(void)
 	  case tc_vector:      free(ptr->storage_as.vector.data); break;
 	  case tc_eof:         break;
 	  case tc_undefined:   break;
-	  case tc_iport:       STk_freeport(ptr); break;
+	  case tc_iport:       
 	  case tc_oport:       STk_freeport(ptr); break;
-	  case tc_isport:      STk_free_string_port(ptr); break;
+	  case tc_isport:      
 	  case tc_osport:      STk_free_string_port(ptr); break;
+	  case tc_ivport:      
+	  case tc_ovport:      STk_free_virtual_port(ptr); break;
 	  case tc_boolean:     break;
 	  case tc_macro:       break;
 	  case tc_localvar:    break;
 	  case tc_globalvar:   break;
+	  case tc_modulevar:   break;
 	  case tc_cont:	       free(ptr->storage_as.cont.data); break;
 	  case tc_env:	       break;
 	  case tc_address:     break;
 	  case tc_autoload:    break;
-	  case tc_Cpointer:    if (!EXTSTATICP(ptr)) free(EXTDATA(ptr));
-			       break;
+	  case tc_Cpointer:    if (!EXTSTATICP(ptr)) free(EXTDATA(ptr)); break;
+	  case tc_module:      STk_free_module(ptr); break;
+	  case tc_frame:       break;
+	  case tc_values:      break;
+#ifdef USE_STKLOS
+	  case tc_instance:    STk_free_instance(ptr); 	break;
+	  case tc_next_method: break;
+#endif
 #ifdef USE_TK
-	  case tc_tkcommand:   if (ptr->storage_as.tk.data->Id[0])
-				/* This object was renamed (rather than deleted) */
-	    			  Tcl_DeleteCommand(STk_main_interp,
-						    ptr->storage_as.tk.data->Id);
-			       free(ptr->storage_as.tk.data); 
+	  case tc_tkcommand:  if (! ptr->storage_as.tk.data->deleted)
+	    			 STk_internal_Tcl_DeleteCommand
+				     (STk_main_interp, ptr->storage_as.tk.data->Id);
+			       free(ptr->storage_as.tk.data);
 			       break;
+	  case tc_tclobject:   break;
 #endif
 	  case tc_quote:       break;
 	  case tc_lambda:      break;
@@ -327,11 +345,10 @@ static void gc_sweep(void)
 	  case tc_unbound:     break;
 	  default:	       if (EXTENDEDP(ptr))
 	    			STk_extended_sweep(ptr);
-	  		       else
-				 fprintf(stderr,
-					 "FATAL ERROR: trying to sweep %lx "
-					 "(type=%d)\n",
-					 (unsigned long) ptr, TYPE(ptr));
+	  		       else 
+				 Fprintf(STk_curr_eport,
+				     "FATAL ERROR: trying to sweep %lx (type=%d)\n",
+				     (unsigned long) ptr, TYPE(ptr));
         }
 	
 	/* Declare this cell free and put it in free list */
@@ -361,8 +378,9 @@ void STk_mark_stack(SCM *start, SCM *end)
   n = end - start;
 
   if (gc_verbose)
-    fprintf(stderr, "[Marking zone <0x%lx->0x%lx> (%ld words)]\n",
+    Fprintf(STk_curr_eport, "[Marking zone <0x%lx->0x%lx> (%ld words)]\n",
 	    (unsigned long) start, (unsigned long) end, (unsigned long) n);
+
   for(j=0; j<n; j++) {
     p = start[j];
     /* if p looks as a SCM pointer mark location */
@@ -382,50 +400,102 @@ static void mark_protected(void)
   
   /* Mark all objects accessible from obarray */
   STk_mark_symbol_table();
+
+  /* Mark the module table */
+  STk_mark_module_table();
+
+  /* Mark the signal table */
+  STk_mark_signal_table();
+
+  /* Mark the table of traced variables */
+  STk_mark_tracevar_table();
+
+#ifdef USE_TK
+  /* Mark all Tcl/Tk callbacks */
+  STk_mark_callbacks();
+#endif
 }
 
 
+static void gc_start(void)
+{
+  time_gc_start       = STk_my_time();
+  gc_calls           += 1;
+  gc_cells_collected  = 0;
+  gc_verbose	      = STk_lookup_variable(GC_VERBOSE, NIL) != Ntruth;
+
+  if (gc_verbose) Puts(";; [starting GC]\n", STk_curr_eport);
+}
+
+static void gc_end(void)
+{
+  long total_cells, used_cells;
+  double time_for_this_gc;
+
+  total_cells = heaps_used * heap_size;
+  used_cells  = total_cells - gc_cells_collected;
+
+  time_for_this_gc   = STk_my_time() - time_gc_start;
+  STk_total_gc_time += time_for_this_gc;
+
+  /* 
+   * If heap is more than 75% filled after gc, allocate a new heap to 
+   * avoid continuous GCs
+   */
+  if (((float) used_cells / total_cells) > 0.75) allocate_new_heap();
+
+  STk_gc_requested = 0;
+
+  if (gc_verbose) 
+    Fprintf(STk_curr_eport, ";; [end of GC (cells used: %ld/%ld; time: %.2fms)]\n", 
+	    used_cells, total_cells, time_for_this_gc);
+}
+
 static void gc_mark_and_sweep(void)
 {
-  SCM stack_end;	/* The topmost variable allocated on stack */
+  long prev_context = STk_err_handler->context;
+  SCM stack_end;     /* The topmost variable (at least a SCM) allocated on stack */
 
+
+  /**** Disallow interrupts while GC'ing because signal handlers may cons */
+  STk_ignore_signals();
+  STk_err_handler->context = ERR_FATAL;
   gc_start();
-  setjmp(save_regs_gc_mark);
-  STk_mark_stack((SCM *) save_regs_gc_mark,
-		 (SCM *) (((char *) save_regs_gc_mark)+sizeof(save_regs_gc_mark)));
-  mark_protected();
-  STk_mark_stack((SCM *) STk_stack_start_ptr, (SCM *) &stack_end);
 
+  /**** Marking phase */
+  setjmp(save_regs_gc_mark);	  				    /* registers */
+  STk_mark_stack((SCM *) save_regs_gc_mark,
+		 (SCM *) (((char *) save_regs_gc_mark)+
+			  sizeof(save_regs_gc_mark)));
+  STk_mark_stack((SCM *) STk_stack_start_ptr, (SCM *) &stack_end);  /* stack */
+  mark_protected();						    /* globals */
+
+  /**** Sweep phase */
   gc_sweep();
+
+  /* Re-allow signals */
   gc_end();
+  STk_err_handler->context = prev_context;
+  STk_allow_signals();
+
+  /* send the pseudo-signal SIGHADGC to say that we had a finished GC'ing */
+  STk_signal_GC();
 }
 
 void STk_gc_for_newcell(void)
 {
-  long flag;
-
-  if (Error_context != ERR_FATAL) {
-    flag = No_interrupt(1);
-    Error_context = ERR_FATAL;
+  if (STk_err_handler->context != ERR_FATAL) {
     gc_mark_and_sweep();
-    Error_context = ERR_OK;
-    No_interrupt(flag);
-    if NNULLP(STk_freelist) return;
+    if (NNULLP(STk_freelist)) return;
   }
-  Err("Out of storage",NIL);
+  panic("Out of storage");
 }
 
 
 PRIMITIVE STk_gc(void)
 {
-  long flag;
-
-  flag      = No_interrupt(1);
-  Error_context = ERR_FATAL;
   gc_mark_and_sweep();
-  Error_context = ERR_OK;
-  No_interrupt(flag);
-  return(UNDEFINED);
+  return UNDEFINED;
 }
 
 void STk_gc_protect(SCM *location)
@@ -439,6 +509,21 @@ void STk_gc_protect(SCM *location)
   protected_registers 	= reg;
 }
 
+void STk_gc_unprotect(SCM *location)
+{
+  struct gc_protected *reg, *prev;
+
+  for (prev=NULL, reg=protected_registers; reg; prev=reg, reg=reg->next) 
+    if (reg->location == location) {
+      if (prev) 
+	prev->next = reg->next;
+      else
+	protected_registers = reg->next;
+      free(reg);
+      return;
+    }
+}
+
 
 PRIMITIVE STk_gc_stats(void)
 {
@@ -463,21 +548,24 @@ PRIMITIVE STk_gc_stats(void)
   }
 
   /* Print statistics */
-  fprintf(stderr, ";; GC statistics\n");
-  fprintf(stderr, ";; -------------\n");
-  fprintf(stderr, ";; cells used %ld/%ld\n", used_cells, heaps_used * heap_size);
-  fprintf(stderr, ";; # of used heaps %d\n", heaps_used);
-  fprintf(stderr, ";; # of GC calls %d (time spent in GC %.2fms)\n", 
-	  	  gc_calls, STk_total_gc_time);
+  Puts(";; GC statistics\n", STk_curr_eport);
+  Puts(";; -------------\n", STk_curr_eport);
+  Fprintf(STk_curr_eport, ";; cells used %ld/%ld\n", used_cells, 
+	  					     heaps_used*heap_size);
+  Fprintf(STk_curr_eport, ";; # of used heaps %d\n", heaps_used); 
+  Fprintf(STk_curr_eport, ";; # of GC calls %d (time spent in GC %.2fms)\n", 
+	  gc_calls, STk_total_gc_time);
 
   for (i=0; i <= tc_stop_extd; i++)
-    if (freq[i]) fprintf(stderr, "(%d %d) ", i,  freq[i]);
-  fprintf(stderr, "\n;;\n");
+    if (freq[i]) Fprintf(STk_curr_eport, "(%d %d) ", i,  freq[i]);
+
+  Puts("\n;;\n", STk_curr_eport);
+  Flush(STk_curr_eport);
  
   return UNDEFINED;
 }
 
-
+#ifdef DEBUG_STK
 PRIMITIVE STk_find_cells(SCM type)
 {
   SCM ptr, z, heap_org, heap_end;
@@ -510,14 +598,15 @@ PRIMITIVE STk_find_cells(SCM type)
   }
   return z;
 }
+#endif
 
 PRIMITIVE STk_expand_heap(SCM arg)
 {
   int i, number_of_heaps, wanted;
 
-  if (NINTEGERP(arg)) Err("expand-heap: bad integer", arg);
+  if (NINTEGERP(arg)) Err("expand-heap: bad value", arg);
 
-  gc_verbose      = VCELL(Intern(GC_VERBOSE))!=Ntruth;
+  gc_verbose      = STk_lookup_variable(GC_VERBOSE, NIL) != Ntruth;
   wanted	  = INTEGER(arg);
   number_of_heaps = (wanted + heap_size - 1) / heap_size;
   
@@ -526,6 +615,10 @@ PRIMITIVE STk_expand_heap(SCM arg)
   return UNDEFINED;
 }
 
+/*=============================================================================*/
+/*				i n i t 				       */
+/*=============================================================================*/
+
 #ifndef max
 #define max(a,b) (((a)<(b))?(b):(a))
 #endif 
