@@ -2,24 +2,22 @@
  *
  * l i s t . c			-- Lists procedures
  *
- * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.  
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??-Oct-1993 21:37
- * Last file update:  7-Nov-1995 21:36
+ * Last file update:  3-Sep-1999 20:21 (eg)
  */
 
 #include "stk.h"
@@ -42,12 +40,14 @@ PRIMITIVE STk_car(SCM x)
 {
   if (TYPEP(x, tc_cons)) return CAR(x);
   Err("car: wrong type of argument", x);
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_cdr(SCM x)
 {
   if (TYPEP(x, tc_cons)) return CDR(x);
   Err("cdr: wrong type of argument", x);
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_setcar(SCM cell, SCM value)
@@ -143,9 +143,10 @@ PRIMITIVE STk_list_length(SCM l)
   int len = STk_llength(l);
   if (len >= 0) return STk_makeinteger((long) len);
   Err("length: not calculable.", NIL);
+  return UNDEFINED; /* never reached */
 }
 
-static SCM append2(SCM l1, SCM l2)
+SCM STk_append2(SCM l1, SCM l2)
 {
   register SCM res, p;
 
@@ -163,13 +164,14 @@ static SCM append2(SCM l1, SCM l2)
       p = CDR(p);
     }
     CAR(p) = CAR(l1);
-    CDR(p) = NIL;		/* Keep alwys a valid list in case of a GC */
+    CDR(p) = NIL;		/* Keep always a valid list in case of a GC */
     if (NCONSP(CDR(l1))) break;
   }
   CDR(p) = l2;
   return res;
 Error: 
-   Err("append: argument is not a list", l1);
+  Err("append: argument is not a list", l1);
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_append(SCM l, int len)
@@ -177,8 +179,8 @@ PRIMITIVE STk_append(SCM l, int len)
   switch (len) {
     case 0:  return NIL;
     case 1:  return CAR(l);
-    case 2:  return append2(CAR(l), CAR(CDR(l)));
-    default: return append2(CAR(l), STk_append(CDR(l), len-1));
+    case 2:  return STk_append2(CAR(l), CAR(CDR(l)));
+    default: return STk_append2(CAR(l), STk_append(CDR(l), len-1));
   }
 }
 
@@ -197,18 +199,21 @@ PRIMITIVE STk_list_tail(SCM list, SCM k)
 {
   register long x;
 
-  if (NCONSP(list)) Err("list-tail: Bad list", list);
+  ENTER_PRIMITIVE("list-tail");
+
+  if (NCONSP(list) && NNULLP(list)) Serror("Bad list", list);
   x = STk_integer_value(k);
   if (x >= 0) {
-    SCM l = list;
+    SCM l;
 
     for (l=list; x > 0; x--) {
-      if (NULLP(l) || NCONSP(l)) Err("list-tail: list too short", list);
+      if (NULLP(l) || NCONSP(l)) Serror("list too short", list);
       l = CDR(l);
     }
     return l;
   }
-  Err("list-tail: index must be exact positive integer", k);
+  Serror("index must be exact positive integer", k);
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_list_ref(SCM list, SCM k)
@@ -230,9 +235,10 @@ PRIMITIVE STk_list_ref(SCM list, SCM k)
     Err("list-ref: list too short", list);
   }
   Err("list-ref: index must be exact positive integer", k);
+  return UNDEFINED; /* never reached */
 }
 
-static SCM lmember(SCM obj, SCM list,  SCM (*predicate)(SCM, SCM) )
+static SCM lmember(SCM obj, SCM list, SCM (*predicate)(SCM, SCM) )
 {
   register SCM ptr;
 	
@@ -244,11 +250,12 @@ static SCM lmember(SCM obj, SCM list,  SCM (*predicate)(SCM, SCM) )
     else 
       /* end of a dotted list */
       return ((*predicate)(ptr, obj) == Truth) ? ptr : Ntruth;
-    if ((ptr=CDR(ptr)) == list) goto Error;
+    if ((ptr=CDR(ptr)) == list) Err("member: circular list", NIL);
   }
   return Ntruth;
 Error:
-  Err("member function: Bad list", list);
+  Err("member: Bad list", list);
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_memq  (SCM obj, SCM list)	{return lmember(obj, list, STk_eq);   }
@@ -267,6 +274,7 @@ static SCM lassoc(SCM obj, SCM alist, SCM (*predicate)(SCM, SCM))
   if (NULLP(l)) return(Ntruth);
 Error:
   Err("assoc function: improper list", alist);
+  return UNDEFINED; /* never reached */
 }
 
 PRIMITIVE STk_assq (SCM obj, SCM alist){return lassoc(obj, alist, STk_eq);   }
@@ -291,3 +299,85 @@ PRIMITIVE STk_copy_tree(SCM l)
 {
   return CONSP(l) ? STk_cons(STk_copy_tree(CAR(l)), STk_copy_tree(CDR(l))): l;
 }
+
+PRIMITIVE STk_last_pair(SCM l)
+{
+  register SCM tmp;
+
+  ENTER_PRIMITIVE("last-pair");
+  if (NCONSP(l)) Serror("bad list", l);
+  for (tmp=l; CONSP(CDR(l)); l = CDR(l))
+    /* Nothing */;
+  return l;
+}
+  
+
+/*
+ * remove functions
+ */
+
+static SCM lremove(SCM obj, SCM list, SCM (*predicate)(SCM, SCM) )
+{
+  register SCM ptr, l;
+  SCM result;
+
+  if (NCONSP(list) && NNULLP(list)) goto Error;
+
+  for (l=list, result=NIL; NNULLP(l); ) {
+    if (NCONSP(l)) goto Error;
+      
+    if ((*predicate)(CAR(l), obj) == Ntruth) {
+      if (NULLP(result)) {
+	NEWCELL(result, tc_cons);
+	ptr = result;
+      }
+      else {
+	NEWCELL(CDR(ptr), tc_cons);
+	ptr = CDR(ptr);
+      }
+      CAR(ptr) = CAR(l);
+      CDR(ptr) = NIL; 	/* Keep always a valid list in case of a GC */
+    }
+    if ((l=CDR(l)) == list) Err("remove: circular list", NIL);
+  }
+  return result;
+Error:
+  Err("remove: Bad list", list);
+  return UNDEFINED; /* never reached */
+}
+
+PRIMITIVE STk_remq  (SCM obj, SCM list)	{return lremove(obj, list, STk_eq);   }
+PRIMITIVE STk_remv  (SCM obj, SCM list)	{return lremove(obj, list, STk_eqv);  }
+PRIMITIVE STk_remove(SCM obj, SCM list)	{return lremove(obj, list, STk_equal);}
+
+/*
+ * 
+ * destructive append (aka append!)
+ *
+ */
+
+SCM STk_dappend2(SCM l1, SCM l2)
+{
+  register SCM tmp;
+
+  if (NULLP(l1)) return l2;
+
+  for (tmp = l1; CONSP(tmp); tmp = CDR(tmp)) {
+    if (NULLP(CDR(tmp))) {
+      CDR(tmp) = l2;
+      return l1;
+    }
+  }
+  Err("append!: argument is not a list", tmp);
+  return UNDEFINED; /* never reached */
+}
+
+PRIMITIVE STk_dappend(SCM l, int len)
+{
+  switch (len) {
+    case 0:  return NIL;
+    case 1:  return CAR(l);
+    case 2:  return STk_dappend2(CAR(l), CAR(CDR(l)));
+    default: return STk_dappend2(CAR(l), STk_dappend(CDR(l), len-1));
+  }
+}
