@@ -2,48 +2,78 @@
  *
  * t o p l e v e l . c				-- The REP loop
  *
- * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1999 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
- *
- * Permission to use, copy, and/or distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that both the above copyright notice and this permission notice appear in
- * all copies and derived works.  Fees for distribution or use of this
- * software or derived works may only be charged with express written
- * permission of the copyright holder.	
- * This software is provided ``as is'' without express or implied warranty.
- *
- * This software is a derivative work of other copyrighted softwares; the
- * copyright notices of these softwares are placed in the file COPYRIGHTS
- *
+ * Permission to use, copy, modify, distribute,and license this
+ * software and its documentation for any purpose is hereby granted,
+ * provided that existing copyright notices are retained in all
+ * copies and that this notice is included verbatim in any
+ * distributions.  No written agreement, license, or royalty fee is
+ * required for any of the authorized uses.
+ * This software is provided ``AS IS'' without express or implied
+ * warranty.
  *
  *	     Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date:  6-Apr-1994 14:46
- * Last file update: 22-Sep-1996 23:48
+ * Last file update:  3-Sep-1999 21:01 (eg)
+ *
+ * Modifications made by Steve Pruitt <steve@pruitt.net> fot Win32
+ * To maintain compatability with Tcl Library paths and
+ * For Win32 support the following changes were made:
+ *     - Defined IMPORT_DLL_GLOBALS for dll support
+ *     - Copyright symbol changed to (c)
  */
 
+/* Import DLL globals from library */ 
+#if defined(WIN32) && defined(USE_DYNLOAD) && defined(MSC_VER) && !defined(CYGWIN32)
+#  define IMPORT_DLL_GLOBALS
+#endif
+
 #include "stk.h"
 #include "gc.h"
+#include "module.h"
 
-/* The cell representing NIL */
-static struct obj VNIL	   = {0, tc_nil};
+#ifndef WIN32
+struct error_handler *STk_err_handler;    /* Current error handler pointer */ 
+#else
+#  if (defined(USE_DYNLOAD) && defined(MSC_VER) && !defined(CYGWIN32))
+     __declspec( dllimport ) struct error_handler *STk_err_handler;
+#  else
+     struct error_handler *STk_err_handler;
+#  endif
+#endif
 
 
+static struct obj VNIL	   = {{NULL}, 0, tc_nil}; /* The cell representing NIL */
+
 static void print_banner(void)
 {
-  if (VCELL(Intern(PRINT_BANNER)) != Ntruth){
-    fprintf(STk_stderr, "Welcome to the STk interpreter version %s [%s]\n", 
+  if (STk_lookup_variable(PRINT_BANNER, NIL) != Ntruth){
+    Fprintf(STk_curr_eport, "Welcome to the STk interpreter version %s [%s]\n", 
 	    STK_VERSION, MACHINE);
-    fprintf(STk_stderr, "Copyright  1993-1996 Erick Gallesio - ");
-    fprintf(STk_stderr, "I3S - CNRS / ESSI <eg@unice.fr>\n");
+#ifdef WIN32
+    /* the copyright symbol does not work on WIN32 system */
+    Fprintf(STk_curr_eport, "Copyright (c) 1993-1999 Erick Gallesio - ");
+#else
+    Fprintf(STk_curr_eport, "Copyright  1993-1999 Erick Gallesio - ");
+#endif
+    Fprintf(STk_curr_eport, "I3S - CNRS / ESSI <eg@unice.fr>\n");
   }
 }
 
 static void weird_dirs(char *argv0)
 {
-  STk_panic("Could not find the directory where STk was installed.\nPerhaps some directories don't exist, or current executable (\"%s\") is in a strange place.\nYou should consider to set the \"STK_LIBRARY\" shell variable.\nExecution is aborted.", argv0);
+  panic("Could not find the directory where STk was installed.\nPerhaps some directories don't exist, or current executable (\"%s\") is in a strange place.\nYou should consider to set the \"STK_LIBRARY\" shell variable.", argv0);
 }
 
+#ifdef USE_TK
+static void no_display(char *argv0)
+{
+  panic("DISPLAY variable is not set. Tk cannot be initialized. Please use command line option ``-no-tk'' when executing \"%s\"", argv0);
+}
+#endif
+
+
 static void load_init_file(void)
 {
   /* Try to load init.stk in "." and, if not present, in $STK_LIBRARY/STk */
@@ -51,20 +81,38 @@ static void load_init_file(void)
   char file[2*MAX_PATH_LENGTH];
 
   sprintf(file, "./%s", init_file);
-  if (STk_loadfile(file, 0) == Truth) return;
+  if (STk_load_file(file, FALSE, STk_selected_module) == Truth) return;
 
   sprintf(file, "%s/STk/%s", STk_library_path, init_file);
-  if (STk_loadfile(file, 0) == Ntruth)
+  if (STk_load_file(file, FALSE, STk_selected_module) == Ntruth)
     weird_dirs(STk_Argv0);
 }
 
+
+static void load_user_init_file(void)
+{
+  /* Try to load .stkrc in "." and, if not present, in $HOME */
+  char init_file[] = ".stkrc";
+  char file[2*MAX_PATH_LENGTH];
+  char *s;
+
+  sprintf(file, "./%s", init_file);
+  if (STk_load_file(file, FALSE, STk_selected_module) == Truth) return;
+
+  if ((s = getenv("HOME")) != NULL) {
+    sprintf(file, "%s/%s", s, init_file);
+    STk_load_file(file, FALSE, STk_selected_module);
+  }
+}
+
+
 static void init_library_path(char *argv0)
 {
   char *s;
 
   STk_library_path = "";
 
-  if (s = getenv("STK_LIBRARY")) {
+  if ((s = getenv("STK_LIBRARY"))) {
     /* Initialize STk_library_path with the content of STK_LIBRARY 
      * shell variable.
      * Make a copy of environment variable (copy is necessary for
@@ -95,13 +143,47 @@ static void init_library_path(char *argv0)
   }
 }
 
+static SCM get_last_defined(char *name)
+{
+  return STk_last_defined;
+}
+
+static void set_last_defined(char *name, SCM val)
+{
+  STk_last_defined = val;
+}
+
+
+/* 
+ * Panic procedure.
+ */
+
+static void panic_proc(char *format, ...) 
+{
+  va_list ap;
+  char buf[1024];
+  char *fmt= "\n**** Fatal error in STk:\n**** %s\n**** ABORT.\n";
+
+  va_start(ap, format);
+  vsprintf(buf, format, ap);
+
+#ifdef WIN32
+  MessageBeep(MB_ICONEXCLAMATION);
+  MessageBox(NULL, buf, "Fatal error in STk", 
+	     MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
+#else
+  if (STk_curr_eport == NULL)
+    fprintf(stderr, fmt, buf);		/* IO are not yet fully initialized */
+  else
+    Fprintf(STk_curr_eport, fmt, buf);
+#endif
+  exit(1);
+}
+
+
+
 static void init_interpreter(void)
 {
-#ifdef WIN32
-  /* First initialize the IO system, to have a console on Windows */
-  STk_init_io();
-#endif
-
   /* Remember if we are running the stk or snow interpreter */
 #ifdef USE_TK
   STk_snow_is_running = FALSE;
@@ -110,10 +192,10 @@ static void init_interpreter(void)
 #endif
 
   /* Global variables to initialize */
-  NIL		   = &VNIL;
   STk_tkbuffer	   = (char *) must_malloc(TKBUFFERN+1);
-  STk_interactivep = STk_arg_interactive ||isatty(fileno(STk_stdin));
-
+  STk_is_safe	   = 0;
+  NIL		   = &VNIL;
+  
   /* Initialize GC */
   STk_init_gc();
 
@@ -122,9 +204,9 @@ static void init_interpreter(void)
   STk_initialize_keyword_table();
 
   /* 
-	* Define some scheme objects used by the interpreter 
-	* and protect them against GC 
-	*/
+   * Define some scheme objects used by the interpreter 
+   * and protect them against GC 
+   */
   NEWCELL(UNDEFINED, tc_undefined); STk_gc_protect(&UNDEFINED);
   NEWCELL(UNBOUND,   tc_unbound);   STk_gc_protect(&UNBOUND);
   NEWCELL(Truth,     tc_boolean);   STk_gc_protect(&Truth);
@@ -147,12 +229,14 @@ static void init_interpreter(void)
    * Otherwise, they will be allocated during a GC and this lead to an infinite 
    * loop
    */
-  VCELL(Intern(GC_VERBOSE))	= Ntruth;
-  VCELL(Intern(REPORT_ERROR))	= NIL;
+  STk_define_variable(GC_VERBOSE,    Ntruth, NIL);
+  STk_define_variable(REPORT_ERROR,  NIL,    NIL);
 
-  VCELL(Intern(LOAD_SUFFIXES))	= NIL;
-  VCELL(Intern(LOAD_PATH))	= NIL;
-  VCELL(Intern(LOAD_VERBOSE))	= Ntruth;
+  /* Initialize standard ports */
+  STk_init_standard_ports();
+
+  /* Initialize module system */
+  STk_init_modules();
 
   /* Initialize the path of the library */
   init_library_path(STk_Argv0);
@@ -160,17 +244,22 @@ static void init_interpreter(void)
   /* Initialize *eval-hook* */
   STk_init_eval_hook();
 
-  /* Initialize standard ports */
-  STk_init_standard_ports();
-
   /* Initialize Scheme primitives */
   STk_init_primitives();
 
   /* Initialize signal table */
   STk_init_signal();
 
-  /* initialize STk_wind_stack and protect it against garbage colection */
-  STk_wind_stack = NIL;	 STk_gc_protect(&STk_wind_stack);
+  /* Define some global variables */
+  STk_define_variable(LOAD_SUFFIXES, NIL,			       NIL);
+  STk_define_variable(LOAD_PATH,     NIL,			       NIL);
+  STk_define_variable(LOAD_VERBOSE,  Ntruth,			       NIL);
+  STk_define_variable(STK_LIBRARY,   STk_makestring(STk_library_path), NIL);
+
+  /* Initialize C variables */
+  STk_last_defined = Ntruth;
+  STk_define_C_variable(LAST_DEFINED, get_last_defined, set_last_defined);
+  STk_gc_protect(&STk_last_defined);
 }
 
 static void finish_initialisation(void)
@@ -180,55 +269,63 @@ static void finish_initialisation(void)
    */
   STk_user_init();
 
-
-#if (defined(USE_TK) && !defined(WIN32))
-  /*  
-   * Set up a handler for characters coming from stdin (Verify that we are
-   * interactive, because background interpreters are locked)
-   */
-  if (STk_interactivep) {
-    Tcl_File f = Tcl_GetFile((ClientData) fileno(STk_stdin),  TCL_UNIX_FD);
-    
-    Tcl_CreateFileHandler(f,
-			  TCL_READABLE,
-			  (Tk_FileProc *) STk_StdinProc, 
-			  (ClientData) NULL);
-  }
-#endif
-  
   /* 
    * See if we have the '-file' option
    */
   if (STk_arg_file) {
     STk_set_signal_handler(STk_makeinteger(SIGINT), Truth);
     STk_interactivep = 0;
-    STk_loadfile(STk_arg_file, TRUE);
+    if (STk_load_file(STk_arg_file, FALSE, STk_selected_module) == Ntruth)
+      panic("Cannot open file \"%s\".", STk_arg_file);
+
 #ifdef USE_TK
-    if (Tk_initialized) Tk_MainLoop();
+    if (Tk_initialized){
+      STk_internal_eval_string("(tk-set-error-handler!)", 0, NIL); /* Do it late */
+      Tk_MainLoop();
+    }
 #endif
     exit(0);
   }
-  
+  else 
+    STk_interactivep = STk_arg_interactive 			|| 
+#ifdef USE_TK
+      		       STk_arg_console 				||
+#endif
+      		       isatty(fileno(PORT_FILE(STk_stdin)));
+      
   /* 
-   * See if we've used the '-interactive' option; if so, 
-   *		- unbufferize stdout and  stderr so that the interpreter can
-   *		  be used with Emacs and 
-   *		- print the STk banner
+   * See if we are interactive:
+   *   1/ Create a console if needed;
+   *   2/ Unbufferize stdout and  stderr so that the interpreter can  be
+   *	  used with Emacs and 
+   *   3/ print the STk banner
+   *   4/ set the input handler if we are on Unix
    */
   if (STk_interactivep) {
     static char *out, *err;
 
+#ifdef USE_TK
+    if (STk_arg_console) STk_init_console();
+#endif
     out = STk_line_bufferize_io(STk_stdout);
     err = STk_line_bufferize_io(STk_stderr);
     print_banner();
+#if (defined(USE_TK) && !defined(WIN32))
+    if (!STk_arg_console) 
+      /* Set up a handler for characters coming from stdin */
+      Tcl_CreateFileHandler(fileno(PORT_FILE(STk_stdin)),
+			    TCL_READABLE,
+			    (Tk_FileProc *) STk_StdinProc, 
+			    (ClientData) NULL);
+#endif
   }
-  fflush(stdout);
+  Flush(STk_curr_oport);
 
   /* 
    * Manage -load option 
    */
   if (STk_arg_load) {
-    STk_loadfile(STk_arg_load, TRUE);
+    STk_load_file(STk_arg_load, TRUE, STk_selected_module);
 #ifdef USE_TK
     if (Tk_initialized) Tcl_GlobalEval(STk_main_interp, "(update)");
 #endif
@@ -240,25 +337,40 @@ static void repl_loop(void)
   /* The print/eval/read loop */
   for( ; ; ) {
     SCM x;
+    SCM env = MOD_ENV(STk_selected_module);
 
     if (STk_interactivep) {
-      fprintf(STk_stderr, "STk> ");
-      fflush(STk_stderr); 
-      fflush(STk_stdout);	/* This is for Ilisp users */
+#ifdef USE_TK
+      if (STk_arg_console) STk_console_prompt(env); else
+#endif
+      if (STk_internal_eval_string("(catch (repl-display-prompt (current-module)))",
+				   0, env) == Truth)
+	Fprintf(STk_curr_eport, "STk> ");
+      Flush(STk_curr_oport); 
+      Flush(STk_curr_eport);	/* This is for Ilisp users */
     }
+
     if (EQ(x=STk_readf(STk_stdin, FALSE), STk_eof_object)) return;
-    x = STk_eval(x, NIL);
+
+    x = STk_eval(x, env);
+
     if (STk_dumped_core) {
       /* 
        * When restoring an image we arrive here x contains the result of applying
        * the saved continuation.
        */
       STk_dumped_core = 0;
-      longjmp(*Top_jmp_buf, JMP_RESTORE);
+      longjmp(STk_err_handler->j, JMP_RESTORE);
     }
     else {
-      STk_print(x, STk_curr_oport, WRT_MODE);
-      Putc('\n', STk_stdout);
+      if (STk_interactivep) {
+	 STk_define_variable("*repl-result*", x, NIL);
+	 if (STk_internal_eval_string("(catch (repl-display-result *repl-result*))",
+				      0, env) == Truth) {
+	   STk_print(x, STk_curr_oport, WRT_MODE);
+	   Putc('\n', STk_curr_oport);
+	 }
+      }
     }
   }
 }
@@ -267,7 +379,18 @@ static void repl_driver(int argc, char **argv)
 {
   static int k;
   static char **new_argv;
-  
+  struct error_handler err_handler;
+
+  /* Inititialize the error handler. */
+  STk_err_handler     	      = &err_handler;
+  err_handler.prev    	      = NULL;
+  err_handler.context 	      = ERR_FATAL;
+  err_handler.dynamic_handler = &VNIL;	/* since NIL is not yet initialized */
+ 
+  /* Initialize IO and set the panic procedure */
+  STk_init_io();
+  Tcl_SetPanicProc(panic_proc);
+
   new_argv = STk_process_argc_argv(argc, argv);
 
   if (STk_arg_image) {
@@ -279,17 +402,17 @@ static void repl_driver(int argc, char **argv)
     STk_reset_eval_stack();
   }
 
-  /* Point where we come back on errors, image restoration, ... */
-  k = setjmp(*Top_jmp_buf);
-  
-  Error_context	     = ERR_OK;	
-  STk_sigint_counter = 0;
-  STk_control_C	     = 0;
+  /* We come back here on errors, image restauration, ... */
+  k = setjmp(err_handler.j); 
+
+  err_handler.context = ERR_OK;	
 
   switch (k) {
     case 0:		init_interpreter();
 			STk_initialize_scheme_args(new_argv);
 			load_init_file();
+			/* And now set the default module to the global one */
+			STk_initialize_stk_module();
 #ifdef USE_TK
 #  ifdef WIN32
 			if (!STk_arg_no_tk)
@@ -301,14 +424,17 @@ static void repl_driver(int argc, char **argv)
 #  else
 			if (!STk_arg_Xdisplay) 
 			  STk_arg_Xdisplay =  getenv("DISPLAY");
-			if (!STk_arg_no_tk && STk_arg_Xdisplay)
+			if (!STk_arg_no_tk) {
+			  if (!STk_arg_Xdisplay) no_display(STk_Argv0);
 			  Tk_main(STk_arg_sync,
 				  STk_arg_name,
 				  STk_arg_file,
 				  STk_arg_Xdisplay,
 				  STk_arg_geometry);
+			}
 #  endif
 #endif
+			load_user_init_file();
 			finish_initialisation();
 			break;
     case JMP_RESTORE:	STk_restore_unix_args_and_environment(&argc, &argv);
@@ -327,13 +453,19 @@ static void repl_driver(int argc, char **argv)
 			break;
     case JMP_THROW:
     case JMP_ERROR:	break;
+    case JMP_INTERRUPT: STk_control_C = 0;
+			STk_err_handler = &err_handler;
+			STk_reset_eval_stack();
+			break;
   }
 
   repl_loop();
-  if (STk_interactivep) fprintf(STk_stderr, "Bye.\n");
+
+  if (STk_interactivep) Fprintf(STk_curr_eport, "Bye.\n");
   STk_quit_interpreter(UNBOUND);
 }
 
+
 /******************************************************************************
  *
  * Toplevel
