@@ -11,10 +11,9 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkCanvPs.c 1.48 96/05/20 13:13:09
+ * SCCS: @(#) tkCanvPs.c 1.57 97/10/28 18:08:39
  */
 
-#include <stdio.h>
 #include "tkInt.h"
 #include "tkCanvas.h"
 #include "tkPort.h"
@@ -59,7 +58,9 @@ typedef struct TkPostscriptInfo {
     char *fileName;		/* Name of file in which to write Postscript;
 				 * NULL means return Postscript info as
 				 * result. Malloc'ed. */
-    FILE *f;			/* Open file corresponding to fileName. */
+    char *channelName;		/* If -channel is specified, the name of
+                                 * the channel to use. */
+    Tcl_Channel chan;		/* Open channel corresponding to fileName. */
     Tcl_HashTable fontTable;	/* Hash table containing names of all font
 				 * families used in output.  The hash table
 				 * values are not used. */
@@ -82,6 +83,8 @@ static Tk_ConfigSpec configSpecs[] = {
 	"", Tk_Offset(TkPostscriptInfo, colorMode), 0},
     {TK_CONFIG_STRING, "-file", (char *) NULL, (char *) NULL,
 	"", Tk_Offset(TkPostscriptInfo, fileName), 0},
+    {TK_CONFIG_STRING, "-channel", (char *) NULL, (char *) NULL,
+	"", Tk_Offset(TkPostscriptInfo, channelName), 0},
     {TK_CONFIG_STRING, "-fontmap", (char *) NULL, (char *) NULL,
 	"", Tk_Offset(TkPostscriptInfo, fontVar), 0},
     {TK_CONFIG_PIXELS, "-height", (char *) NULL, (char *) NULL,
@@ -108,6 +111,319 @@ static Tk_ConfigSpec configSpecs[] = {
 	(char *) NULL, 0, 0}
 };
 
+/*
+ * The prolog data. Generated by str2c from prolog.ps
+ * This was split in small chunks by str2c because
+ * some C compiler have limitations on the size of static strings.
+ */
+static CONST char * CONST  prolog[]= {
+	/* Start of part 1 (2000 characters) */
+	"%%BeginProlog\n\
+50 dict begin\n\
+\n\
+% This is a standard prolog for Postscript generated by Tk's canvas\n\
+% widget.\n\
+% SCCS: @(#) prolog.ps 1.7 96/07/08 17:52:14\n\
+\n\
+% The definitions below just define all of the variables used in\n\
+% any of the procedures here.  This is needed for obscure reasons\n\
+% explained on p. 716 of the Postscript manual (Section H.2.7,\n\
+% \"Initializing Variables,\" in the section on Encapsulated Postscript).\n\
+\n\
+/baseline 0 def\n\
+/stipimage 0 def\n\
+/height 0 def\n\
+/justify 0 def\n\
+/lineLength 0 def\n\
+/spacing 0 def\n\
+/stipple 0 def\n\
+/strings 0 def\n\
+/xoffset 0 def\n\
+/yoffset 0 def\n\
+/tmpstip null def\n\
+\n\
+% Define the array ISOLatin1Encoding (which specifies how characters are\n\
+% encoded for ISO-8859-1 fonts), if it isn't already present (Postscript\n\
+% level 2 is supposed to define it, but level 1 doesn't).\n\
+\n\
+systemdict /ISOLatin1Encoding known not {\n\
+    /ISOLatin1Encoding [\n\
+	/space /space /space /space /space /space /space /space\n\
+	/space /space /space /space /space /space /space /space\n\
+	/space /space /space /space /space /space /space /space\n\
+	/space /space /space /space /space /space /space /space\n\
+	/space /exclam /quotedbl /numbersign /dollar /percent /ampersand\n\
+	    /quoteright\n\
+	/parenleft /parenright /asterisk /plus /comma /minus /period /slash\n\
+	/zero /one /two /three /four /five /six /seven\n\
+	/eight /nine /colon /semicolon /less /equal /greater /question\n\
+	/at /A /B /C /D /E /F /G\n\
+	/H /I /J /K /L /M /N /O\n\
+	/P /Q /R /S /T /U /V /W\n\
+	/X /Y /Z /bracketleft /backslash /bracketright /asciicircum /underscore\n\
+	/quoteleft /a /b /c /d /e /f /g\n\
+	/h /i /j /k /l /m /n /o\n\
+	/p /q /r /s /t /u /v /w\n\
+	/x /y /z /braceleft /bar /braceright /asciitilde /space\n\
+	/space /space /space /space /space /space /space /space\n\
+	/space /space /space /space /space /space /space /space\n\
+	/dotlessi /grave /acute /circumflex /tilde /macron /breve /dotaccent\n\
+	/dieresis /space /ring /cedilla /space /hungarumlaut /ogonek /caron\n\
+	/space /exclamdown /cent /sterling /currency /yen /brokenbar /section\n\
+	/dieresis /copyright /ordfem",
+	/* End of part 1 */
+
+	/* Start of part 2 (2000 characters) */
+	"inine /guillemotleft /logicalnot /hyphen\n\
+	    /registered /macron\n\
+	/degree /plusminus /twosuperior /threesuperior /acute /mu /paragraph\n\
+	    /periodcentered\n\
+	/cedillar /onesuperior /ordmasculine /guillemotright /onequarter\n\
+	    /onehalf /threequarters /questiondown\n\
+	/Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n\
+	/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex\n\
+	    /Idieresis\n\
+	/Eth /Ntilde /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply\n\
+	/Oslash /Ugrave /Uacute /Ucircumflex /Udieresis /Yacute /Thorn\n\
+	    /germandbls\n\
+	/agrave /aacute /acircumflex /atilde /adieresis /aring /ae /ccedilla\n\
+	/egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex\n\
+	    /idieresis\n\
+	/eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide\n\
+	/oslash /ugrave /uacute /ucircumflex /udieresis /yacute /thorn\n\
+	    /ydieresis\n\
+    ] def\n\
+} if\n\
+\n\
+% font ISOEncode font\n\
+% This procedure changes the encoding of a font from the default\n\
+% Postscript encoding to ISOLatin1.  It's typically invoked just\n\
+% before invoking \"setfont\".  The body of this procedure comes from\n\
+% Section 5.6.1 of the Postscript book.\n\
+\n\
+/ISOEncode {\n\
+    dup length dict begin\n\
+	{1 index /FID ne {def} {pop pop} ifelse} forall\n\
+	/Encoding ISOLatin1Encoding def\n\
+	currentdict\n\
+    end\n\
+\n\
+    % I'm not sure why it's necessary to use \"definefont\" on this new\n\
+    % font, but it seems to be important; just use the name \"Temporary\"\n\
+    % for the font.\n\
+\n\
+    /Temporary exch definefont\n\
+} bind def\n\
+\n\
+% StrokeClip\n\
+%\n\
+% This procedure converts the current path into a clip area under\n\
+% the assumption of stroking.  It's a bit tricky because some Postscript\n\
+% interpreters get errors during strokepath for dashed lines.  If\n\
+% this happens then turn off dashes and try again.\n\
+\n\
+/StrokeClip {\n\
+    {strokepath} stopped {\n\
+	(This Postscript printer gets limitcheck overflows when) =\n\
+	(stippling dashed lines;  lines will be printed solid instead.) =\n\
+	[] 0 setdash strokepath} if\n\
+    clip\n\
+} bind def\n\
+\n\
+% d",
+	/* End of part 2 */
+
+	/* Start of part 3 (2000 characters) */
+	"esiredSize EvenPixels closestSize\n\
+%\n\
+% The procedure below is used for stippling.  Given the optimal size\n\
+% of a dot in a stipple pattern in the current user coordinate system,\n\
+% compute the closest size that is an exact multiple of the device's\n\
+% pixel size.  This allows stipple patterns to be displayed without\n\
+% aliasing effects.\n\
+\n\
+/EvenPixels {\n\
+    % Compute exact number of device pixels per stipple dot.\n\
+    dup 0 matrix currentmatrix dtransform\n\
+    dup mul exch dup mul add sqrt\n\
+\n\
+    % Round to an integer, make sure the number is at least 1, and compute\n\
+    % user coord distance corresponding to this.\n\
+    dup round dup 1 lt {pop 1} if\n\
+    exch div mul\n\
+} bind def\n\
+\n\
+% width height string StippleFill --\n\
+%\n\
+% Given a path already set up and a clipping region generated from\n\
+% it, this procedure will fill the clipping region with a stipple\n\
+% pattern.  \"String\" contains a proper image description of the\n\
+% stipple pattern and \"width\" and \"height\" give its dimensions.  Each\n\
+% stipple dot is assumed to be about one unit across in the current\n\
+% user coordinate system.  This procedure trashes the graphics state.\n\
+\n\
+/StippleFill {\n\
+    % The following code is needed to work around a NeWSprint bug.\n\
+\n\
+    /tmpstip 1 index def\n\
+\n\
+    % Change the scaling so that one user unit in user coordinates\n\
+    % corresponds to the size of one stipple dot.\n\
+    1 EvenPixels dup scale\n\
+\n\
+    % Compute the bounding box occupied by the path (which is now\n\
+    % the clipping region), and round the lower coordinates down\n\
+    % to the nearest starting point for the stipple pattern.  Be\n\
+    % careful about negative numbers, since the rounding works\n\
+    % differently on them.\n\
+\n\
+    pathbbox\n\
+    4 2 roll\n\
+    5 index div dup 0 lt {1 sub} if cvi 5 index mul 4 1 roll\n\
+    6 index div dup 0 lt {1 sub} if cvi 6 index mul 3 2 roll\n\
+\n\
+    % Stack now: width height string y1 y2 x1 x2\n\
+    % Below is a doubly-nested for loop to iterate across this area\n\
+    % in units of the stipple pattern size, going up columns then\n\
+    % acr",
+	/* End of part 3 */
+
+	/* Start of part 4 (2000 characters) */
+	"oss rows, blasting out a stipple-pattern-sized rectangle at\n\
+    % each position\n\
+\n\
+    6 index exch {\n\
+	2 index 5 index 3 index {\n\
+	    % Stack now: width height string y1 y2 x y\n\
+\n\
+	    gsave\n\
+	    1 index exch translate\n\
+	    5 index 5 index true matrix tmpstip imagemask\n\
+	    grestore\n\
+	} for\n\
+	pop\n\
+    } for\n\
+    pop pop pop pop pop\n\
+} bind def\n\
+\n\
+% -- AdjustColor --\n\
+% Given a color value already set for output by the caller, adjusts\n\
+% that value to a grayscale or mono value if requested by the CL\n\
+% variable.\n\
+\n\
+/AdjustColor {\n\
+    CL 2 lt {\n\
+	currentgray\n\
+	CL 0 eq {\n\
+	    .5 lt {0} {1} ifelse\n\
+	} if\n\
+	setgray\n\
+    } if\n\
+} bind def\n\
+\n\
+% x y strings spacing xoffset yoffset justify stipple DrawText --\n\
+% This procedure does all of the real work of drawing text.  The\n\
+% color and font must already have been set by the caller, and the\n\
+% following arguments must be on the stack:\n\
+%\n\
+% x, y -	Coordinates at which to draw text.\n\
+% strings -	An array of strings, one for each line of the text item,\n\
+%		in order from top to bottom.\n\
+% spacing -	Spacing between lines.\n\
+% xoffset -	Horizontal offset for text bbox relative to x and y: 0 for\n\
+%		nw/w/sw anchor, -0.5 for n/center/s, and -1.0 for ne/e/se.\n\
+% yoffset -	Vertical offset for text bbox relative to x and y: 0 for\n\
+%		nw/n/ne anchor, +0.5 for w/center/e, and +1.0 for sw/s/se.\n\
+% justify -	0 for left justification, 0.5 for center, 1 for right justify.\n\
+% stipple -	Boolean value indicating whether or not text is to be\n\
+%		drawn in stippled fashion.  If text is stippled,\n\
+%		procedure StippleText must have been defined to call\n\
+%		StippleFill in the right way.\n\
+%\n\
+% Also, when this procedure is invoked, the color and font must already\n\
+% have been set for the text.\n\
+\n\
+/DrawText {\n\
+    /stipple exch def\n\
+    /justify exch def\n\
+    /yoffset exch def\n\
+    /xoffset exch def\n\
+    /spacing exch def\n\
+    /strings exch def\n\
+\n\
+    % First scan through all of the text to find the widest line.\n\
+\n\
+    /lineLength 0 def\n\
+    strings {\n\
+	stringwidth pop\n\
+	dup lineLength gt {/lineLength exch def}",
+	/* End of part 4 */
+
+	/* Start of part 5 (1546 characters) */
+	" {pop} ifelse\n\
+	newpath\n\
+    } forall\n\
+\n\
+    % Compute the baseline offset and the actual font height.\n\
+\n\
+    0 0 moveto (TXygqPZ) false charpath\n\
+    pathbbox dup /baseline exch def\n\
+    exch pop exch sub /height exch def pop\n\
+    newpath\n\
+\n\
+    % Translate coordinates first so that the origin is at the upper-left\n\
+    % corner of the text's bounding box. Remember that x and y for\n\
+    % positioning are still on the stack.\n\
+\n\
+    translate\n\
+    lineLength xoffset mul\n\
+    strings length 1 sub spacing mul height add yoffset mul translate\n\
+\n\
+    % Now use the baseline and justification information to translate so\n\
+    % that the origin is at the baseline and positioning point for the\n\
+    % first line of text.\n\
+\n\
+    justify lineLength mul baseline neg translate\n\
+\n\
+    % Iterate over each of the lines to output it.  For each line,\n\
+    % compute its width again so it can be properly justified, then\n\
+    % display it.\n\
+\n\
+    strings {\n\
+	dup stringwidth pop\n\
+	justify neg mul 0 moveto\n\
+	stipple {\n\
+\n\
+	    % The text is stippled, so turn it into a path and print\n\
+	    % by calling StippledText, which in turn calls StippleFill.\n\
+	    % Unfortunately, many Postscript interpreters will get\n\
+	    % overflow errors if we try to do the whole string at\n\
+	    % once, so do it a character at a time.\n\
+\n\
+	    gsave\n\
+	    /char (X) def\n\
+	    {\n\
+		char 0 3 -1 roll put\n\
+		currentpoint\n\
+		gsave\n\
+		char true charpath clip StippleText\n\
+		grestore\n\
+		char stringwidth translate\n\
+		moveto\n\
+	    } forall\n\
+	    grestore\n\
+	} {show} ifelse\n\
+	0 spacing neg translate\n\
+    } forall\n\
+} bind def\n\
+\n\
+%%EndProlog\n\
+",
+	/* End of part 5 */
+
+	NULL	/* End of data marker */
+};
+
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -145,16 +461,15 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 					 * "postscript". */
 {
     TkPostscriptInfo psInfo, *oldInfoPtr;
-    int result = TCL_ERROR;
+    int result;
     Tk_Item *itemPtr;
+#ifdef BGLK_CODE
+    char string[400+1], *p;
+#else
 #define STRING_LENGTH 400
     char string[STRING_LENGTH+1], *p;
-    char *prologPathParts[2];
+#endif
     time_t now;
-#if !(defined(__WIN32__) || defined(MAC_TCL))
-    struct passwd *pwPtr;
-#endif /* __WIN32__ || MAC_TCL */
-    FILE *f;
     size_t length;
     int deltaX = 0, deltaY = 0;		/* Offset of lower-left corner of
 					 * area to be marked up, measured
@@ -165,8 +480,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 					 * warnings. */
     Tcl_HashSearch search;
     Tcl_HashEntry *hPtr;
-    Tcl_DString buffer, buffer2;
-    char *libDir;
+    Tcl_DString buffer;
+    CONST char * CONST *chunk;
 
     /*
      *----------------------------------------------------------------
@@ -195,7 +510,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     psInfo.colorMode = NULL;
     psInfo.colorLevel = 0;
     psInfo.fileName = NULL;
-    psInfo.f = NULL;
+    psInfo.channelName = NULL;
+    psInfo.chan = NULL;
     psInfo.prepass = 0;
     Tcl_InitHashTable(&psInfo.fontTable, TCL_STRING_KEYS);
     result = Tk_ConfigureWidget(canvasPtr->interp, canvasPtr->tkwin,
@@ -296,20 +612,68 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     }
 
     if (psInfo.fileName != NULL) {
+
+        /*
+         * Check that -file and -channel are not both specified.
+         */
+
+        if (psInfo.channelName != NULL) {
+            Tcl_AppendResult(canvasPtr->interp, "can't specify both -file",
+                    " and -channel", (char *) NULL);
+            result = TCL_ERROR;
+            goto cleanup;
+        }
+
+        /*
+         * Check that we are not in a safe interpreter. If we are, disallow
+         * the -file specification.
+         */
+
+        if (Tcl_IsSafe(canvasPtr->interp)) {
+            Tcl_AppendResult(canvasPtr->interp, "can't specify -file in a",
+                    " safe interpreter", (char *) NULL);
+            result = TCL_ERROR;
+            goto cleanup;
+        }
+        
 	p = Tcl_TranslateFileName(canvasPtr->interp, psInfo.fileName, &buffer);
 	if (p == NULL) {
 	    goto cleanup;
 	}
-	psInfo.f = fopen(p, "w");
+	psInfo.chan = Tcl_OpenFileChannel(canvasPtr->interp, p, "w", 0666);
 	Tcl_DStringFree(&buffer);
-	if (psInfo.f == NULL) {
-	    Tcl_AppendResult(canvasPtr->interp, "couldn't write file \"",
-		    psInfo.fileName, "\": ",
-		    Tcl_PosixError(canvasPtr->interp), (char *) NULL);
+	if (psInfo.chan == NULL) {
 	    goto cleanup;
 	}
     }
 
+    if (psInfo.channelName != NULL) {
+        int mode;
+
+#ifdef SCM_CODE
+        mode = 0;
+#endif
+        /*
+         * Check that the channel is found in this interpreter and that it
+         * is open for writing.
+         */
+#ifdef NYI
+        psInfo.chan = Tcl_GetChannel(canvasPtr->interp, psInfo.channelName,
+                &mode);
+#endif
+        if (psInfo.chan == (Tcl_Channel) NULL) {
+            result = TCL_ERROR;
+            goto cleanup;
+        }
+        if ((mode & TCL_WRITABLE) == 0) {
+            Tcl_AppendResult(canvasPtr->interp, "channel \"",
+                    psInfo.channelName, "\" wasn't opened for writing",
+                    (char *) NULL);
+            result = TCL_ERROR;
+            goto cleanup;
+        }
+    }
+    
     /*
      *--------------------------------------------------------
      * Make a pre-pass over all of the items, generating Postscript
@@ -355,11 +719,13 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     Tcl_AppendResult(canvasPtr->interp, "%!PS-Adobe-3.0 EPSF-3.0\n",
 	    "%%Creator: Tk Canvas Widget\n", (char *) NULL);
 #if !(defined(__WIN32__) || defined(MAC_TCL))
-    pwPtr = getpwuid(getuid());
-    Tcl_AppendResult(canvasPtr->interp, "%%For: ",
-	    (pwPtr != NULL) ? pwPtr->pw_gecos : "Unknown", "\n",
-	    (char *) NULL);
-    endpwent();
+    if (!Tcl_IsSafe(interp)) {
+	struct passwd *pwPtr = getpwuid(getuid());
+	Tcl_AppendResult(canvasPtr->interp, "%%For: ",
+		(pwPtr != NULL) ? pwPtr->pw_gecos : "Unknown", "\n",
+		(char *) NULL);
+	endpwent();
+    }
 #endif /* __WIN32__ || MAC_TCL */
     Tcl_AppendResult(canvasPtr->interp, "%%Title: Window ",
 	    Tk_PathName(canvasPtr->tkwin), "\n", (char *) NULL);
@@ -399,63 +765,14 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     Tcl_AppendResult(canvasPtr->interp, "%%EndComments\n\n", (char *) NULL);
 
     /*
-     * Read a standard prolog file from disk and insert it into
-     * the Postscript.
+     * Insert the prolog
      */
+    for (chunk=prolog; *chunk; chunk++) {
+	Tcl_AppendResult(interp, *chunk, (char *) NULL);
+    }
 
-#ifdef STk_CODE
-    libDir = Tcl_GetVar(canvasPtr->interp, "*stk-library*", TCL_GLOBAL_ONLY);
-#else
-    libDir = Tcl_GetVar(canvasPtr->interp, "tk_library", TCL_GLOBAL_ONLY);
-#endif
-    if (libDir == NULL) {
-	Tcl_ResetResult(canvasPtr->interp);
-	Tcl_AppendResult(canvasPtr->interp, "couldn't find library directory: ",
-#ifdef STk_CODE
-		"*stk-library* variable doesn't exist", (char *) NULL);
-#else
-		"tk_library variable doesn't exist", (char *) NULL);
-#endif
-	goto cleanup;
-    }
-#ifdef STk_CODE
-    sprintf(string, "%.350s/STk/prolog.ps", libDir);
-    f = fopen(string, "r");
-    Tcl_DStringInit(&buffer2);
-    Tcl_DStringAppend(&buffer2, string, -1);
-#else
-    sprintf(string, "%.350s", libDir);
-    Tcl_TranslateFileName(canvasPtr->interp, string, &buffer);
-    prologPathParts[0] = buffer.string;
-    prologPathParts[1] = "prolog.ps";
-    Tcl_DStringInit(&buffer2);
-    Tcl_JoinPath(2, prologPathParts, &buffer2);
-    Tcl_DStringFree(&buffer);
-    f = fopen(buffer2.string, "r");
-#endif
-    if (f == NULL) {
-	Tcl_ResetResult(canvasPtr->interp);
-	Tcl_AppendResult(canvasPtr->interp, "couldn't open prolog file \"",
-		buffer2.string, "\": ", Tcl_PosixError(canvasPtr->interp),
-		(char *) NULL);
-	Tcl_DStringFree(&buffer2);
-	goto cleanup;
-    }
-    Tcl_DStringFree(&buffer2);
-    while (fgets(string, STRING_LENGTH, f) != NULL) {
-	Tcl_AppendResult(canvasPtr->interp, string, (char *) NULL);
-    }
-    if (ferror(f)) {
-	fclose(f);
-	Tcl_ResetResult(canvasPtr->interp);
-	Tcl_AppendResult(canvasPtr->interp, "error reading prolog file \"",
-		string, "\": ",
-		Tcl_PosixError(canvasPtr->interp), (char *) NULL);
-	goto cleanup;
-    }
-    fclose(f);
-    if (psInfo.f != NULL) {
-	fputs(canvasPtr->interp->result, psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	Tcl_ResetResult(canvasPtr->interp);
     }
 
@@ -501,8 +818,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	    psInfo.x, Tk_CanvasPsY((Tk_Canvas) canvasPtr, (double) psInfo.y2));
     Tcl_AppendResult(canvasPtr->interp, string,
 	" lineto closepath clip newpath\n", (char *) NULL);
-    if (psInfo.f != NULL) {
-	fputs(canvasPtr->interp->result, psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	Tcl_ResetResult(canvasPtr->interp);
     }
 
@@ -535,8 +852,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	    goto cleanup;
 	}
 	Tcl_AppendResult(canvasPtr->interp, "grestore\n", (char *) NULL);
-	if (psInfo.f != NULL) {
-	    fputs(canvasPtr->interp->result, psInfo.f);
+	if (psInfo.chan != NULL) {
+	    Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	    Tcl_ResetResult(canvasPtr->interp);
 	}
     }
@@ -550,8 +867,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 
     Tcl_AppendResult(canvasPtr->interp, "restore showpage\n\n",
 	    "%%Trailer\nend\n%%EOF\n", (char *) NULL);
-    if (psInfo.f != NULL) {
-	fputs(canvasPtr->interp->result, psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	Tcl_ResetResult(canvasPtr->interp);
     }
 
@@ -584,8 +901,11 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     if (psInfo.fileName != NULL) {
 	ckfree(psInfo.fileName);
     }
-    if (psInfo.f != NULL) {
-	fclose(psInfo.f);
+    if ((psInfo.chan != NULL) && (psInfo.channelName == NULL)) {
+	Tcl_Close(canvasPtr->interp, psInfo.chan);
+    }
+    if (psInfo.channelName != NULL) {
+        ckfree(psInfo.channelName);
     }
     Tcl_DeleteHashTable(&psInfo.fontTable);
     canvasPtr->psInfoPtr = oldInfoPtr;
@@ -697,28 +1017,19 @@ Tk_CanvasPsColor(interp, canvas, colorPtr)
  */
 
 int
-Tk_CanvasPsFont(interp, canvas, fontStructPtr)
+Tk_CanvasPsFont(interp, canvas, tkfont)
     Tcl_Interp *interp;			/* Interpreter for returning Postscript
 					 * or error message. */
     Tk_Canvas canvas;			/* Information about canvas. */
-    XFontStruct *fontStructPtr;		/* Information about font in which text
+    Tk_Font tkfont;			/* Information about font in which text
 					 * is to be printed. */
 {
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
     TkPostscriptInfo *psInfoPtr = canvasPtr->psInfoPtr;
-    char *name, *end, *weightString, *slantString;
-#define TOTAL_FIELDS	8
-#define FAMILY_FIELD	1
-#define WEIGHT_FIELD	2
-#define SLANT_FIELD	3
-#define SIZE_FIELD	7
-    char *fieldPtrs[TOTAL_FIELDS];
-#define MAX_NAME_SIZE 100
-    char fontName[MAX_NAME_SIZE+50], pointString[20];
-    int i, c, weightSize, nameSize, points;
-    char *p;
-
-    name = Tk_NameOfFontStruct(fontStructPtr);
+    char *end;
+    char pointString[20];
+    Tcl_DString ds;
+    int i, points;
 
     /*
      * First, look up the font's name in the font map, if there is one.
@@ -726,13 +1037,16 @@ Tk_CanvasPsFont(interp, canvas, fontStructPtr)
      * containing font name and size.  Use this information.
      */
 
+    Tcl_DStringInit(&ds);
+    
     if (psInfoPtr->fontVar != NULL) {
 	char *list, **argv;
 	int argc;
 	double size;
+	char *name;
 
-	list = Tcl_GetVar2(interp, psInfoPtr->fontVar,
-		name, 0);
+	name = Tk_NameOfFont(tkfont);
+	list = Tcl_GetVar2(interp, psInfoPtr->fontVar, name, 0);
 	if (list != NULL) {
 	    if (Tcl_SplitList(interp, list, &argc, &argv) != TCL_OK) {
 		badMapEntry:
@@ -748,117 +1062,29 @@ Tk_CanvasPsFont(interp, canvas, fontStructPtr)
 	    if ((size <= 0) || (*end != 0)) {
 		goto badMapEntry;
 	    }
-	    sprintf(pointString, "%.15g", size);
-	    Tcl_AppendResult(interp, "/", argv[0], " findfont ",
-		    pointString, " scalefont ", (char *) NULL);
-	    if (strncasecmp(argv[0], "Symbol", 7) != 0) {
-		Tcl_AppendResult(interp, "ISOEncode ", (char *) NULL);
-	    }
-	    Tcl_AppendResult(interp, "setfont\n", (char *) NULL);
-	    Tcl_CreateHashEntry(&psInfoPtr->fontTable, argv[0], &i);
+
+	    Tcl_DStringAppend(&ds, argv[0], -1);
+	    points = (int) size;
+	    
 	    ckfree((char *) argv);
-	    return TCL_OK;
+	    goto findfont;
 	}
-    }
+    } 
 
-    /*
-     * Not in the font map.  Try to parse the name to get four fields:
-     * family name, weight, slant, and point size.  To do this, split the
-     * font name up into fields, storing pointers to the first character
-     * of each field in fieldPtrs.
-     */
+    points = Tk_PostscriptFontName(tkfont, &ds);
 
-    if (name[0] != '-') {
-	goto error;
-    }
-    for (p =  name+1, i = 0; i < TOTAL_FIELDS; i++) {
-	fieldPtrs[i] = p;
-	while (*p != '-') {
-	    if (*p == 0) {
-		goto error;
-	    }
-	    p++;
-	}
-	p++;
-    }
-
-    /*
-     * Use the information from the X font name to make a guess at a
-     * Postscript font name of the form "<family>-<weight><slant>" where
-     * <weight> and <slant> may be omitted and if both are omitted then
-     * the dash is also omitted.  Postscript is very picky about font names,
-     * so there are several heuristics in the code below (e.g. don't
-     * include a "Roman" slant except for "Times" font, and make sure
-     * that the first letter of each field is capitalized but no other
-     * letters are in caps).
-     */
-
-    nameSize = fieldPtrs[FAMILY_FIELD+1] - 1 - fieldPtrs[FAMILY_FIELD];
-    if ((nameSize == 0) || (nameSize > MAX_NAME_SIZE)) {
-	goto error;
-    }
-    strncpy(fontName, fieldPtrs[FAMILY_FIELD], (size_t) nameSize);
-    if (islower(UCHAR(fontName[0]))) {
-	fontName[0] = toupper(UCHAR(fontName[0]));
-    }
-    for (p = fontName+1, i = nameSize-1; i > 0; p++, i--) {
-	if (isupper(UCHAR(*p))) {
-	    *p = tolower(UCHAR(*p));
-	}
-    }
-    *p = 0;
-    weightSize = fieldPtrs[WEIGHT_FIELD+1] - 1 - fieldPtrs[WEIGHT_FIELD];
-    if (weightSize == 0) {
-	goto error;
-    }
-    if (strncasecmp(fieldPtrs[WEIGHT_FIELD], "medium",
-	    (size_t) weightSize) == 0) {
-	weightString = "";
-    } else if (strncasecmp(fieldPtrs[WEIGHT_FIELD], "bold",
-	    (size_t) weightSize) == 0) {
-	weightString = "Bold";
-    } else {
-	goto error;
-    }
-    if (fieldPtrs[SLANT_FIELD+1] != (fieldPtrs[SLANT_FIELD] + 2)) {
-	goto error;
-    }
-    c = fieldPtrs[SLANT_FIELD][0];
-    if ((c == 'r') || (c == 'R')) {
-	slantString = "";
-	if ((weightString[0] == 0) && (nameSize == 5)
-		&& (strncmp(fontName, "Times", 5) == 0)) {
-	    slantString = "Roman";
-	}
-    } else if ((c == 'i') || (c == 'I')) {
-	slantString = "Italic";
-    } else if ((c == 'o') || (c == 'O')) {
-	slantString = "Oblique";
-    } else {
-	goto error;
-    }
-    if ((weightString[0] != 0) || (slantString[0] != 0)) {
-	sprintf(p, "-%s%s", weightString, slantString);
-    }
-    points = strtoul(fieldPtrs[SIZE_FIELD], &end, 0);
-    if (points == 0) {
-	goto error;
-    }
-    sprintf(pointString, "%.15g", ((double) points)/10.0);
-    Tcl_AppendResult(interp, "/", fontName, " findfont ",
+    findfont:
+    sprintf(pointString, "%d", points);
+    Tcl_AppendResult(interp, "/", Tcl_DStringValue(&ds), " findfont ",
 	    pointString, " scalefont ", (char *) NULL);
-    if (strcmp(fontName, "Symbol") != 0) {
+    if (strncasecmp(Tcl_DStringValue(&ds), "Symbol", 7) != 0) {
 	Tcl_AppendResult(interp, "ISOEncode ", (char *) NULL);
     }
     Tcl_AppendResult(interp, "setfont\n", (char *) NULL);
-    Tcl_CreateHashEntry(&psInfoPtr->fontTable, fontName, &i);
-    return TCL_OK;
+    Tcl_CreateHashEntry(&psInfoPtr->fontTable, Tcl_DStringValue(&ds), &i);
+    Tcl_DStringFree(&ds);
 
-    error:
-    Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp, "couldn't translate font name \"",
-	    name, "\" to Postscript", (char *) NULL);
-    return TCL_ERROR;
+    return TCL_OK;
 }
 
 /*
