@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkConfig.c 1.52 96/02/15 18:52:39
+ * SCCS: @(#) tkConfig.c 1.53 96/04/26 10:29:31
  */
 
 #include "tkPort.h"
@@ -39,25 +39,26 @@ static Tk_ConfigSpec *	FindConfigSpec _ANSI_ARGS_((Tcl_Interp *interp,
 static char *		FormatConfigInfo _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Window tkwin, Tk_ConfigSpec *specPtr,
 			    char *widgRec));
-#ifdef STk_CODE
-static char *		FormatConfigValue _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Window tkwin, Tk_ConfigSpec *specPtr,
-			    char *widgRec, char *buffer,
-			    Tcl_FreeProc **freeProcPtr, int* stringp));
-#else
 static char *		FormatConfigValue _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Window tkwin, Tk_ConfigSpec *specPtr,
 			    char *widgRec, char *buffer,
 			    Tcl_FreeProc **freeProcPtr));
-#endif
 
 #ifdef STk_CODE
+#  define DEFAULT_STK_ENV "#.(global-environment)"
+#endif
+#ifdef BGLK_CODE
+#  define STk_valid_callback 	SCM_valid_callback
+#  define STk_add_callback 	SCM_add_callback
+#endif
+
+#ifdef SCM_CODE
 
 /* This UGLY code is used only for menus items.
- * It saves in the static variable menu_addr the addresse of the menu item
+ * It saves in the static variable menu_addr the address of the menu item
  * we are configuring. This addresse is necessary for storing the closure
  * associated to the "command" of a menu item. Address will be used to associate 
- * a unique signature to this item
+ * a unique signature to this item.
  */
 
 static void *menu_addr = NULL;
@@ -404,20 +405,43 @@ DoConfig(interp, tkwin, specPtr, value, valueIsUid, widgRec)
 		}
 		break;
 #ifdef STk_CODE
+	    case TK_CONFIG_ENV: {
+	      char buffer[50], *s = "";		
+	      SCM p;
+		
+		if (*value) {
+		  if (!STk_valid_environment(value, &p)) {
+		    Tcl_AppendResult(interp, "bad environment specification \"",
+				             value, "\"", (char *) NULL);
+		    return TCL_ERROR;
+		  }
+		  if (p != NULL) {
+		    if (menu_addr != NULL) {
+		      sprintf(buffer, "%lx", (unsigned long) menu_addr);
+		      s = buffer;
+		    }
+
+		    /* add this environment to the callback table */
+		    STk_add_callback(Tk_PathName(tkwin), specPtr->argvName, s, p);
+		  }
+		}
+		goto String;
+	    }
+#endif
+#ifdef SCM_CODE
 	    case TK_CONFIG_CLOSURE: {
 	        char buffer[50], *s = "";
 		SCM p;
 
 		if (*value) {
 		  if (!STk_valid_callback(value, &p)) {
-		  BadSpec:	
 		    Tcl_AppendResult(interp, "bad closure specification \"",
 				             value, "\"", (char *) NULL);
 		    return TCL_ERROR;
 		  }
 		  if (p != NULL) {
 		    if (menu_addr != NULL) {
-		      sprintf(buffer, "%x", menu_addr);
+		      sprintf(buffer, "%lx", (unsigned long) menu_addr);
 		      s = buffer;
 		    }
 		    
@@ -428,9 +452,10 @@ DoConfig(interp, tkwin, specPtr, value, valueIsUid, widgRec)
 	    }
 	    /* NOBREAK */
 	    /* And now continue to register this command as a string */
-	    case TK_CONFIG_SINT:
+String:	    case TK_CONFIG_SINT:
 	    case TK_CONFIG_SBOOLEAN:
 	    case TK_CONFIG_BSTRING:
+	    case TK_CONFIG_IMAGE:
 #endif
 	    case TK_CONFIG_STRING: {
 		char *old, *new;
@@ -438,7 +463,11 @@ DoConfig(interp, tkwin, specPtr, value, valueIsUid, widgRec)
 		if (nullValue) {
 		    new = NULL;
 		} else {
+#ifdef BGLK_CODE
+		    new = ckalloc_atomic((unsigned) (strlen(value) + 1));
+#else
 		    new = (char *) ckalloc((unsigned) (strlen(value) + 1));
+#endif
 		    strcpy(new, value);
 		}
 		old = *((char **) ptr);
@@ -448,17 +477,22 @@ DoConfig(interp, tkwin, specPtr, value, valueIsUid, widgRec)
 		*((char **) ptr) = new;
 		break;
 	    }
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    case TK_CONFIG_MENU: {
 		char *old, *new;
 
 		if (nullValue) {
 		    new = NULL;
 		} else {
+#ifdef BGLK_CODE
+		    new = ckalloc_atomic((unsigned) (strlen(value) + 1));
+		    strcpy(new, value);
+#else
 		    new = (char *) ckalloc((unsigned) (strlen(value) + 3));
 		    new[0] = '#';
 		    new[1] = '.';
 		    strcpy(new+2, value);
+#endif
 		}
 		old = *((char **) ptr);
 		if (old != NULL) {
@@ -497,22 +531,18 @@ DoConfig(interp, tkwin, specPtr, value, valueIsUid, widgRec)
 		break;
 	    }
 	    case TK_CONFIG_FONT: {
-		XFontStruct *newPtr, *oldPtr;
+		Tk_Font new;
 
 		if (nullValue) {
-		    newPtr = NULL;
+		    new = NULL;
 		} else {
-		    uid = valueIsUid ? (Tk_Uid) value : Tk_GetUid(value);
-		    newPtr = Tk_GetFontStruct(interp, tkwin, uid);
-		    if (newPtr == NULL) {
+		    new = Tk_GetFont(interp, tkwin, value);
+		    if (new == NULL) {
 			return TCL_ERROR;
 		    }
 		}
-		oldPtr = *((XFontStruct **) ptr);
-		if (oldPtr != NULL) {
-		    Tk_FreeFontStruct(oldPtr);
-		}
-		*((XFontStruct **) ptr) = newPtr;
+		Tk_FreeFont(*((Tk_Font *) ptr));
+		*((Tk_Font *) ptr) = new;
 		break;
 	    }
 	    case TK_CONFIG_BITMAP: {
@@ -697,7 +727,7 @@ Tk_ConfigureInfo(interp, tkwin, specs, widgRec, argvName, flags)
     register Tk_ConfigSpec *specPtr;
     int needFlags, hateFlags;
     char *list;
-#ifdef STk_CODE
+#ifdef SCM_CODE
     char *leader = "(";
 #else
     char *leader = "{";
@@ -731,7 +761,9 @@ Tk_ConfigureInfo(interp, tkwin, specs, widgRec, argvName, flags)
      * Loop through all the specs, creating a big list with all
      * their information.
      */
-
+#ifdef SCM_CODE
+    Tcl_AppendResult(interp, "(", NULL);
+#endif
     for (specPtr = specs; specPtr->type != TK_CONFIG_END; specPtr++) {
 	if ((argvName != NULL) && (specPtr->argvName != argvName)) {
 	    continue;
@@ -744,7 +776,7 @@ Tk_ConfigureInfo(interp, tkwin, specs, widgRec, argvName, flags)
 	    continue;
 	}
 	list = FormatConfigInfo(interp, tkwin, specPtr, widgRec);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, leader, list, ")", (char *) NULL);
 	ckfree(list);
 	leader = " (";
@@ -754,6 +786,9 @@ Tk_ConfigureInfo(interp, tkwin, specs, widgRec, argvName, flags)
 	leader = " {";
 #endif
     }
+#ifdef SCM_CODE
+    Tcl_AppendResult(interp, ")", NULL);
+#endif
     return TCL_OK;
 }
 
@@ -786,10 +821,10 @@ FormatConfigInfo(interp, tkwin, specPtr, widgRec)
 					 * values of info for widget. */
 {
     char *argv[6], *result;
-#ifdef STk_CODE
+#ifdef SCM_CODE
 #   define MAX_BUFFER 200
     char buffer[MAX_BUFFER], dflt[MAX_BUFFER];
-    int len, stringp;
+    size_t len;
 #else
     char buffer[200];
 #endif
@@ -800,42 +835,65 @@ FormatConfigInfo(interp, tkwin, specPtr, widgRec)
     argv[2] = specPtr->dbClass;
     argv[3] = specPtr->defValue;
     if (specPtr->type == TK_CONFIG_SYNONYM) {
-#ifdef STk_CODE
-      result = ckalloc(strlen(argv[0]) + strlen(argv [1]) + 4);
+#ifdef SCM_CODE
+#ifdef BGLK_CODE
+      result = (char *)ckalloc_atomic(strlen(argv[0]) + strlen(argv [1]) + 4);
+#else
+      result = (char *)ckalloc(strlen(argv[0]) + strlen(argv [1]) + 4);
+#endif
       sprintf(result, ":%s \"%s\"", argv[0]+1, argv [1]);
       return result;
 #else
 	return Tcl_Merge(2, argv);
 #endif
     }
-#ifdef STk_CODE
-    argv[4] = FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer,
-	    &freeProc, &stringp);
-#else
     argv[4] = FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer,
 	    &freeProc);
-#endif
     if (argv[1] == NULL) {
 	argv[1] = "";
     }
     if (argv[2] == NULL) {
 	argv[2] = "";
     }
-#ifdef STk_CODE
+#ifdef SCM_CODE
     /* 
      * Default value of an option (the one at index 3) can contain weird 
      * characters (e.g. fonts can contain '#'). Quote it if necessary.
      */
-    if (argv[3] == NULL) {
-	argv[3] = "\"\"";
-    }
-    else if (stringp) {
-      sprintf(dflt, "\"%s\"", argv[3]);
-      argv[3] = dflt;		
+
+    switch (specPtr->type) {
+      case TK_CONFIG_BOOLEAN:
+      case TK_CONFIG_SBOOLEAN:
+      case TK_CONFIG_INT:
+      case TK_CONFIG_DOUBLE:
+      case TK_CONFIG_SINT:
+      case TK_CONFIG_MENU:
+      case TK_CONFIG_PIXELS:
+      case TK_CONFIG_MM:
+      case TK_CONFIG_CUSTOM:   if (argv[3] == NULL)
+				 argv[3] = "\"\"";
+      			       break;
+#ifdef STk_CODE
+      case TK_CONFIG_ENV:      /* The Default environment is always the global 1 */
+			       argv[3] = DEFAULT_STK_ENV;
+			       break;
+#endif
+      case TK_CONFIG_IMAGE:    
+      case TK_CONFIG_CLOSURE:  if (argv[3] == NULL || argv[3][0] == '\0')
+				  argv[3] = "\"\"";
+	                       break;
+      default: 		       if (argv[3] == NULL)
+				 argv[3] = "\"\"";
+      			       else {
+				 sprintf(dflt, "\"%s\"", argv[3]);
+				 argv[3] = dflt;
+			       }
     }
+
     if (argv[4] == NULL) {
 	argv[4] = "\"\"";
-    }  
+    }
+
     len = strlen(argv[0])+strlen(argv[1])+strlen(argv[2])+
           strlen(argv[3])+strlen(argv[4]) + 9; /* 4 spaces, 4 quotes  and a null */
 
@@ -844,7 +902,11 @@ FormatConfigInfo(interp, tkwin, specPtr, widgRec)
      * Qutotication is a little bit simplistic here, but those parameters 
      * are identifier (a priori)
      */
-    result = ckalloc(len);      
+#ifdef BGLK_CODE
+    result = (char *)ckalloc_atomic(len);
+#else
+    result = (char *)ckalloc(len);
+#endif
     sprintf(result, ":%s \"%s\" \"%s\" %s %s", 
 	    	    argv[0]+1,argv[1],argv[2],argv[3],argv[4]);
 #else
@@ -857,7 +919,11 @@ FormatConfigInfo(interp, tkwin, specPtr, widgRec)
     result = Tcl_Merge(5, argv);
 #endif
     if (freeProc != NULL) {
+#ifdef BGLK_CODE
+	if ((freeProc == TCL_DYNAMIC) || (freeProc == (Tcl_FreeProc *) GC_free)) {
+#else
 	if ((freeProc == TCL_DYNAMIC) || (freeProc == (Tcl_FreeProc *) free)) {
+#endif
 	    ckfree(argv[4]);
 	} else {
 	    (*freeProc)(argv[4]);
@@ -889,11 +955,7 @@ FormatConfigInfo(interp, tkwin, specPtr, widgRec)
  */
 
 static char *
-#ifndef STk_CODE
 FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr)
-#else
-FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
-#endif
     Tcl_Interp *interp;		/* Interpreter for use in real conversions. */
     Tk_Window tkwin;		/* Window corresponding to widget. */
     Tk_ConfigSpec *specPtr;	/* Pointer to information describing option.
@@ -905,24 +967,15 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
     Tcl_FreeProc **freeProcPtr;	/* Pointer to word to fill in with address
 				 * of procedure to free the result, or NULL
 				 * if result is static. */
-#ifdef STk_CODE
-    int *stringp;		/* 1 if the default value of this configuration 
-				 * option is a string
-				 */
-#endif
 {
     char *ptr, *result;
 
-#ifdef STk_CODE
-    *stringp = 1;
-#endif
     *freeProcPtr = NULL;
     ptr = widgRec + specPtr->offset;
     result = "";
     switch (specPtr->type) {
 	case TK_CONFIG_BOOLEAN:
-#ifdef STk_CODE
-	    *stringp = 0;
+#ifdef SCM_CODE
 	    /* NO BREAK */
         case TK_CONFIG_SBOOLEAN:
 	    return (*((int *) ptr) == 0) ? "#f" : "#t";
@@ -936,8 +989,7 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 #endif
 	case TK_CONFIG_INT:
 	    sprintf(buffer, "%d", *((int *) ptr));
-#ifdef STk_CODE
-	    *stringp = 0;
+#ifdef SCM_CODE
 	    return buffer;
 #else
 	    result = buffer;
@@ -945,24 +997,29 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 #endif
 	case TK_CONFIG_DOUBLE:
 	    Tcl_PrintDouble(interp, *((double *) ptr), buffer);
-#ifdef STk_CODE
-	    *stringp = 0;
+#ifdef SCM_CODE
 	    return buffer;
 #else
 	    result = buffer;
 	    break;
 #endif
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	case TK_CONFIG_SINT:
-	    *stringp = 0;
 	    result   = (*(char **) ptr);
 	    if (result == NULL) result = "0";
 	    return result;
         case TK_CONFIG_MENU:
-	    *stringp = 0;
 	    result   = (*(char **) ptr);
 	    if (result == NULL) result = "#f";
 	    return result;
+#   ifdef STk_CODE
+        case TK_CONFIG_ENV:
+	    result = (*(char **) ptr);
+	    if (result == NULL || *result == 0) {
+	      result = DEFAULT_STK_ENV;
+	    }
+	    return result;
+#   endif
         case TK_CONFIG_CLOSURE:
 	    result = (*(char **) ptr);
 	    if (result == NULL) {
@@ -970,11 +1027,23 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 	      break;
 	    }
 	    else {
-	      *stringp = 0;
 	      return result;
 	    }
+#   ifdef STk_CODE
+        case TK_CONFIG_IMAGE:
+	    result = (*(char **) ptr);
+	    if (result == NULL)
+	      return  "\"\"";
+	    else {
+	      sprintf(buffer, "#.|%s|", result);
+	      return  buffer;
+	    }
+#   endif
+#   ifdef BGLK_CODE
+        case TK_CONFIG_IMAGE:
+	    /* NO BREAK */
+#   endif	    
         case TK_CONFIG_BSTRING:
-	    *stringp = 0;
 	    /* NO BREAK */
 #endif
 	case TK_CONFIG_STRING:
@@ -998,9 +1067,9 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 	    break;
 	}
 	case TK_CONFIG_FONT: {
-	    XFontStruct *fontStructPtr = *((XFontStruct **) ptr);
-	    if (fontStructPtr != NULL) {
-		result = Tk_NameOfFontStruct(fontStructPtr);
+	    Tk_Font tkfont = *((Tk_Font *) ptr);
+	    if (tkfont != NULL) {
+		result = Tk_NameOfFont(tkfont);
 	    }
 	    break;
 	}
@@ -1043,9 +1112,7 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 	    break;
 	case TK_CONFIG_PIXELS:
 	    sprintf(buffer, "%d", *((int *) ptr));
-#ifdef STk_CODE
-	    *stringp = 0; /* most of the time correct "2c" will be seen as 
-			   * the symbol 2c but this should be rare */
+#ifdef SCM_CODE
 	    return buffer;
 #else
 	    result = buffer;
@@ -1053,8 +1120,7 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 #endif
 	case TK_CONFIG_MM:
 	    Tcl_PrintDouble(interp, *((double *) ptr), buffer);
-#ifdef STk_CODE
-	    *stringp = 0;
+#ifdef SCM_CODE
 	    return buffer;
 #else
 	    result = buffer;
@@ -1073,22 +1139,21 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 	    result = (*specPtr->customPtr->printProc)(
 		    specPtr->customPtr->clientData, tkwin, widgRec,
 		    specPtr->offset, freeProcPtr);
-#ifdef STk_CODE
-	    *stringp = 0;
+#ifdef SCM_CODE
 	    return result;
 #endif
 	    break;
 	default: 
 	    result = "?? unknown type ??";
     }
-#ifdef STk_CODE
+#ifdef SCM_CODE
     /* result contain a value which must be converted to a string */
     if (result == NULL) {
       return  "\"\"";
     }
     else {
       register char *s, *d;
-      int len;
+      size_t len;
       char *r      	    = buffer;
       Tcl_FreeProc *oldFree = *freeProcPtr;
 
@@ -1096,8 +1161,13 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
       len          = strlen(result);
 
       if (len > (200/2)-3) {
-	r = ckalloc(len * 2 + 3); /* worst overestimation */
+#ifdef BGLK_CODE
+	r = (char *)ckalloc_atomic(len * 2 + 3); /* worst overestimation */
+	*freeProcPtr = (Tcl_FreeProc *) GC_free;
+#else
+	r = (char *)ckalloc(len * 2 + 3); /* worst overestimation */
 	*freeProcPtr = (Tcl_FreeProc *) free;
+#endif
       }
 
       d = r; *d++ = '"';
@@ -1114,7 +1184,11 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr, stringp)
 	 * In this case result points something which must be unallocated
 	 *
 	 */
+#ifdef BGLK_CODE
+	if (oldFree == (Tcl_FreeProc *) GC_free) {
+#else
 	if (oldFree == (Tcl_FreeProc *) free) {
+#endif	   
 	    ckfree(result);
 	} else {
 	    (*oldFree)(result);
@@ -1162,9 +1236,6 @@ Tk_ConfigureValue(interp, tkwin, specs, widgRec, argvName, flags)
 {
     Tk_ConfigSpec *specPtr;
     int needFlags, hateFlags;
-#ifdef STk_CODE
-    int dumb;
-#endif
 
     needFlags = flags & ~(TK_CONFIG_USER_BIT - 1);
     if (Tk_Depth(tkwin) <= 1) {
@@ -1176,13 +1247,8 @@ Tk_ConfigureValue(interp, tkwin, specs, widgRec, argvName, flags)
     if (specPtr == NULL) {
 	return TCL_ERROR;
     }
-#ifdef STk_CODE
-    interp->result = FormatConfigValue(interp, tkwin, specPtr, widgRec,
-	    interp->result, &interp->freeProc, &dumb);
-#else
     interp->result = FormatConfigValue(interp, tkwin, specPtr, widgRec,
 	    interp->result, &interp->freeProc);
-#endif
     return TCL_OK;
 }
 
@@ -1226,11 +1292,15 @@ Tk_FreeOptions(specs, widgRec, display, needFlags)
 	ptr = widgRec + specPtr->offset;
 	switch (specPtr->type) {
 #ifdef STk_CODE
+	    case TK_CONFIG_ENV:
+#endif
+#ifdef SCM_CODE
 	    case TK_CONFIG_CLOSURE:
 	    case TK_CONFIG_MENU:
 	    case TK_CONFIG_SINT:
 	    case TK_CONFIG_SBOOLEAN:
 	    case TK_CONFIG_BSTRING:
+	    case TK_CONFIG_IMAGE:
 #endif
 	    case TK_CONFIG_STRING:
 		if (*((char **) ptr) != NULL) {
@@ -1245,10 +1315,8 @@ Tk_FreeOptions(specs, widgRec, display, needFlags)
 		}
 		break;
 	    case TK_CONFIG_FONT:
-		if (*((XFontStruct **) ptr) != NULL) {
-		    Tk_FreeFontStruct(*((XFontStruct **) ptr));
-		    *((XFontStruct **) ptr) = NULL;
-		}
+		Tk_FreeFont(*((Tk_Font *) ptr));
+		*((Tk_Font *) ptr) = NULL;
 		break;
 	    case TK_CONFIG_BITMAP:
 		if (*((Pixmap *) ptr) != None) {
