@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkGrab.c 1.50 96/02/15 18:53:34
+ * SCCS: @(#) tkGrab.c 1.52 97/03/21 11:14:34
  */
 
 #include "tkPort.h"
@@ -200,7 +200,7 @@ Tk_GrabCmd(clientData, interp, argc, argv)
     if (argc < 2) {
 	badArgs:
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
-#ifdef STk_CODE
+#ifdef SCM_CODE
 		argv[0], " ?:global? window\" or \"", argv[0],
 #else
 		argv[0], " ?-global? window\" or \"", argv[0],
@@ -248,12 +248,12 @@ Tk_GrabCmd(clientData, interp, argc, argv)
 		interp->result = dispPtr->eventualGrabWinPtr->pathName;
 #endif
 	    }
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    else
 	      interp->result = "#f";
 #endif
 	} else {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, "(", (char *) NULL);
 #endif
 	    for (dispPtr = tkDisplayList; dispPtr != NULL;
@@ -269,7 +269,7 @@ Tk_GrabCmd(clientData, interp, argc, argv)
 #endif
 		}
 	    }
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, ")", (char *) NULL);
 #endif
 	}
@@ -290,7 +290,7 @@ Tk_GrabCmd(clientData, interp, argc, argv)
 	    && (length >= 2)) {
 	if ((argc != 3) && (argc != 4)) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-#ifdef STk_CODE
+#ifdef SCM_CODE
 		    argv[0], " set ?:global? window\"", (char *) NULL);
 #else
 		    argv[0], " set ?-global? window\"", (char *) NULL);
@@ -305,7 +305,7 @@ Tk_GrabCmd(clientData, interp, argc, argv)
 	    length = strlen(argv[2]);
 	    if ((strncmp(argv[2], "-global", length) != 0) || (length < 2)) {
 		Tcl_AppendResult(interp, "bad argument \"", argv[2],
-#ifdef STk_CODE
+#ifdef SCM_CODE
 			"\": must be \"", argv[0], " set ?:global? window\"",
 #else
 			"\": must be \"", argv[0], " set ?-global? window\"",
@@ -334,7 +334,7 @@ Tk_GrabCmd(clientData, interp, argc, argv)
 	    return TCL_ERROR;
 	}
 	dispPtr = winPtr->dispPtr;
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	if (dispPtr->eventualGrabWinPtr != winPtr) {
 	    interp->result = "\"none\"";
 	} else if (dispPtr->grabFlags & GRAB_GLOBAL) {
@@ -394,7 +394,7 @@ Tk_Grab(interp, tkwin, grabGlobal)
 					 * Zero means the grab only applies
 					 * within this application. */
 {
-    int grabResult;
+    int grabResult, numTries;
     TkWindow *winPtr = (TkWindow *) tkwin;
     TkDisplay *dispPtr = winPtr->dispPtr;
     TkWindow *winPtr2;
@@ -449,10 +449,28 @@ Tk_Grab(interp, tkwin, grabGlobal)
 
 	XUngrabPointer(dispPtr->display, CurrentTime);
 	serial = NextRequest(dispPtr->display);
-	grabResult = XGrabPointer(dispPtr->display, winPtr->window,
-		True, ButtonPressMask|ButtonReleaseMask|ButtonMotionMask
-		|PointerMotionMask, GrabModeAsync, GrabModeAsync, None,
-		None, CurrentTime);
+
+	/*
+	 * Another tricky point: there are races with some window
+	 * managers that can cause grabs to fail because the window
+	 * manager hasn't released its grab quickly enough.  To work
+	 * around this problem, retry a few times after AlreadyGrabbed
+	 * errors to give the grab release enough time to register with
+	 * the server.
+	 */
+
+	grabResult = 0;			/* Needed only to prevent gcc
+					 * compiler warnings. */
+	for (numTries = 0; numTries < 10; numTries++) {
+	    grabResult = XGrabPointer(dispPtr->display, winPtr->window,
+		    True, ButtonPressMask|ButtonReleaseMask|ButtonMotionMask
+		    |PointerMotionMask, GrabModeAsync, GrabModeAsync, None,
+		    None, CurrentTime);
+	    if (grabResult != AlreadyGrabbed) {
+		break;
+	    }
+	    Tcl_Sleep(100);
+	}
 	if (grabResult != 0) {
 	    grabError:
 	    if (grabResult == GrabNotViewable) {
@@ -1230,9 +1248,9 @@ EatGrabEvents(dispPtr, serial)
 
     info.display = dispPtr->display;
     info.serial = serial;
-    XSync(dispPtr->display, False);
+    TkpSync(info.display);
     oldProc = Tk_RestrictEvents(GrabRestrictProc, (ClientData)&info, &oldArg);
-    while (Tcl_DoOneEvent(TCL_DONT_WAIT|TCL_WINDOW_EVENTS)) {
+    while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {
     }
     Tk_RestrictEvents(oldProc, oldArg, &dummy);
 }
