@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkPack.c 1.63 96/02/15 18:52:33
+ * SCCS: @(#) tkPack.c 1.64 96/05/03 10:51:52
  */
 
 #include "tkPort.h"
@@ -131,6 +131,7 @@ static Tk_GeomMgr packerType = {
 static void		ArrangePacking _ANSI_ARGS_((ClientData clientData));
 static int		ConfigureSlaves _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Window tkwin, int argc, char *argv[]));
+static void             DestroyPacker _ANSI_ARGS_((char *memPtr));
 static Packer *		GetPacker _ANSI_ARGS_((Tk_Window tkwin));
 static int		PackAfter _ANSI_ARGS_((Tcl_Interp *interp,
 			    Packer *prevPtr, Packer *masterPtr, int argc,
@@ -298,9 +299,13 @@ Tk_PackCmd(clientData, interp, argc, argv)
 		    "\" isn't packed", (char *) NULL);
 	    return TCL_ERROR;
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, 
+#  ifdef STk_CODE
 			 ":in #.", Tk_PathName(slavePtr->masterPtr->tkwin), " ",
+#  else
+			 ":in \"", Tk_PathName(slavePtr->masterPtr->tkwin), "\" ",
+#  endif
 			 ":anchor \"", Tk_NameOfAnchor(slavePtr->anchor), "\" ",
 			 ":expand ",(slavePtr->flags & EXPAND) ? "#t" : "#f", " ",
 			 ":fill ",
@@ -315,7 +320,7 @@ Tk_PackCmd(clientData, interp, argc, argv)
 		(char *) NULL);
 #endif
 	switch (slavePtr->flags & (FILLX|FILLY)) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    case 0:
 		Tcl_AppendResult(interp, "\"none\"", (char *) NULL);
 		break;
@@ -343,14 +348,14 @@ Tk_PackCmd(clientData, interp, argc, argv)
 		break;
 #endif
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	sprintf(buffer, " :ipadx %d :ipady %d :padx %d :pady %d",
 #else
 	sprintf(buffer, " -ipadx %d -ipady %d -padx %d -pady %d",
 #endif
 		slavePtr->iPadX/2, slavePtr->iPadY/2, slavePtr->padX/2,
 		slavePtr->padY/2);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, buffer, " :side \"",sideNames[slavePtr->side],"\"",
 #else
 	Tcl_AppendResult(interp, buffer, " -side ", sideNames[slavePtr->side],
@@ -372,7 +377,7 @@ Tk_PackCmd(clientData, interp, argc, argv)
 	}
 	masterPtr = GetPacker(master);
 	if (argc == 3) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	  interp->result = (masterPtr->flags & DONT_PROPAGATE) ? "#t" : "#f";
 #else
 	    if (masterPtr->flags & DONT_PROPAGATE) {
@@ -418,14 +423,14 @@ Tk_PackCmd(clientData, interp, argc, argv)
 	    return TCL_ERROR;
 	}
 	masterPtr = GetPacker(master);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, "(", NULL);
 #endif
 	for (slavePtr = masterPtr->slavePtr; slavePtr != NULL;
 		slavePtr = slavePtr->nextPtr) {
 	    Tcl_AppendElement(interp, Tk_PathName(slavePtr->tkwin));
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, ")", NULL);
 #endif
     } else if ((c == 'u') && (strncmp(argv[1], "unpack", length) == 0)) {
