@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkPlace.c 1.25 96/02/15 18:52:32
+ * SCCS: @(#) tkPlace.c 1.27 96/08/20 17:05:31
  */
 
 #include "tkPort.h"
@@ -255,88 +255,88 @@ Tk_PlaceCmd(clientData, interp, argc, argv)
 	    return TCL_OK;
 	}
 	slavePtr = (Slave *) Tcl_GetHashValue(hPtr);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	sprintf(buffer, ":x %d", slavePtr->x);
 #else
 	sprintf(buffer, "-x %d", slavePtr->x);
 #endif
 	Tcl_AppendResult(interp, buffer, (char *) NULL);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	sprintf(buffer, " :relx %.4g", slavePtr->relX);
 #else
 	sprintf(buffer, " -relx %.4g", slavePtr->relX);
 #endif
 	Tcl_AppendResult(interp, buffer, (char *) NULL);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	sprintf(buffer, " :y %d", slavePtr->y);
 #else
 	sprintf(buffer, " -y %d", slavePtr->y);
 #endif
 	Tcl_AppendResult(interp, buffer, (char *) NULL);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	sprintf(buffer, " :rely %.4g", slavePtr->relY);
 #else
 	sprintf(buffer, " -rely %.4g", slavePtr->relY);
 #endif
 	Tcl_AppendResult(interp, buffer, (char *) NULL);
 	if (slavePtr->flags & CHILD_WIDTH) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    sprintf(buffer, " :width %d", slavePtr->width);
 #else
 	    sprintf(buffer, " -width %d", slavePtr->width);
 #endif
 	    Tcl_AppendResult(interp, buffer, (char *) NULL);
 	} else {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, " :width \"\"", (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -width {}", (char *) NULL);
 #endif
 	}
 	if (slavePtr->flags & CHILD_REL_WIDTH) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    sprintf(buffer, " :relwidth %.4g", slavePtr->relWidth);
 #else
 	    sprintf(buffer, " -relwidth %.4g", slavePtr->relWidth);
 #endif
 	    Tcl_AppendResult(interp, buffer, (char *) NULL);
 	} else {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, " :relwidth \"\"", (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -relwidth {}", (char *) NULL);
 #endif
 	}
 	if (slavePtr->flags & CHILD_HEIGHT) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    sprintf(buffer, " :height %d", slavePtr->height);
 #else
 	    sprintf(buffer, " -height %d", slavePtr->height);
 #endif
 	    Tcl_AppendResult(interp, buffer, (char *) NULL);
 	} else {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, " :height \"\"", (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -height {}", (char *) NULL);
 #endif
 	}
 	if (slavePtr->flags & CHILD_REL_HEIGHT) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    sprintf(buffer, " :relheight %.4g", slavePtr->relHeight);
 #else
 	    sprintf(buffer, " -relheight %.4g", slavePtr->relHeight);
 #endif
 	    Tcl_AppendResult(interp, buffer, (char *) NULL);
 	} else {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, " :relheight \"\"", (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -relheight {}", (char *) NULL);
 #endif
 	}
 
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, 
 			 " :anchor \"", Tk_NameOfAnchor(slavePtr->anchor), "\"",
 #else
@@ -344,13 +344,13 @@ Tk_PlaceCmd(clientData, interp, argc, argv)
 #endif
 		(char *) NULL);
 	if (slavePtr->borderMode == BM_OUTSIDE) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, " :bordermode \"outside\"", (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -bordermode outside", (char *) NULL);
 #endif
 	} else if (slavePtr->borderMode == BM_IGNORE) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    Tcl_AppendResult(interp, " :bordermode \"ignore\"", (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -bordermode ignore", (char *) NULL);
@@ -358,8 +358,12 @@ Tk_PlaceCmd(clientData, interp, argc, argv)
 	}
 	if ((slavePtr->masterPtr != NULL)
 		&& (slavePtr->masterPtr->tkwin != Tk_Parent(slavePtr->tkwin))) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
+#  ifdef STk_CODE
 	    Tcl_AppendResult(interp, " :in #.",
+#  else
+	    Tcl_AppendResult(interp, " :in ",
+#  endif
 		    Tk_PathName(slavePtr->masterPtr->tkwin), (char *) NULL);
 #else
 	    Tcl_AppendResult(interp, " -in ",
@@ -373,7 +377,7 @@ Tk_PlaceCmd(clientData, interp, argc, argv)
 	    return TCL_ERROR;
 	}
 	hPtr = Tcl_FindHashEntry(&masterTable, (char *) tkwin);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendElement(interp, "(");
 #endif
 	if (hPtr != NULL) {
@@ -383,10 +387,10 @@ Tk_PlaceCmd(clientData, interp, argc, argv)
 		    slavePtr = slavePtr->nextPtr) {
 		Tcl_AppendElement(interp, Tk_PathName(slavePtr->tkwin));
 	    }
-#ifdef STk_CODE
+	}
+#ifdef SCM_CODE
 	Tcl_AppendElement(interp, ")");
 #endif
-	}
     } else {
 	Tcl_AppendResult(interp, "unknown or ambiguous option \"", argv[1],
 		"\": must be configure, forget, info, or slaves",
@@ -428,9 +432,9 @@ FindSlave(tkwin)
 	slavePtr->masterPtr = NULL;
 	slavePtr->nextPtr = NULL;
 	slavePtr->x = slavePtr->y = 0;
-	slavePtr->relX = slavePtr->relY = 0.0;
+	slavePtr->relX = slavePtr->relY = (float) 0.0;
 	slavePtr->width = slavePtr->height = 0;
-	slavePtr->relWidth = slavePtr->relHeight = 0.0;
+	slavePtr->relWidth = slavePtr->relHeight = (float) 0.0;
 	slavePtr->anchor = TK_ANCHOR_NW;
 	slavePtr->borderMode = BM_INSIDE;
 	slavePtr->flags = 0;
@@ -674,7 +678,7 @@ ConfigureSlave(interp, slavePtr, argc, argv)
 		    result = TCL_ERROR;
 		    goto done;
 		}
-		slavePtr->relHeight = d;
+		slavePtr->relHeight = (float) d;
 		slavePtr->flags |= CHILD_REL_HEIGHT;
 	    }
 	} else if ((c == 'r') && (strncmp(argv[0], "-relwidth", length) == 0)
@@ -686,7 +690,7 @@ ConfigureSlave(interp, slavePtr, argc, argv)
 		    result = TCL_ERROR;
 		    goto done;
 		}
-		slavePtr->relWidth = d;
+		slavePtr->relWidth = (float) d;
 		slavePtr->flags |= CHILD_REL_WIDTH;
 	    }
 	} else if ((c == 'r') && (strncmp(argv[0], "-relx", length) == 0)
@@ -695,14 +699,14 @@ ConfigureSlave(interp, slavePtr, argc, argv)
 		result = TCL_ERROR;
 		goto done;
 	    }
-	    slavePtr->relX = d;
+	    slavePtr->relX = (float) d;
 	} else if ((c == 'r') && (strncmp(argv[0], "-rely", length) == 0)
 		&& (length >= 5)) {
 	    if (Tcl_GetDouble(interp, argv[1], &d) != TCL_OK) {
 		result = TCL_ERROR;
 		goto done;
 	    }
-	    slavePtr->relY = d;
+	    slavePtr->relY = (float) d;
 	} else if ((c == 'w') && (strncmp(argv[0], "-width", length) == 0)) {
 	    if (argv[1][0] == 0) {
 		slavePtr->flags &= ~CHILD_WIDTH;
@@ -814,9 +818,9 @@ RecomputePlacement(clientData)
 	 */
 
 	x1 = slavePtr->x + masterBW + (slavePtr->relX*masterWidth);
-	x = x1 + ((x1 > 0) ? 0.5 : -0.5);
+	x = (int) (x1 + ((x1 > 0) ? 0.5 : -0.5));
 	y1 = slavePtr->y + masterBW + (slavePtr->relY*masterHeight);
-	y = y1 + ((y1 > 0) ? 0.5 : -0.5);
+	y = (int) (y1 + ((y1 > 0) ? 0.5 : -0.5));
 	if (slavePtr->flags & (CHILD_WIDTH|CHILD_REL_WIDTH)) {
 	    width = 0;
 	    if (slavePtr->flags & CHILD_WIDTH) {
@@ -832,7 +836,7 @@ RecomputePlacement(clientData)
 		 */
 
 		x2 = x1 + (slavePtr->relWidth*masterWidth);
-		tmp = x2 + ((x2 > 0) ? 0.5 : -0.5);
+		tmp = (int) (x2 + ((x2 > 0) ? 0.5 : -0.5));
 		width += tmp - x;
 	    }
 	} else {
@@ -850,7 +854,7 @@ RecomputePlacement(clientData)
 		 */
 
 		y2 = y1 + (slavePtr->relHeight*masterHeight);
-		tmp = y2 + ((y2 > 0) ? 0.5 : -0.5);
+		tmp = (int) (y2 + ((y2 > 0) ? 0.5 : -0.5));
 		height += tmp - y;
 	    }
 	} else {
