@@ -13,7 +13,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkText.c 1.91 96/05/16 13:19:58
+ * SCCS: @(#) tkText.c 1.104 97/10/13 15:18:24
  */
 
 #include "default.h"
@@ -25,6 +25,14 @@
 #define DInfo TkDInfo
 #endif
 
+#ifdef BGLK_CODE
+#  define STk_stringify_result	SCM_stringify_result
+#  define STk_stringify		SCM_stringify
+#  define STk_tcl_setvar	SCM_tcl_setvar
+#  define STk_tcl_getvar	SCM_tcl_getvar
+#endif
+
+
 #include "tkText.h"
 
 /*
@@ -50,7 +58,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
 	(char *) NULL, 0, 0},
     {TK_CONFIG_FONT, "-font", "font", "Font",
-	DEF_TEXT_FONT, Tk_Offset(TkText, fontPtr), 0},
+	DEF_TEXT_FONT, Tk_Offset(TkText, tkfont), 0},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
 	DEF_TEXT_FG, Tk_Offset(TkText, fgColor), 0},
     {TK_CONFIG_PIXELS, "-height", "height", "Height",
@@ -116,7 +124,7 @@ static Tk_ConfigSpec configSpecs[] = {
 	DEF_TEXT_STATE, Tk_Offset(TkText, state), 0},
     {TK_CONFIG_STRING, "-tabs", "tabs", "Tabs",
 	DEF_TEXT_TABS, Tk_Offset(TkText, tabOptionString), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_CLOSURE, "-takefocus", "takeFocus", "TakeFocus",
 #else
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
@@ -127,14 +135,14 @@ static Tk_ConfigSpec configSpecs[] = {
 	DEF_TEXT_WIDTH, Tk_Offset(TkText, width), 0},
     {TK_CONFIG_UID, "-wrap", "wrap", "Wrap",
 	DEF_TEXT_WRAP, Tk_Offset(TkText, wrapMode), 0},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_CLOSURE, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
 #else
     {TK_CONFIG_STRING, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
 #endif
 	DEF_TEXT_XSCROLL_COMMAND, Tk_Offset(TkText, xScrollCmd),
 	TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_CLOSURE, "-yscrollcommand", "yScrollCommand", "ScrollCommand",
 #else
     {TK_CONFIG_STRING, "-yscrollcommand", "yScrollCommand", "ScrollCommand",
@@ -184,6 +192,8 @@ static int		TextSearchCmd _ANSI_ARGS_((TkText *textPtr,
 			    Tcl_Interp *interp, int argc, char **argv));
 static int		TextWidgetCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
+static void		TextWorldChanged _ANSI_ARGS_((
+			    ClientData instanceData));
 static int		TextDumpCmd _ANSI_ARGS_((TkText *textPtr,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		DumpLine _ANSI_ARGS_((Tcl_Interp *interp, 
@@ -193,6 +203,17 @@ static int		DumpSegment _ANSI_ARGS_((Tcl_Interp *interp, char *key,
 			    char *value, char * command, int lineno, int offset,
 			    int what));
 
+/*
+ * The structure below defines text class behavior by means of procedures
+ * that can be invoked from generic window code.
+ */
+
+static TkClassProcs textClass = {
+    NULL,			/* createProc. */
+    TextWorldChanged,		/* geometryProc. */
+    NULL			/* modalProc. */
+};
+
 
 /*
  *--------------------------------------------------------------
@@ -263,6 +284,7 @@ Tk_TextCmd(clientData, interp, argc, argv)
     textPtr->numTags = 0;
     Tcl_InitHashTable(&textPtr->markTable, TCL_STRING_KEYS);
     Tcl_InitHashTable(&textPtr->windowTable, TCL_STRING_KEYS);
+    Tcl_InitHashTable(&textPtr->imageTable, TCL_STRING_KEYS);
     textPtr->state = tkTextNormalUid;
     textPtr->border = NULL;
     textPtr->borderWidth = 0;
@@ -274,7 +296,7 @@ Tk_TextCmd(clientData, interp, argc, argv)
     textPtr->highlightColorPtr = NULL;
     textPtr->cursor = None;
     textPtr->fgColor = NULL;
-    textPtr->fontPtr = NULL;
+    textPtr->tkfont = NULL;
     textPtr->charWidth = 1;
     textPtr->spacing1 = 0;
     textPtr->spacing2 = 0;
@@ -321,19 +343,20 @@ Tk_TextCmd(clientData, interp, argc, argv)
 
     textPtr->selTagPtr = TkTextCreateTag(textPtr, "sel");
     textPtr->selTagPtr->reliefString = (char *) ckalloc(7);
-    strcpy(textPtr->selTagPtr->reliefString, "raised");
+    strcpy(textPtr->selTagPtr->reliefString, DEF_TEXT_SELECT_RELIEF);
     textPtr->selTagPtr->relief = TK_RELIEF_RAISED;
     textPtr->currentMarkPtr = TkTextSetMark(textPtr, "current", &startIndex);
     textPtr->insertMarkPtr = TkTextSetMark(textPtr, "insert", &startIndex);
 
-    Tk_SetClass(new, "Text");
+    Tk_SetClass(textPtr->tkwin, "Text");
+    TkSetClassProcs(textPtr->tkwin, &textClass, (ClientData) textPtr);
     Tk_CreateEventHandler(textPtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
 	    TextEventProc, (ClientData) textPtr);
     Tk_CreateEventHandler(textPtr->tkwin, KeyPressMask|KeyReleaseMask
 	    |ButtonPressMask|ButtonReleaseMask|EnterWindowMask
-	    |LeaveWindowMask|PointerMotionMask, TkTextBindProc,
-	    (ClientData) textPtr);
+	    |LeaveWindowMask|PointerMotionMask|VirtualEventMask,
+	    TkTextBindProc, (ClientData) textPtr);
     Tk_CreateSelHandler(textPtr->tkwin, XA_PRIMARY, XA_STRING,
 	    TextFetchSelection, (ClientData) textPtr, XA_STRING);
     if (ConfigureText(interp, textPtr, argc-2, argv+2, 0) != TCL_OK) {
@@ -460,7 +483,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	} else {
 	    goto compareError;
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	interp->result = (value) ? "#t" : "#f";
 #else
 	interp->result = (value) ? "1" : "0";
@@ -486,7 +509,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    goto done;
 	}
 	if (argc == 2) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    interp->result = (tkBTreeDebug) ? "#t" : "#f";
 #else
 	    interp->result = (tkBTreeDebug) ? "1" : "0";
@@ -549,7 +572,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    goto done;
 	}
 	if (TkTextIndexCmp(&index1, &index2) >= 0) {
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	    interp->result = "\"\"";
 #endif
 	    goto done;
@@ -580,7 +603,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    }
 	    TkTextIndexForwChars(&index1, last-offset, &index1);
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	STk_stringify_result(interp, interp->result);
 #endif
     } else if ((c == 'i') && (strncmp(argv[1], "index", length) == 0)
@@ -644,6 +667,8 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	}
     } else if ((c == 'd') && (strncmp(argv[1], "dump", length) == 0)) {
 	result = TextDumpCmd(textPtr, interp, argc, argv);
+    } else if ((c == 'i') && (strncmp(argv[1], "image", length) == 0)) {
+	result = TkTextImageCmd(textPtr, interp, argc, argv);
     } else if ((c == 'm') && (strncmp(argv[1], "mark", length) == 0)) {
 	result = TkTextMarkCmd(textPtr, interp, argc, argv);
     } else if ((c == 's') && (strcmp(argv[1], "scan") == 0) && (length >= 2)) {
@@ -665,7 +690,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
     } else {
 	Tcl_AppendResult(interp, "bad option \"", argv[1],
 		"\": must be bbox, cget, compare, configure, debug, delete, ",
-		"dlineinfo, get, index, insert, mark, scan, search, see, ",
+		"dlineinfo, get, image, index, insert, mark, scan, search, see, ",
 		"tag, window, xview, or yview",
 		(char *) NULL);
 	result = TCL_ERROR;
@@ -733,7 +758,6 @@ DestroyText(memPtr)
     if (textPtr->bindingTable != NULL) {
 	Tk_DeleteBindingTable(textPtr->bindingTable);
     }
-
     /*
      * NOTE: do NOT free up selBorder, selBdString, or selFgColorPtr:
      * they are duplicates of information in the "sel" tag, which was
@@ -778,7 +802,6 @@ ConfigureText(interp, textPtr, argc, argv, flags)
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     int oldExport = textPtr->exportSelection;
-    int charHeight;
 
     if (Tk_ConfigureWidget(interp, textPtr->tkwin, configSpecs,
 	    argc, argv, (char *) textPtr, flags) != TCL_OK) {
@@ -868,7 +891,7 @@ ConfigureText(interp, textPtr, argc, argv, flags)
 	    || (textPtr->selTagPtr->reliefString != NULL)
 	    || (textPtr->selTagPtr->bgStipple != None)
 	    || (textPtr->selTagPtr->fgColor != NULL)
-	    || (textPtr->selTagPtr->fontPtr != None)
+	    || (textPtr->selTagPtr->tkfont != None)
 	    || (textPtr->selTagPtr->fgStipple != None)
 	    || (textPtr->selTagPtr->justifyString != NULL)
 	    || (textPtr->selTagPtr->lMargin1String != NULL)
@@ -919,27 +942,60 @@ ConfigureText(interp, textPtr, argc, argv, flags)
     if (textPtr->height <= 0) {
 	textPtr->height = 1;
     }
-    textPtr->charWidth = XTextWidth(textPtr->fontPtr, "0", 1);
+    TextWorldChanged((ClientData) textPtr);
+    return TCL_OK;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * TextWorldChanged --
+ *
+ *      This procedure is called when the world has changed in some
+ *      way and the widget needs to recompute all its graphics contexts
+ *	and determine its new geometry.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *	Configures all tags in the Text with a empty argc/argv, for
+ *	the side effect of causing all the items to recompute their
+ *	geometry and to be redisplayed.
+ *
+ *---------------------------------------------------------------------------
+ */
+ 
+static void
+TextWorldChanged(instanceData)
+    ClientData instanceData;	/* Information about widget. */
+{
+    TkText *textPtr;
+    Tk_FontMetrics fm;
+
+    textPtr = (TkText *) instanceData;
+
+    textPtr->charWidth = Tk_TextWidth(textPtr->tkfont, "0", 1);
     if (textPtr->charWidth <= 0) {
 	textPtr->charWidth = 1;
     }
-    charHeight = (textPtr->fontPtr->ascent + textPtr->fontPtr->descent);
+    Tk_GetFontMetrics(textPtr->tkfont, &fm);
     Tk_GeometryRequest(textPtr->tkwin,
 	    textPtr->width * textPtr->charWidth + 2*textPtr->borderWidth
 		    + 2*textPtr->padX + 2*textPtr->highlightWidth,
-	    textPtr->height * charHeight + 2*textPtr->borderWidth
+	    textPtr->height * (fm.linespace + textPtr->spacing1
+		    + textPtr->spacing3) + 2*textPtr->borderWidth
 		    + 2*textPtr->padY + 2*textPtr->highlightWidth);
     Tk_SetInternalBorder(textPtr->tkwin,
 	    textPtr->borderWidth + textPtr->highlightWidth);
     if (textPtr->setGrid) {
 	Tk_SetGrid(textPtr->tkwin, textPtr->width, textPtr->height,
-		textPtr->charWidth, charHeight);
+		textPtr->charWidth, fm.linespace);
     } else {
 	Tk_UnsetGrid(textPtr->tkwin);
     }
 
     TkTextRelayoutWindow(textPtr);
-    return TCL_OK;
 }
 
 /*
@@ -986,9 +1042,8 @@ TextEventProc(clientData, eventPtr)
 		Tk_UnsetGrid(textPtr->tkwin);
 	    }
 	    textPtr->tkwin = NULL;
-	    Tcl_DeleteCommand(textPtr->interp,
-		    Tcl_GetCommandName(textPtr->interp,
-		    textPtr->widgetCmd));
+	    Tcl_DeleteCommandFromToken(textPtr->interp,
+		    textPtr->widgetCmd);
 	}
 	Tcl_EventuallyFree((ClientData) textPtr, DestroyText);
     } else if ((eventPtr->type == FocusIn) || (eventPtr->type == FocusOut)) {
@@ -1005,6 +1060,9 @@ TextEventProc(clientData, eventPtr)
 		textPtr->flags &= ~(GOT_FOCUS | INSERT_ON);
 		textPtr->insertBlinkHandler = (Tcl_TimerToken) NULL;
 	    }
+#ifndef ALWAYS_SHOW_SELECTION
+	    TkTextRedrawTag(textPtr, NULL, NULL, textPtr->selTagPtr, 1);
+#endif
 	    TkTextMarkSegToIndex(textPtr, textPtr->insertMarkPtr, &index);
 	    TkTextIndexForwChars(&index, 1, &index2);
 	    TkTextChanged(textPtr, &index, &index2);
@@ -1424,7 +1482,10 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
  * TkTextLostSelection --
  *
  *	This procedure is called back by Tk when the selection is
- *	grabbed away from a text widget.
+ *	grabbed away from a text widget.  On Windows and Mac systems, we
+ *	want to remember the selection for the next time the focus
+ *	enters the window.  On Unix, just remove the "sel" tag from
+ *	everything in the widget.
  *
  * Results:
  *	None.
@@ -1440,6 +1501,7 @@ TkTextLostSelection(clientData)
     ClientData clientData;		/* Information about text widget. */
 {
     register TkText *textPtr = (TkText *) clientData;
+#ifdef ALWAYS_SHOW_SELECTION
     TkTextIndex start, end;
 
     if (!textPtr->exportSelection) {
@@ -1447,13 +1509,16 @@ TkTextLostSelection(clientData)
     }
 
     /*
-     * Just remove the "sel" tag from everything in the widget.
+     * On Windows and Mac systems, we want to remember the selection
+     * for the next time the focus enters the window.  On Unix, 
+     * just remove the "sel" tag from everything in the widget.
      */
 
     TkTextMakeIndex(textPtr->tree, 0, 0, &start);
     TkTextMakeIndex(textPtr->tree, TkBTreeNumLines(textPtr->tree), 0, &end);
     TkTextRedrawTag(textPtr, &start, &end, textPtr->selTagPtr, 1);
     TkBTreeTag(&start, &end, textPtr->selTagPtr, 0);
+#endif
     textPtr->flags &= ~GOT_SELECTION;
 }
 
@@ -1480,7 +1545,8 @@ TextBlinkProc(clientData)
     ClientData clientData;	/* Pointer to record describing text. */
 {
     register TkText *textPtr = (TkText *) clientData;
-    TkTextIndex index, index2;
+    TkTextIndex index;
+    int x, y, w, h;
 
     if (!(textPtr->flags & GOT_FOCUS) || (textPtr->insertOffTime == 0)) {
 	return;
@@ -1495,8 +1561,9 @@ TextBlinkProc(clientData)
 		textPtr->insertOnTime, TextBlinkProc, (ClientData) textPtr);
     }
     TkTextMarkSegToIndex(textPtr, textPtr->insertMarkPtr, &index);
-    TkTextIndexForwChars(&index, 1, &index2);
-    TkTextChanged(textPtr, &index, &index2);
+    TkTextCharBbox(textPtr, &index, &x, &y, &w, &h);
+    TkTextRedrawRegion(textPtr, x - textPtr->insertWidth / 2, y,
+	    textPtr->insertWidth, h);
 }
 
 /*
@@ -1530,6 +1597,9 @@ TextSearchCmd(textPtr, interp, argc, argv)
     int code, matchLength, matchChar, passes, stopLine, searchWholeText;
     int patLength;
     char *arg, *pattern, *varName, *p, *startOfLine;
+#ifdef SCM_CODE
+    char *env;
+#endif
     char buffer[20];
     TkTextIndex index, stopIndex;
     Tcl_DString line, patDString;
@@ -1546,6 +1616,9 @@ TextSearchCmd(textPtr, interp, argc, argv)
     backwards = 0;
     noCase = 0;
     varName = NULL;
+#ifdef SCM_CODE
+    env = "";
+#endif
     for (i = 2; i < argc; i++) {
 	arg = argv[i];
 	if (arg[0] != '-') {
@@ -1555,8 +1628,8 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	if (length < 2) {
 	    badSwitch:
 	    Tcl_AppendResult(interp, "bad switch \"", arg,
-#ifdef STk_CODE
-		    "\": must be -forward, :backward, :exact, :regexp, ",
+#ifdef SCM_CODE
+		    "\": must be :forward, :backward, :exact, :regexp, ",
 		    ":nocase, :count, or ::", (char *) NULL);
 #else
 		    "\": must be -forward, -backward, -exact, -regexp, ",
@@ -1569,11 +1642,30 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	    backwards = 1;
 	} else if ((c == 'c') && (strncmp(argv[i], "-count", length) == 0)) {
 	    if (i >= (argc-1)) {
+#ifdef SCM_CODE
+		interp->result = "no value given for \":count\" option";
+#else
 		interp->result = "no value given for \"-count\" option";
+#endif
 		return TCL_ERROR;
 	    }
 	    i++;
 	    varName = argv[i];
+#ifdef STk_CODE
+	} else if ((c == 'e') && (strncmp(argv[i], "-environment", length) == 0)) {
+	    SCM env_val;
+	  
+	    if (i >= (argc-1)) {
+		interp->result = "no value given for \":environment\" option";
+		return TCL_ERROR;
+	    }
+	    i++;
+	    env = argv[i];
+	    if (!STk_valid_environment(env, &env_val)){
+		interp->result = "Parameter of \":environment\" is not valid";
+		return TCL_ERROR;
+	    }
+#endif
 	} else if ((c == 'e') && (strncmp(argv[i], "-exact", length) == 0)) {
 	    exact = 1;
 	} else if ((c == 'f') && (strncmp(argv[i], "-forwards", length) == 0)) {
@@ -1826,8 +1918,12 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	    }
 	    if (varName != NULL) {
 		sprintf(buffer, "%d", matchLength);
+#ifdef SCM_CODE
+		if (STk_tcl_setvar(varName, buffer, 0, env) == NULL) {
+#else
 		if (Tcl_SetVar(interp, varName, buffer, TCL_LEAVE_ERR_MSG)
 			== NULL) {
+#endif
 		    code = TCL_ERROR;
 		    goto done;
 		}
@@ -1863,7 +1959,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	Tcl_DStringSetLength(&line, 0);
     }
     done:
-#ifdef STk_CODE
+#ifdef SCM_CODE
     if (*interp->result == '\0') {
       /* No match ==> return #f */
       interp->result = "#f";
@@ -1990,8 +2086,8 @@ TkTextGetTabs(interp, tkwin, string)
  * TextDumpCmd --
  *
  *	Return information about the text, tags, marks, and embedded windows
- *	in a text widget.  See the man page for the description of the
- *	text dump operation for all the details.
+ *	and images in a text widget.  See the man page for the description
+ *	of the text dump operation for all the details.
  *
  * Results:
  *	A standard Tcl result.
@@ -2023,7 +2119,9 @@ TextDumpCmd(textPtr, interp, argc, argv)
 #define TK_DUMP_MARK	0x2
 #define TK_DUMP_TAG	0x4
 #define TK_DUMP_WIN	0x8
-#define TK_DUMP_ALL	(TK_DUMP_TEXT|TK_DUMP_MARK|TK_DUMP_TAG|TK_DUMP_WIN)
+#define TK_DUMP_IMG	0x10
+#define TK_DUMP_ALL	(TK_DUMP_TEXT|TK_DUMP_MARK|TK_DUMP_TAG| \
+	TK_DUMP_WIN|TK_DUMP_IMG)
 
     for (arg=2 ; argv[arg] != (char *) NULL ; arg++) {
 	size_t len;
@@ -2039,22 +2137,36 @@ TextDumpCmd(textPtr, interp, argc, argv)
 	    what |= TK_DUMP_TAG;
 	} else if (strncmp("-mark", argv[arg], len) == 0) {
 	    what |= TK_DUMP_MARK;
+	} else if (strncmp("-image", argv[arg], len) == 0) {
+	    what |= TK_DUMP_IMG;
 	} else if (strncmp("-window", argv[arg], len) == 0) {
 	    what |= TK_DUMP_WIN;
 	} else if (strncmp("-command", argv[arg], len) == 0) {
 	    arg++;
 	    if (arg >= argc) {
-		Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?-all -text -mark -tag -window? ?-command script? index ?index2?", NULL);
+#ifdef SCM_CODE
+		Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?:all :image :text :mark :tag :window? ?:command script? index ?index2?", NULL);
+#else
+		Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?-all -image -text -mark -tag -window? ?-command script? index ?index2?", NULL);
+#endif
 		return TCL_ERROR;
 	    }
 	    command = argv[arg];
 	} else {
-	    Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?-all -text -mark -tag -window? ?-command script? index ?index2?", NULL);
+#ifdef SCM_CODE
+	    Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?:all :image :text :mark :tag :window? ?:command script? index ?index2?", NULL);
+#else
+	    Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?-all -image -text -mark -tag -window? ?-command script? index ?index2?", NULL);
+#endif
 	    return TCL_ERROR;
 	}
     }
     if (arg >= argc) {
-	Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?-all -text -mark -tag -window? ?-command script? index ?index2?", NULL);
+#ifdef SCM_CODE
+	Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?:all :image :text :mark :tag :window? ?:command script? index ?index2?", NULL);
+#else
+	Tcl_AppendResult(interp, "Usage: ", argv[0], " dump ?-all -image -text -mark -tag -window? ?-command script? index ?index2?", NULL);
+#endif
 	return TCL_ERROR;
     }
     if (what == 0) {
@@ -2103,7 +2215,6 @@ TextDumpCmd(textPtr, interp, argc, argv)
     if (atEnd) {
 	DumpLine(interp, textPtr, what & ~TK_DUMP_TEXT, index2.linePtr,
 		0, 1, lineno, command);
-
     }
     return TCL_OK;
 }
@@ -2136,12 +2247,12 @@ DumpLine(interp, textPtr, what, linePtr, start, end, lineno, command)
      * character
      * toggleOn, toggleOff
      * mark
+     * image
      * window
      */
     for (offset = 0, segPtr = linePtr->segPtr ;
 	    (offset < end) && (segPtr != (TkTextSegment *)NULL) ;
 	    offset += segPtr->size, segPtr = segPtr->nextPtr) {
-	int result = TCL_OK;
 	if ((what & TK_DUMP_TEXT) && (segPtr->typePtr == &tkTextCharType) &&
 		(offset + segPtr->size > start)) {
 	    char savedChar;			/* Last char used in the seg */
@@ -2155,25 +2266,31 @@ DumpLine(interp, textPtr, what, linePtr, start, end, lineno, command)
 	    }
 	    savedChar = segPtr->body.chars[last];
 	    segPtr->body.chars[last] = '\0';
-	    result = DumpSegment(interp, "text", segPtr->body.chars + first,
+	    DumpSegment(interp, "text", segPtr->body.chars + first,
 		    command, lineno, offset + first, what);
 	    segPtr->body.chars[last] = savedChar;
 	} else if ((offset >= start)) {
 	    if ((what & TK_DUMP_MARK) && (segPtr->typePtr->name[0] == 'm')) {
 		TkTextMark *markPtr = (TkTextMark *)&segPtr->body;
 		char *name = Tcl_GetHashKey(&textPtr->markTable, markPtr->hPtr);
-		result = DumpSegment(interp, "mark", name,
+		DumpSegment(interp, "mark", name,
 			command, lineno, offset, what);
 	    } else if ((what & TK_DUMP_TAG) &&
 			(segPtr->typePtr == &tkTextToggleOnType)) {
-		result = DumpSegment(interp, "tagon",
+		DumpSegment(interp, "tagon",
 			segPtr->body.toggle.tagPtr->name,
 			command, lineno, offset, what);
 	    } else if ((what & TK_DUMP_TAG) && 
 			(segPtr->typePtr == &tkTextToggleOffType)) {
-		result = DumpSegment(interp, "tagoff",
+		DumpSegment(interp, "tagoff",
 			segPtr->body.toggle.tagPtr->name,
 			command, lineno, offset, what);
+	    } else if ((what & TK_DUMP_IMG) && 
+			(segPtr->typePtr->name[0] == 'i')) {
+		TkTextEmbImage *eiPtr = (TkTextEmbImage *)&segPtr->body;
+		char *name = (eiPtr->name ==  NULL) ? "" : eiPtr->name;
+		DumpSegment(interp, "image", name,
+			command, lineno, offset, what);
 	    } else if ((what & TK_DUMP_WIN) && 
 			(segPtr->typePtr->name[0] == 'w')) {
 		TkTextEmbWindow *ewPtr = (TkTextEmbWindow *)&segPtr->body;
@@ -2183,7 +2300,7 @@ DumpLine(interp, textPtr, what, linePtr, start, end, lineno, command)
 		} else {
 		    pathname = Tk_PathName(ewPtr->tkwin);
 		}
-		result = DumpSegment(interp, "window", pathname,
+		DumpSegment(interp, "window", pathname,
 			command, lineno, offset, what);
 	    }
 	}
@@ -2211,6 +2328,28 @@ DumpSegment(interp, key, value, command, lineno, offset, what)
     int offset;			/* Character position */
     int what;			/* Look for TK_DUMP_INDEX bit */
 {
+#ifdef SCM_CODE
+    char buffer[50], *s;
+    sprintf(buffer, "(%d . %d)", lineno, offset);
+    if (command == (char *) NULL) {
+        s = STk_stringify(value, 0);
+        Tcl_AppendResult(interp, "(", key, " ", s, " " , buffer, ")", NULL);
+	free(s);
+	return TCL_OK;
+    } else {
+	char *argv[4];
+	char *list;
+	int result;
+	argv[0] = key;
+	argv[1] = value;
+	argv[2] = buffer;
+	argv[3] = (char *) NULL;
+	list = Tcl_Merge(3, argv);
+	result = Tcl_VarEval(interp, command, " ", list, (char *) NULL);
+	ckfree(list);
+	return result;
+    }
+#else
     char buffer[30];
     sprintf(buffer, "%d.%d", lineno, offset);
     if (command == (char *) NULL) {
@@ -2231,5 +2370,6 @@ DumpSegment(interp, key, value, command, lineno, offset, what)
 	ckfree(list);
 	return result;
     }
+#endif
 }
 
