@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkTextIndex.c 1.13 96/02/15 18:52:57
+ * SCCS: @(#) tkTextIndex.c 1.15 97/06/17 17:49:24
  */
 
 #include "default.h"
@@ -214,7 +214,7 @@ TkTextGetIndex(interp, textPtr, string, indexPtr)
 
     /*
      *---------------------------------------------------------------------
-     * Stage 1: check to see if the index consists of nothing but a mar
+     * Stage 1: check to see if the index consists of nothing but a mark
      * name.  We do this check now even though it's also done later, in
      * order to allow mark names that include funny characters such as
      * spaces or "+1c".
@@ -301,8 +301,8 @@ TkTextGetIndex(interp, textPtr, string, indexPtr)
 	endOfBase = end;
 	goto gotBase; 
     }
-#ifdef STk_CODE
-    if (string[0] == '(' && isdigit(UCHAR(string[1])) || (string[1] == '-')) {
+#ifdef SCM_CODE
+    if ((string[0] == '(' && isdigit(UCHAR(string[1]))) || (string[1] == '-')) {
 	int lineIndex, charIndex;
 
 	/*
@@ -310,10 +310,20 @@ TkTextGetIndex(interp, textPtr, string, indexPtr)
 	 */
 
 	lineIndex = strtol(string+1, &end, 0) - 1;
+#ifdef STk_CODE
 	if ((end == string+1) || (strncmp(end, " . ", 3) != 0)) {
+#endif
+#ifdef BGLK_CODE
+	if ((end == string+1) || (end[ 0 ] != ' ')) {
+#endif
 	    goto error;
 	}
+#ifdef STk_CODE
 	p = end+3; string = 0;
+#endif
+#ifdef BGLK_CODE
+	p = end+1; string = 0;
+#endif
 	if ((*p == 'e')   && (strncmp(p, "end)", 4) == 0)) {
 	    charIndex = LAST_CHAR;
 	    endOfBase = p+4;
@@ -389,7 +399,6 @@ TkTextGetIndex(interp, textPtr, string, indexPtr)
 	/*
 	 * See if the base position is the name of a mark.
 	 */
-
 	c = *endOfBase;
 	*endOfBase = 0;
 	result = TkTextMarkNameToIndex(textPtr, string, indexPtr);
@@ -397,6 +406,18 @@ TkTextGetIndex(interp, textPtr, string, indexPtr)
 	if (result == TCL_OK) {
 	    goto gotBase;
 	}
+
+	/*
+	 * See if the base position is the name of an embedded image
+	 */
+
+	c = *endOfBase;
+	*endOfBase = 0;
+	result = TkTextImageIndex(textPtr, string, indexPtr);
+	*endOfBase = c;
+	if (result != 0) {
+	    goto gotBase;
+	}
     }
     goto error;
 
@@ -461,11 +482,17 @@ TkTextPrintIndex(indexPtr, string)
 				 * at least TK_POS_CHARS characters. */
 {
 #ifdef STk_CODE
-    sprintf(string, "(%d . %d)", TkBTreeLineIndex(indexPtr->linePtr) + 1,
+   sprintf(string, "(%d . %d)", TkBTreeLineIndex(indexPtr->linePtr) + 1,
+	   indexPtr->charIndex);
 #else
-    sprintf(string, "%d.%d", TkBTreeLineIndex(indexPtr->linePtr) + 1,
-#endif
+# ifdef BGLK_CODE
+    sprintf(string, "(%d %d)", TkBTreeLineIndex(indexPtr->linePtr) + 1,
 	    indexPtr->charIndex);
+# else
+    sprintf(string, "%d.%d", TkBTreeLineIndex(indexPtr->linePtr) + 1,
+	    indexPtr->charIndex);
+# endif
+#endif
 }
 
 /*
