@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkTextTag.c 1.36 96/03/07 15:30:43
+ * SCCS: @(#) tkTextTag.c 1.39 97/02/07 13:51:52
  */
 
 #include "default.h"
@@ -28,7 +28,7 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
 	(char *) NULL, Tk_Offset(TkTextTag, border), TK_CONFIG_NULL_OK},
     {TK_CONFIG_BITMAP, "-bgstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, bgStipple), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-borderwidth", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-borderwidth", (char *) NULL, (char *) NULL,
@@ -38,30 +38,30 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_BITMAP, "-fgstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, fgStipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_FONT, "-font", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(TkTextTag, fontPtr), TK_CONFIG_NULL_OK},
+	(char *) NULL, Tk_Offset(TkTextTag, tkfont), TK_CONFIG_NULL_OK},
     {TK_CONFIG_COLOR, "-foreground", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, fgColor), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-justify", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, justifyString), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-lmargin1", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-lmargin1", (char *) NULL, (char *) NULL,
 #endif
 	(char *) NULL, Tk_Offset(TkTextTag, lMargin1String), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-lmargin2", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-lmargin2", (char *) NULL, (char *) NULL,
 #endif
 	(char *) NULL, Tk_Offset(TkTextTag, lMargin2String), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-offset", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-offset", (char *) NULL, (char *) NULL,
 #endif
 	(char *) NULL, Tk_Offset(TkTextTag, offsetString), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SBOOLEAN, "-overstrike", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-overstrike", (char *) NULL, (char *) NULL,
@@ -70,25 +70,25 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
 	TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-relief", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, reliefString), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-rmargin", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-rmargin", (char *) NULL, (char *) NULL,
 #endif
 	(char *) NULL, Tk_Offset(TkTextTag, rMarginString), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-spacing1", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-spacing1", (char *) NULL, (char *) NULL,
 #endif
 	(char *) NULL, Tk_Offset(TkTextTag, spacing1String), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-spacing2", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-spacing2", (char *) NULL, (char *) NULL,
 #endif
 	(char *) NULL, Tk_Offset(TkTextTag, spacing2String), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SINT, "-spacing3", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-spacing3", (char *) NULL, (char *) NULL,
@@ -96,7 +96,7 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
 	(char *) NULL, Tk_Offset(TkTextTag, spacing3String), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-tabs", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, tabString), TK_CONFIG_NULL_OK},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_SBOOLEAN, "-underline", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-underline", (char *) NULL, (char *) NULL,
@@ -247,16 +247,18 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		return Tk_DeleteBinding(interp, textPtr->bindingTable,
 			(ClientData) tagPtr, argv[4]);
 	    }
-#ifdef STk_CODE
-	    mask = Tk_CreateBinding(interp, textPtr->bindingTable,
-		    (ClientData) tagPtr, argv[4], argv[5], argv[0], argv[3]);
-#else
+#ifndef SCM_CODE
 	    if (argv[5][0] == '+') {
 		argv[5]++;
 		append = 1;
 	    }
+#endif
+#ifndef SCM_CODE
 	    mask = Tk_CreateBinding(interp, textPtr->bindingTable,
 		    (ClientData) tagPtr, argv[4], argv[5], append);
+#else
+	    mask = Tk_CreateBinding(interp, textPtr->bindingTable,
+		    (ClientData) tagPtr, argv[4], argv[5], argv[0], argv[3]);
 #endif
 	    if (mask == 0) {
 		return TCL_ERROR;
@@ -265,12 +267,12 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    |Button2MotionMask|Button3MotionMask|Button4MotionMask
 		    |Button5MotionMask|ButtonPressMask|ButtonReleaseMask
 		    |EnterWindowMask|LeaveWindowMask|KeyPressMask
-		    |KeyReleaseMask|PointerMotionMask)) {
+		    |KeyReleaseMask|PointerMotionMask|VirtualEventMask)) {
 		Tk_DeleteBinding(interp, textPtr->bindingTable,
 			(ClientData) tagPtr, argv[4]);
 		Tcl_ResetResult(interp);
 		Tcl_AppendResult(interp, "requested illegal events; ",
-			"only key, button, motion, and enter/leave ",
+			"only key, button, motion, enter, leave, and virtual ",
 			"events may be used", (char *) NULL);
 		return TCL_ERROR;
 	    }
@@ -451,7 +453,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    || (tagPtr->bdString != NULL)
 		    || (tagPtr->reliefString != NULL)
 		    || (tagPtr->bgStipple != None)
-		    || (tagPtr->fgColor != NULL) || (tagPtr->fontPtr != None)
+		    || (tagPtr->fgColor != NULL) || (tagPtr->tkfont != None)
 		    || (tagPtr->fgStipple != None)
 		    || (tagPtr->justifyString != NULL)
 		    || (tagPtr->lMargin1String != NULL)
@@ -574,7 +576,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 	    }
 	}
 	SortTags(arraySize, arrayPtr);
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, "(", NULL);
 	for (i = 0; i < arraySize; i++) {
 	    tagPtr = arrayPtr[i];
@@ -841,7 +843,7 @@ TkTextCreateTag(textPtr, tagName)
     tagPtr->relief = TK_RELIEF_FLAT;
     tagPtr->bgStipple = None;
     tagPtr->fgColor = NULL;
-    tagPtr->fontPtr = NULL;
+    tagPtr->tkfont = NULL;
     tagPtr->fgStipple = None;
     tagPtr->justifyString = NULL;
     tagPtr->justify = TK_JUSTIFY_LEFT;
@@ -949,9 +951,7 @@ TkTextFreeTag(textPtr, tagPtr)
     if (tagPtr->fgColor != None) {
 	Tk_FreeColor(tagPtr->fgColor);
     }
-    if (tagPtr->fontPtr != None) {
-	Tk_FreeFontStruct(tagPtr->fontPtr);
-    }
+    Tk_FreeFont(tagPtr->tkfont);
     if (tagPtr->fgStipple != None) {
 	Tk_FreeBitmap(textPtr->display, tagPtr->fgStipple);
     }
@@ -1198,7 +1198,7 @@ TkTextBindProc(clientData, eventPtr)
 		mask = 0;
 		break;
 	}
-	if ((eventPtr->xbutton.state & AnyButtonMask) == mask) {
+	if ((eventPtr->xbutton.state & AnyButtonMask) == (unsigned) mask) {
 	    textPtr->flags &= ~BUTTON_DOWN;
 	    repick = 1;
 	}
