@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkTextWind.c 1.13 96/02/15 18:53:02
+ * SCCS: @(#) tkTextWind.c 1.14 97/04/25 16:52:09
  */
 
 #include "tk.h"
@@ -113,7 +113,7 @@ static Tk_CustomOption alignOption = {AlignParseProc, AlignPrintProc,
 static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_CUSTOM, "-align", (char *) NULL, (char *) NULL,
 	"center", 0, TK_CONFIG_DONT_SET_DEFAULT, &alignOption},
-#ifdef STk_CODE
+#ifdef SCM_CODE
     {TK_CONFIG_CLOSURE, "-create", (char *) NULL, (char *) NULL,
 #else
     {TK_CONFIG_STRING, "-create", (char *) NULL, (char *) NULL,
@@ -127,7 +127,7 @@ static Tk_ConfigSpec configSpecs[] = {
 	"0", Tk_Offset(TkTextEmbWindow, padY),
 	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_BOOLEAN, "-stretch", (char *) NULL, (char *) NULL,
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	"#f", Tk_Offset(TkTextEmbWindow, stretch),
 #else
 	"0", Tk_Offset(TkTextEmbWindow, stretch),
@@ -167,7 +167,6 @@ TkTextWindowCmd(textPtr, interp, argc, argv)
 				 * parsed this command enough to know that
 				 * argv[1] is "window". */
 {
-    int c;
     size_t length;
     register TkTextSegment *ewPtr;
 
@@ -176,7 +175,6 @@ TkTextWindowCmd(textPtr, interp, argc, argv)
 		argv[0], " window option ?arg arg ...?\"", (char *) NULL);
 	return TCL_ERROR;
     }
-    c = argv[2][0];
     length = strlen(argv[2]);
     if ((strncmp(argv[2], "cget", length) == 0) && (length >= 2)) {
 	TkTextIndex index;
@@ -297,7 +295,7 @@ TkTextWindowCmd(textPtr, interp, argc, argv)
 		    argv[0], " window names\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, "(", (char *) NULL);
 #endif
 	for (hPtr = Tcl_FirstHashEntry(&textPtr->windowTable, &search);
@@ -311,7 +309,7 @@ TkTextWindowCmd(textPtr, interp, argc, argv)
 		    Tcl_GetHashKey(&textPtr->markTable, hPtr));
 #endif
 	}
-#ifdef STk_CODE
+#ifdef SCM_CODE
 	Tcl_AppendResult(interp, ")", (char *) NULL);
 #endif
 
@@ -806,11 +804,20 @@ EmbWinLayoutProc(textPtr, indexPtr, ewPtr, offset, maxX, maxChars,
  	  void * dumb;
  	  /* Convert last result for Tk */
  	  Tcl_DStringAppend(&name, 
-			    STk_convert_for_Tk(STk_last_Tk_as_SCM(), &dumb),
+			    STk_convert_for_Tcl(STk_last_Tk_as_SCM(), &dumb),
  			    -1);
 	}
-#else
+#else 
+#  ifdef BGLK_CODE
+ 	{
+ 	  /* Convert last result for Tk */
+ 	  Tcl_DStringAppend(&name, 
+			    SCM_scm_to_tk_string(SCM_last_Tk_as_scheme()),
+ 			    -1); 
+	}
+#  else
 	Tcl_DStringAppend(&name, textPtr->interp->result, -1);
+#  endif
 #endif
 	Tcl_ResetResult(textPtr->interp);
 	ewPtr->body.ew.tkwin = Tk_NameToWindow(textPtr->interp,
